/* automatically generated by rust-bindgen 0.57.0 */

pub const _FILE_OFFSET_BITS: u32 = 64;
pub const LIBRAW_USE_STREAMS_DATASTREAM_MAXSIZE: u32 = 262144000;
pub const _LIBC_LIMITS_H_: u32 = 1;
pub const _FEATURES_H: u32 = 1;
pub const _DEFAULT_SOURCE: u32 = 1;
pub const __GLIBC_USE_ISOC2X: u32 = 0;
pub const __USE_ISOC11: u32 = 1;
pub const __USE_ISOC99: u32 = 1;
pub const __USE_ISOC95: u32 = 1;
pub const __USE_POSIX_IMPLICITLY: u32 = 1;
pub const _POSIX_SOURCE: u32 = 1;
pub const _POSIX_C_SOURCE: u32 = 200809;
pub const __USE_POSIX: u32 = 1;
pub const __USE_POSIX2: u32 = 1;
pub const __USE_POSIX199309: u32 = 1;
pub const __USE_POSIX199506: u32 = 1;
pub const __USE_XOPEN2K: u32 = 1;
pub const __USE_XOPEN2K8: u32 = 1;
pub const _ATFILE_SOURCE: u32 = 1;
pub const __USE_FILE_OFFSET64: u32 = 1;
pub const __USE_MISC: u32 = 1;
pub const __USE_ATFILE: u32 = 1;
pub const __USE_FORTIFY_LEVEL: u32 = 0;
pub const __GLIBC_USE_DEPRECATED_GETS: u32 = 0;
pub const __GLIBC_USE_DEPRECATED_SCANF: u32 = 0;
pub const _STDC_PREDEF_H: u32 = 1;
pub const __STDC_IEC_559__: u32 = 1;
pub const __STDC_IEC_559_COMPLEX__: u32 = 1;
pub const __STDC_ISO_10646__: u32 = 201706;
pub const __GNU_LIBRARY__: u32 = 6;
pub const __GLIBC__: u32 = 2;
pub const __GLIBC_MINOR__: u32 = 31;
pub const _SYS_CDEFS_H: u32 = 1;
pub const __glibc_c99_flexarr_available: u32 = 1;
pub const __WORDSIZE: u32 = 64;
pub const __WORDSIZE_TIME64_COMPAT32: u32 = 1;
pub const __SYSCALL_WORDSIZE: u32 = 64;
pub const __LONG_DOUBLE_USES_FLOAT128: u32 = 0;
pub const __HAVE_GENERIC_SELECTION: u32 = 1;
pub const __GLIBC_USE_LIB_EXT2: u32 = 0;
pub const __GLIBC_USE_IEC_60559_BFP_EXT: u32 = 0;
pub const __GLIBC_USE_IEC_60559_BFP_EXT_C2X: u32 = 0;
pub const __GLIBC_USE_IEC_60559_FUNCS_EXT: u32 = 0;
pub const __GLIBC_USE_IEC_60559_FUNCS_EXT_C2X: u32 = 0;
pub const __GLIBC_USE_IEC_60559_TYPES_EXT: u32 = 0;
pub const MB_LEN_MAX: u32 = 16;
pub const _BITS_POSIX1_LIM_H: u32 = 1;
pub const _POSIX_AIO_LISTIO_MAX: u32 = 2;
pub const _POSIX_AIO_MAX: u32 = 1;
pub const _POSIX_ARG_MAX: u32 = 4096;
pub const _POSIX_CHILD_MAX: u32 = 25;
pub const _POSIX_DELAYTIMER_MAX: u32 = 32;
pub const _POSIX_HOST_NAME_MAX: u32 = 255;
pub const _POSIX_LINK_MAX: u32 = 8;
pub const _POSIX_LOGIN_NAME_MAX: u32 = 9;
pub const _POSIX_MAX_CANON: u32 = 255;
pub const _POSIX_MAX_INPUT: u32 = 255;
pub const _POSIX_MQ_OPEN_MAX: u32 = 8;
pub const _POSIX_MQ_PRIO_MAX: u32 = 32;
pub const _POSIX_NAME_MAX: u32 = 14;
pub const _POSIX_NGROUPS_MAX: u32 = 8;
pub const _POSIX_OPEN_MAX: u32 = 20;
pub const _POSIX_PATH_MAX: u32 = 256;
pub const _POSIX_PIPE_BUF: u32 = 512;
pub const _POSIX_RE_DUP_MAX: u32 = 255;
pub const _POSIX_RTSIG_MAX: u32 = 8;
pub const _POSIX_SEM_NSEMS_MAX: u32 = 256;
pub const _POSIX_SEM_VALUE_MAX: u32 = 32767;
pub const _POSIX_SIGQUEUE_MAX: u32 = 32;
pub const _POSIX_SSIZE_MAX: u32 = 32767;
pub const _POSIX_STREAM_MAX: u32 = 8;
pub const _POSIX_SYMLINK_MAX: u32 = 255;
pub const _POSIX_SYMLOOP_MAX: u32 = 8;
pub const _POSIX_TIMER_MAX: u32 = 32;
pub const _POSIX_TTY_NAME_MAX: u32 = 9;
pub const _POSIX_TZNAME_MAX: u32 = 6;
pub const _POSIX_CLOCKRES_MIN: u32 = 20000000;
pub const NR_OPEN: u32 = 1024;
pub const NGROUPS_MAX: u32 = 65536;
pub const ARG_MAX: u32 = 131072;
pub const LINK_MAX: u32 = 127;
pub const MAX_CANON: u32 = 255;
pub const MAX_INPUT: u32 = 255;
pub const NAME_MAX: u32 = 255;
pub const PATH_MAX: u32 = 4096;
pub const PIPE_BUF: u32 = 4096;
pub const XATTR_NAME_MAX: u32 = 255;
pub const XATTR_SIZE_MAX: u32 = 65536;
pub const XATTR_LIST_MAX: u32 = 65536;
pub const RTSIG_MAX: u32 = 32;
pub const _POSIX_THREAD_KEYS_MAX: u32 = 128;
pub const PTHREAD_KEYS_MAX: u32 = 1024;
pub const _POSIX_THREAD_DESTRUCTOR_ITERATIONS: u32 = 4;
pub const PTHREAD_DESTRUCTOR_ITERATIONS: u32 = 4;
pub const _POSIX_THREAD_THREADS_MAX: u32 = 64;
pub const AIO_PRIO_DELTA_MAX: u32 = 20;
pub const PTHREAD_STACK_MIN: u32 = 16384;
pub const DELAYTIMER_MAX: u32 = 2147483647;
pub const TTY_NAME_MAX: u32 = 32;
pub const LOGIN_NAME_MAX: u32 = 256;
pub const HOST_NAME_MAX: u32 = 64;
pub const MQ_PRIO_MAX: u32 = 32768;
pub const SEM_VALUE_MAX: u32 = 2147483647;
pub const _BITS_POSIX2_LIM_H: u32 = 1;
pub const _POSIX2_BC_BASE_MAX: u32 = 99;
pub const _POSIX2_BC_DIM_MAX: u32 = 2048;
pub const _POSIX2_BC_SCALE_MAX: u32 = 99;
pub const _POSIX2_BC_STRING_MAX: u32 = 1000;
pub const _POSIX2_COLL_WEIGHTS_MAX: u32 = 2;
pub const _POSIX2_EXPR_NEST_MAX: u32 = 32;
pub const _POSIX2_LINE_MAX: u32 = 2048;
pub const _POSIX2_RE_DUP_MAX: u32 = 255;
pub const _POSIX2_CHARCLASS_NAME_MAX: u32 = 14;
pub const BC_BASE_MAX: u32 = 99;
pub const BC_DIM_MAX: u32 = 2048;
pub const BC_SCALE_MAX: u32 = 99;
pub const BC_STRING_MAX: u32 = 1000;
pub const COLL_WEIGHTS_MAX: u32 = 255;
pub const EXPR_NEST_MAX: u32 = 32;
pub const LINE_MAX: u32 = 2048;
pub const CHARCLASS_NAME_MAX: u32 = 2048;
pub const RE_DUP_MAX: u32 = 32767;
pub const _MEMORY_H: u32 = 1;
pub const _STRING_H: u32 = 1;
pub const _BITS_TYPES_LOCALE_T_H: u32 = 1;
pub const _BITS_TYPES___LOCALE_T_H: u32 = 1;
pub const _STRINGS_H: u32 = 1;
pub const _STDIO_H: u32 = 1;
pub const __GNUC_VA_LIST: u32 = 1;
pub const _BITS_TYPES_H: u32 = 1;
pub const __TIMESIZE: u32 = 64;
pub const _BITS_TYPESIZES_H: u32 = 1;
pub const __OFF_T_MATCHES_OFF64_T: u32 = 1;
pub const __INO_T_MATCHES_INO64_T: u32 = 1;
pub const __RLIM_T_MATCHES_RLIM64_T: u32 = 1;
pub const __STATFS_MATCHES_STATFS64: u32 = 1;
pub const __FD_SETSIZE: u32 = 1024;
pub const _BITS_TIME64_H: u32 = 1;
pub const _____fpos_t_defined: u32 = 1;
pub const ____mbstate_t_defined: u32 = 1;
pub const _____fpos64_t_defined: u32 = 1;
pub const ____FILE_defined: u32 = 1;
pub const __FILE_defined: u32 = 1;
pub const __struct_FILE_defined: u32 = 1;
pub const _IO_EOF_SEEN: u32 = 16;
pub const _IO_ERR_SEEN: u32 = 32;
pub const _IO_USER_LOCK: u32 = 32768;
pub const _IOFBF: u32 = 0;
pub const _IOLBF: u32 = 1;
pub const _IONBF: u32 = 2;
pub const BUFSIZ: u32 = 8192;
pub const EOF: i32 = -1;
pub const SEEK_SET: u32 = 0;
pub const SEEK_CUR: u32 = 1;
pub const SEEK_END: u32 = 2;
pub const P_tmpdir: &'static [u8; 5usize] = b"/tmp\0";
pub const _BITS_STDIO_LIM_H: u32 = 1;
pub const L_tmpnam: u32 = 20;
pub const TMP_MAX: u32 = 238328;
pub const FILENAME_MAX: u32 = 4096;
pub const L_ctermid: u32 = 9;
pub const FOPEN_MAX: u32 = 16;
pub const _STDLIB_H: u32 = 1;
pub const WNOHANG: u32 = 1;
pub const WUNTRACED: u32 = 2;
pub const WSTOPPED: u32 = 2;
pub const WEXITED: u32 = 4;
pub const WCONTINUED: u32 = 8;
pub const WNOWAIT: u32 = 16777216;
pub const __WNOTHREAD: u32 = 536870912;
pub const __WALL: u32 = 1073741824;
pub const __WCLONE: u32 = 2147483648;
pub const __ENUM_IDTYPE_T: u32 = 1;
pub const __W_CONTINUED: u32 = 65535;
pub const __WCOREFLAG: u32 = 128;
pub const __HAVE_FLOAT128: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT128: u32 = 0;
pub const __HAVE_FLOAT64X: u32 = 1;
pub const __HAVE_FLOAT64X_LONG_DOUBLE: u32 = 1;
pub const __HAVE_FLOAT16: u32 = 0;
pub const __HAVE_FLOAT32: u32 = 1;
pub const __HAVE_FLOAT64: u32 = 1;
pub const __HAVE_FLOAT32X: u32 = 1;
pub const __HAVE_FLOAT128X: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT16: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT32: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT64: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT32X: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT64X: u32 = 0;
pub const __HAVE_DISTINCT_FLOAT128X: u32 = 0;
pub const __HAVE_FLOATN_NOT_TYPEDEF: u32 = 0;
pub const __ldiv_t_defined: u32 = 1;
pub const __lldiv_t_defined: u32 = 1;
pub const RAND_MAX: u32 = 2147483647;
pub const EXIT_FAILURE: u32 = 1;
pub const EXIT_SUCCESS: u32 = 0;
pub const _SYS_TYPES_H: u32 = 1;
pub const __clock_t_defined: u32 = 1;
pub const __clockid_t_defined: u32 = 1;
pub const __time_t_defined: u32 = 1;
pub const __timer_t_defined: u32 = 1;
pub const _BITS_STDINT_INTN_H: u32 = 1;
pub const __BIT_TYPES_DEFINED__: u32 = 1;
pub const _ENDIAN_H: u32 = 1;
pub const _BITS_ENDIAN_H: u32 = 1;
pub const __LITTLE_ENDIAN: u32 = 1234;
pub const __BIG_ENDIAN: u32 = 4321;
pub const __PDP_ENDIAN: u32 = 3412;
pub const _BITS_ENDIANNESS_H: u32 = 1;
pub const __BYTE_ORDER: u32 = 1234;
pub const __FLOAT_WORD_ORDER: u32 = 1234;
pub const LITTLE_ENDIAN: u32 = 1234;
pub const BIG_ENDIAN: u32 = 4321;
pub const PDP_ENDIAN: u32 = 3412;
pub const BYTE_ORDER: u32 = 1234;
pub const _BITS_BYTESWAP_H: u32 = 1;
pub const _BITS_UINTN_IDENTITY_H: u32 = 1;
pub const _SYS_SELECT_H: u32 = 1;
pub const __FD_ZERO_STOS: &'static [u8; 6usize] = b"stosq\0";
pub const __sigset_t_defined: u32 = 1;
pub const __timeval_defined: u32 = 1;
pub const _STRUCT_TIMESPEC: u32 = 1;
pub const FD_SETSIZE: u32 = 1024;
pub const _BITS_PTHREADTYPES_COMMON_H: u32 = 1;
pub const _THREAD_SHARED_TYPES_H: u32 = 1;
pub const _BITS_PTHREADTYPES_ARCH_H: u32 = 1;
pub const __SIZEOF_PTHREAD_MUTEX_T: u32 = 40;
pub const __SIZEOF_PTHREAD_ATTR_T: u32 = 56;
pub const __SIZEOF_PTHREAD_RWLOCK_T: u32 = 56;
pub const __SIZEOF_PTHREAD_BARRIER_T: u32 = 32;
pub const __SIZEOF_PTHREAD_MUTEXATTR_T: u32 = 4;
pub const __SIZEOF_PTHREAD_COND_T: u32 = 48;
pub const __SIZEOF_PTHREAD_CONDATTR_T: u32 = 4;
pub const __SIZEOF_PTHREAD_RWLOCKATTR_T: u32 = 8;
pub const __SIZEOF_PTHREAD_BARRIERATTR_T: u32 = 4;
pub const _THREAD_MUTEX_INTERNAL_H: u32 = 1;
pub const __PTHREAD_MUTEX_HAVE_PREV: u32 = 1;
pub const __have_pthread_attr_t: u32 = 1;
pub const _ALLOCA_H: u32 = 1;
pub const _MATH_H: u32 = 1;
pub const _BITS_LIBM_SIMD_DECL_STUBS_H: u32 = 1;
pub const __FP_LOGB0_IS_MIN: u32 = 1;
pub const __FP_LOGBNAN_IS_MIN: u32 = 1;
pub const FP_ILOGB0: i32 = -2147483648;
pub const FP_ILOGBNAN: i32 = -2147483648;
pub const __MATH_DECLARING_DOUBLE: u32 = 1;
pub const __MATH_DECLARING_FLOATN: u32 = 0;
pub const __MATH_DECLARE_LDOUBLE: u32 = 1;
pub const MATH_ERRNO: u32 = 1;
pub const MATH_ERREXCEPT: u32 = 2;
pub const math_errhandling: u32 = 3;
pub const M_E: f64 = 2.718281828459045;
pub const M_LOG2E: f64 = 1.4426950408889634;
pub const M_LOG10E: f64 = 0.4342944819032518;
pub const M_LN2: f64 = 0.6931471805599453;
pub const M_LN10: f64 = 2.302585092994046;
pub const M_PI: f64 = 3.141592653589793;
pub const M_PI_2: f64 = 1.5707963267948966;
pub const M_PI_4: f64 = 0.7853981633974483;
pub const M_1_PI: f64 = 0.3183098861837907;
pub const M_2_PI: f64 = 0.6366197723675814;
pub const M_2_SQRTPI: f64 = 1.1283791670955126;
pub const M_SQRT2: f64 = 1.4142135623730951;
pub const M_SQRT1_2: f64 = 0.7071067811865476;
pub const _ERRNO_H: u32 = 1;
pub const _BITS_ERRNO_H: u32 = 1;
pub const EPERM: u32 = 1;
pub const ENOENT: u32 = 2;
pub const ESRCH: u32 = 3;
pub const EINTR: u32 = 4;
pub const EIO: u32 = 5;
pub const ENXIO: u32 = 6;
pub const E2BIG: u32 = 7;
pub const ENOEXEC: u32 = 8;
pub const EBADF: u32 = 9;
pub const ECHILD: u32 = 10;
pub const EAGAIN: u32 = 11;
pub const ENOMEM: u32 = 12;
pub const EACCES: u32 = 13;
pub const EFAULT: u32 = 14;
pub const ENOTBLK: u32 = 15;
pub const EBUSY: u32 = 16;
pub const EEXIST: u32 = 17;
pub const EXDEV: u32 = 18;
pub const ENODEV: u32 = 19;
pub const ENOTDIR: u32 = 20;
pub const EISDIR: u32 = 21;
pub const EINVAL: u32 = 22;
pub const ENFILE: u32 = 23;
pub const EMFILE: u32 = 24;
pub const ENOTTY: u32 = 25;
pub const ETXTBSY: u32 = 26;
pub const EFBIG: u32 = 27;
pub const ENOSPC: u32 = 28;
pub const ESPIPE: u32 = 29;
pub const EROFS: u32 = 30;
pub const EMLINK: u32 = 31;
pub const EPIPE: u32 = 32;
pub const EDOM: u32 = 33;
pub const ERANGE: u32 = 34;
pub const EDEADLK: u32 = 35;
pub const ENAMETOOLONG: u32 = 36;
pub const ENOLCK: u32 = 37;
pub const ENOSYS: u32 = 38;
pub const ENOTEMPTY: u32 = 39;
pub const ELOOP: u32 = 40;
pub const EWOULDBLOCK: u32 = 11;
pub const ENOMSG: u32 = 42;
pub const EIDRM: u32 = 43;
pub const ECHRNG: u32 = 44;
pub const EL2NSYNC: u32 = 45;
pub const EL3HLT: u32 = 46;
pub const EL3RST: u32 = 47;
pub const ELNRNG: u32 = 48;
pub const EUNATCH: u32 = 49;
pub const ENOCSI: u32 = 50;
pub const EL2HLT: u32 = 51;
pub const EBADE: u32 = 52;
pub const EBADR: u32 = 53;
pub const EXFULL: u32 = 54;
pub const ENOANO: u32 = 55;
pub const EBADRQC: u32 = 56;
pub const EBADSLT: u32 = 57;
pub const EDEADLOCK: u32 = 35;
pub const EBFONT: u32 = 59;
pub const ENOSTR: u32 = 60;
pub const ENODATA: u32 = 61;
pub const ETIME: u32 = 62;
pub const ENOSR: u32 = 63;
pub const ENONET: u32 = 64;
pub const ENOPKG: u32 = 65;
pub const EREMOTE: u32 = 66;
pub const ENOLINK: u32 = 67;
pub const EADV: u32 = 68;
pub const ESRMNT: u32 = 69;
pub const ECOMM: u32 = 70;
pub const EPROTO: u32 = 71;
pub const EMULTIHOP: u32 = 72;
pub const EDOTDOT: u32 = 73;
pub const EBADMSG: u32 = 74;
pub const EOVERFLOW: u32 = 75;
pub const ENOTUNIQ: u32 = 76;
pub const EBADFD: u32 = 77;
pub const EREMCHG: u32 = 78;
pub const ELIBACC: u32 = 79;
pub const ELIBBAD: u32 = 80;
pub const ELIBSCN: u32 = 81;
pub const ELIBMAX: u32 = 82;
pub const ELIBEXEC: u32 = 83;
pub const EILSEQ: u32 = 84;
pub const ERESTART: u32 = 85;
pub const ESTRPIPE: u32 = 86;
pub const EUSERS: u32 = 87;
pub const ENOTSOCK: u32 = 88;
pub const EDESTADDRREQ: u32 = 89;
pub const EMSGSIZE: u32 = 90;
pub const EPROTOTYPE: u32 = 91;
pub const ENOPROTOOPT: u32 = 92;
pub const EPROTONOSUPPORT: u32 = 93;
pub const ESOCKTNOSUPPORT: u32 = 94;
pub const EOPNOTSUPP: u32 = 95;
pub const EPFNOSUPPORT: u32 = 96;
pub const EAFNOSUPPORT: u32 = 97;
pub const EADDRINUSE: u32 = 98;
pub const EADDRNOTAVAIL: u32 = 99;
pub const ENETDOWN: u32 = 100;
pub const ENETUNREACH: u32 = 101;
pub const ENETRESET: u32 = 102;
pub const ECONNABORTED: u32 = 103;
pub const ECONNRESET: u32 = 104;
pub const ENOBUFS: u32 = 105;
pub const EISCONN: u32 = 106;
pub const ENOTCONN: u32 = 107;
pub const ESHUTDOWN: u32 = 108;
pub const ETOOMANYREFS: u32 = 109;
pub const ETIMEDOUT: u32 = 110;
pub const ECONNREFUSED: u32 = 111;
pub const EHOSTDOWN: u32 = 112;
pub const EHOSTUNREACH: u32 = 113;
pub const EALREADY: u32 = 114;
pub const EINPROGRESS: u32 = 115;
pub const ESTALE: u32 = 116;
pub const EUCLEAN: u32 = 117;
pub const ENOTNAM: u32 = 118;
pub const ENAVAIL: u32 = 119;
pub const EISNAM: u32 = 120;
pub const EREMOTEIO: u32 = 121;
pub const EDQUOT: u32 = 122;
pub const ENOMEDIUM: u32 = 123;
pub const EMEDIUMTYPE: u32 = 124;
pub const ECANCELED: u32 = 125;
pub const ENOKEY: u32 = 126;
pub const EKEYEXPIRED: u32 = 127;
pub const EKEYREVOKED: u32 = 128;
pub const EKEYREJECTED: u32 = 129;
pub const EOWNERDEAD: u32 = 130;
pub const ENOTRECOVERABLE: u32 = 131;
pub const ERFKILL: u32 = 132;
pub const EHWPOISON: u32 = 133;
pub const ENOTSUP: u32 = 95;
pub const _SYS_TIME_H: u32 = 1;
pub const _INTTYPES_H: u32 = 1;
pub const _STDINT_H: u32 = 1;
pub const _BITS_WCHAR_H: u32 = 1;
pub const _BITS_STDINT_UINTN_H: u32 = 1;
pub const INT8_MIN: i32 = -128;
pub const INT16_MIN: i32 = -32768;
pub const INT32_MIN: i32 = -2147483648;
pub const INT8_MAX: u32 = 127;
pub const INT16_MAX: u32 = 32767;
pub const INT32_MAX: u32 = 2147483647;
pub const UINT8_MAX: u32 = 255;
pub const UINT16_MAX: u32 = 65535;
pub const UINT32_MAX: u32 = 4294967295;
pub const INT_LEAST8_MIN: i32 = -128;
pub const INT_LEAST16_MIN: i32 = -32768;
pub const INT_LEAST32_MIN: i32 = -2147483648;
pub const INT_LEAST8_MAX: u32 = 127;
pub const INT_LEAST16_MAX: u32 = 32767;
pub const INT_LEAST32_MAX: u32 = 2147483647;
pub const UINT_LEAST8_MAX: u32 = 255;
pub const UINT_LEAST16_MAX: u32 = 65535;
pub const UINT_LEAST32_MAX: u32 = 4294967295;
pub const INT_FAST8_MIN: i32 = -128;
pub const INT_FAST16_MIN: i64 = -9223372036854775808;
pub const INT_FAST32_MIN: i64 = -9223372036854775808;
pub const INT_FAST8_MAX: u32 = 127;
pub const INT_FAST16_MAX: u64 = 9223372036854775807;
pub const INT_FAST32_MAX: u64 = 9223372036854775807;
pub const UINT_FAST8_MAX: u32 = 255;
pub const UINT_FAST16_MAX: i32 = -1;
pub const UINT_FAST32_MAX: i32 = -1;
pub const INTPTR_MIN: i64 = -9223372036854775808;
pub const INTPTR_MAX: u64 = 9223372036854775807;
pub const UINTPTR_MAX: i32 = -1;
pub const PTRDIFF_MIN: i64 = -9223372036854775808;
pub const PTRDIFF_MAX: u64 = 9223372036854775807;
pub const SIG_ATOMIC_MIN: i32 = -2147483648;
pub const SIG_ATOMIC_MAX: u32 = 2147483647;
pub const SIZE_MAX: i32 = -1;
pub const WINT_MIN: u32 = 0;
pub const WINT_MAX: u32 = 4294967295;
pub const ____gwchar_t_defined: u32 = 1;
pub const __PRI64_PREFIX: &'static [u8; 2usize] = b"l\0";
pub const __PRIPTR_PREFIX: &'static [u8; 2usize] = b"l\0";
pub const PRId8: &'static [u8; 2usize] = b"d\0";
pub const PRId16: &'static [u8; 2usize] = b"d\0";
pub const PRId32: &'static [u8; 2usize] = b"d\0";
pub const PRId64: &'static [u8; 3usize] = b"ld\0";
pub const PRIdLEAST8: &'static [u8; 2usize] = b"d\0";
pub const PRIdLEAST16: &'static [u8; 2usize] = b"d\0";
pub const PRIdLEAST32: &'static [u8; 2usize] = b"d\0";
pub const PRIdLEAST64: &'static [u8; 3usize] = b"ld\0";
pub const PRIdFAST8: &'static [u8; 2usize] = b"d\0";
pub const PRIdFAST16: &'static [u8; 3usize] = b"ld\0";
pub const PRIdFAST32: &'static [u8; 3usize] = b"ld\0";
pub const PRIdFAST64: &'static [u8; 3usize] = b"ld\0";
pub const PRIi8: &'static [u8; 2usize] = b"i\0";
pub const PRIi16: &'static [u8; 2usize] = b"i\0";
pub const PRIi32: &'static [u8; 2usize] = b"i\0";
pub const PRIi64: &'static [u8; 3usize] = b"li\0";
pub const PRIiLEAST8: &'static [u8; 2usize] = b"i\0";
pub const PRIiLEAST16: &'static [u8; 2usize] = b"i\0";
pub const PRIiLEAST32: &'static [u8; 2usize] = b"i\0";
pub const PRIiLEAST64: &'static [u8; 3usize] = b"li\0";
pub const PRIiFAST8: &'static [u8; 2usize] = b"i\0";
pub const PRIiFAST16: &'static [u8; 3usize] = b"li\0";
pub const PRIiFAST32: &'static [u8; 3usize] = b"li\0";
pub const PRIiFAST64: &'static [u8; 3usize] = b"li\0";
pub const PRIo8: &'static [u8; 2usize] = b"o\0";
pub const PRIo16: &'static [u8; 2usize] = b"o\0";
pub const PRIo32: &'static [u8; 2usize] = b"o\0";
pub const PRIo64: &'static [u8; 3usize] = b"lo\0";
pub const PRIoLEAST8: &'static [u8; 2usize] = b"o\0";
pub const PRIoLEAST16: &'static [u8; 2usize] = b"o\0";
pub const PRIoLEAST32: &'static [u8; 2usize] = b"o\0";
pub const PRIoLEAST64: &'static [u8; 3usize] = b"lo\0";
pub const PRIoFAST8: &'static [u8; 2usize] = b"o\0";
pub const PRIoFAST16: &'static [u8; 3usize] = b"lo\0";
pub const PRIoFAST32: &'static [u8; 3usize] = b"lo\0";
pub const PRIoFAST64: &'static [u8; 3usize] = b"lo\0";
pub const PRIu8: &'static [u8; 2usize] = b"u\0";
pub const PRIu16: &'static [u8; 2usize] = b"u\0";
pub const PRIu32: &'static [u8; 2usize] = b"u\0";
pub const PRIu64: &'static [u8; 3usize] = b"lu\0";
pub const PRIuLEAST8: &'static [u8; 2usize] = b"u\0";
pub const PRIuLEAST16: &'static [u8; 2usize] = b"u\0";
pub const PRIuLEAST32: &'static [u8; 2usize] = b"u\0";
pub const PRIuLEAST64: &'static [u8; 3usize] = b"lu\0";
pub const PRIuFAST8: &'static [u8; 2usize] = b"u\0";
pub const PRIuFAST16: &'static [u8; 3usize] = b"lu\0";
pub const PRIuFAST32: &'static [u8; 3usize] = b"lu\0";
pub const PRIuFAST64: &'static [u8; 3usize] = b"lu\0";
pub const PRIx8: &'static [u8; 2usize] = b"x\0";
pub const PRIx16: &'static [u8; 2usize] = b"x\0";
pub const PRIx32: &'static [u8; 2usize] = b"x\0";
pub const PRIx64: &'static [u8; 3usize] = b"lx\0";
pub const PRIxLEAST8: &'static [u8; 2usize] = b"x\0";
pub const PRIxLEAST16: &'static [u8; 2usize] = b"x\0";
pub const PRIxLEAST32: &'static [u8; 2usize] = b"x\0";
pub const PRIxLEAST64: &'static [u8; 3usize] = b"lx\0";
pub const PRIxFAST8: &'static [u8; 2usize] = b"x\0";
pub const PRIxFAST16: &'static [u8; 3usize] = b"lx\0";
pub const PRIxFAST32: &'static [u8; 3usize] = b"lx\0";
pub const PRIxFAST64: &'static [u8; 3usize] = b"lx\0";
pub const PRIX8: &'static [u8; 2usize] = b"X\0";
pub const PRIX16: &'static [u8; 2usize] = b"X\0";
pub const PRIX32: &'static [u8; 2usize] = b"X\0";
pub const PRIX64: &'static [u8; 3usize] = b"lX\0";
pub const PRIXLEAST8: &'static [u8; 2usize] = b"X\0";
pub const PRIXLEAST16: &'static [u8; 2usize] = b"X\0";
pub const PRIXLEAST32: &'static [u8; 2usize] = b"X\0";
pub const PRIXLEAST64: &'static [u8; 3usize] = b"lX\0";
pub const PRIXFAST8: &'static [u8; 2usize] = b"X\0";
pub const PRIXFAST16: &'static [u8; 3usize] = b"lX\0";
pub const PRIXFAST32: &'static [u8; 3usize] = b"lX\0";
pub const PRIXFAST64: &'static [u8; 3usize] = b"lX\0";
pub const PRIdMAX: &'static [u8; 3usize] = b"ld\0";
pub const PRIiMAX: &'static [u8; 3usize] = b"li\0";
pub const PRIoMAX: &'static [u8; 3usize] = b"lo\0";
pub const PRIuMAX: &'static [u8; 3usize] = b"lu\0";
pub const PRIxMAX: &'static [u8; 3usize] = b"lx\0";
pub const PRIXMAX: &'static [u8; 3usize] = b"lX\0";
pub const PRIdPTR: &'static [u8; 3usize] = b"ld\0";
pub const PRIiPTR: &'static [u8; 3usize] = b"li\0";
pub const PRIoPTR: &'static [u8; 3usize] = b"lo\0";
pub const PRIuPTR: &'static [u8; 3usize] = b"lu\0";
pub const PRIxPTR: &'static [u8; 3usize] = b"lx\0";
pub const PRIXPTR: &'static [u8; 3usize] = b"lX\0";
pub const SCNd8: &'static [u8; 4usize] = b"hhd\0";
pub const SCNd16: &'static [u8; 3usize] = b"hd\0";
pub const SCNd32: &'static [u8; 2usize] = b"d\0";
pub const SCNd64: &'static [u8; 3usize] = b"ld\0";
pub const SCNdLEAST8: &'static [u8; 4usize] = b"hhd\0";
pub const SCNdLEAST16: &'static [u8; 3usize] = b"hd\0";
pub const SCNdLEAST32: &'static [u8; 2usize] = b"d\0";
pub const SCNdLEAST64: &'static [u8; 3usize] = b"ld\0";
pub const SCNdFAST8: &'static [u8; 4usize] = b"hhd\0";
pub const SCNdFAST16: &'static [u8; 3usize] = b"ld\0";
pub const SCNdFAST32: &'static [u8; 3usize] = b"ld\0";
pub const SCNdFAST64: &'static [u8; 3usize] = b"ld\0";
pub const SCNi8: &'static [u8; 4usize] = b"hhi\0";
pub const SCNi16: &'static [u8; 3usize] = b"hi\0";
pub const SCNi32: &'static [u8; 2usize] = b"i\0";
pub const SCNi64: &'static [u8; 3usize] = b"li\0";
pub const SCNiLEAST8: &'static [u8; 4usize] = b"hhi\0";
pub const SCNiLEAST16: &'static [u8; 3usize] = b"hi\0";
pub const SCNiLEAST32: &'static [u8; 2usize] = b"i\0";
pub const SCNiLEAST64: &'static [u8; 3usize] = b"li\0";
pub const SCNiFAST8: &'static [u8; 4usize] = b"hhi\0";
pub const SCNiFAST16: &'static [u8; 3usize] = b"li\0";
pub const SCNiFAST32: &'static [u8; 3usize] = b"li\0";
pub const SCNiFAST64: &'static [u8; 3usize] = b"li\0";
pub const SCNu8: &'static [u8; 4usize] = b"hhu\0";
pub const SCNu16: &'static [u8; 3usize] = b"hu\0";
pub const SCNu32: &'static [u8; 2usize] = b"u\0";
pub const SCNu64: &'static [u8; 3usize] = b"lu\0";
pub const SCNuLEAST8: &'static [u8; 4usize] = b"hhu\0";
pub const SCNuLEAST16: &'static [u8; 3usize] = b"hu\0";
pub const SCNuLEAST32: &'static [u8; 2usize] = b"u\0";
pub const SCNuLEAST64: &'static [u8; 3usize] = b"lu\0";
pub const SCNuFAST8: &'static [u8; 4usize] = b"hhu\0";
pub const SCNuFAST16: &'static [u8; 3usize] = b"lu\0";
pub const SCNuFAST32: &'static [u8; 3usize] = b"lu\0";
pub const SCNuFAST64: &'static [u8; 3usize] = b"lu\0";
pub const SCNo8: &'static [u8; 4usize] = b"hho\0";
pub const SCNo16: &'static [u8; 3usize] = b"ho\0";
pub const SCNo32: &'static [u8; 2usize] = b"o\0";
pub const SCNo64: &'static [u8; 3usize] = b"lo\0";
pub const SCNoLEAST8: &'static [u8; 4usize] = b"hho\0";
pub const SCNoLEAST16: &'static [u8; 3usize] = b"ho\0";
pub const SCNoLEAST32: &'static [u8; 2usize] = b"o\0";
pub const SCNoLEAST64: &'static [u8; 3usize] = b"lo\0";
pub const SCNoFAST8: &'static [u8; 4usize] = b"hho\0";
pub const SCNoFAST16: &'static [u8; 3usize] = b"lo\0";
pub const SCNoFAST32: &'static [u8; 3usize] = b"lo\0";
pub const SCNoFAST64: &'static [u8; 3usize] = b"lo\0";
pub const SCNx8: &'static [u8; 4usize] = b"hhx\0";
pub const SCNx16: &'static [u8; 3usize] = b"hx\0";
pub const SCNx32: &'static [u8; 2usize] = b"x\0";
pub const SCNx64: &'static [u8; 3usize] = b"lx\0";
pub const SCNxLEAST8: &'static [u8; 4usize] = b"hhx\0";
pub const SCNxLEAST16: &'static [u8; 3usize] = b"hx\0";
pub const SCNxLEAST32: &'static [u8; 2usize] = b"x\0";
pub const SCNxLEAST64: &'static [u8; 3usize] = b"lx\0";
pub const SCNxFAST8: &'static [u8; 4usize] = b"hhx\0";
pub const SCNxFAST16: &'static [u8; 3usize] = b"lx\0";
pub const SCNxFAST32: &'static [u8; 3usize] = b"lx\0";
pub const SCNxFAST64: &'static [u8; 3usize] = b"lx\0";
pub const SCNdMAX: &'static [u8; 3usize] = b"ld\0";
pub const SCNiMAX: &'static [u8; 3usize] = b"li\0";
pub const SCNoMAX: &'static [u8; 3usize] = b"lo\0";
pub const SCNuMAX: &'static [u8; 3usize] = b"lu\0";
pub const SCNxMAX: &'static [u8; 3usize] = b"lx\0";
pub const SCNdPTR: &'static [u8; 3usize] = b"ld\0";
pub const SCNiPTR: &'static [u8; 3usize] = b"li\0";
pub const SCNoPTR: &'static [u8; 3usize] = b"lo\0";
pub const SCNuPTR: &'static [u8; 3usize] = b"lu\0";
pub const SCNxPTR: &'static [u8; 3usize] = b"lx\0";
pub const LIBRAW_DEFAULT_ADJUST_MAXIMUM_THRESHOLD: f64 = 0.75;
pub const LIBRAW_DEFAULT_AUTO_BRIGHTNESS_THRESHOLD: f64 = 0.01;
pub const LIBRAW_MAX_ALLOC_MB_DEFAULT: u32 = 2048;
pub const LIBRAW_MAX_THUMBNAIL_MB: u32 = 512;
pub const LIBRAW_MAX_METADATA_BLOCKS: u32 = 1024;
pub const LIBRAW_CBLACK_SIZE: u32 = 4104;
pub const LIBRAW_IFD_MAXCOUNT: u32 = 10;
pub const LIBRAW_CRXTRACKS_MAXCOUNT: u32 = 16;
pub const LIBRAW_AHD_TILE: u32 = 512;
pub const LIBRAW_LENS_NOT_SET: i32 = -1;
pub const LIBRAW_XTRANS: u32 = 9;
pub const LIBRAW_PROGRESS_THUMB_MASK: u32 = 268435455;
pub const LIBRAW_MAJOR_VERSION: u32 = 0;
pub const LIBRAW_MINOR_VERSION: u32 = 20;
pub const LIBRAW_PATCH_VERSION: u32 = 1;
pub const LIBRAW_SHLIB_CURRENT: u32 = 20;
pub const LIBRAW_SHLIB_REVISION: u32 = 0;
pub const LIBRAW_SHLIB_AGE: u32 = 0;
pub const LibRawBigEndian: u32 = 0;
pub const LIBRAW_HISTOGRAM_SIZE: u32 = 8192;
extern "C" {
    pub fn memcpy(
        __dest: *mut libc::c_void,
        __src: *const libc::c_void,
        __n: libc::c_ulong,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn memmove(
        __dest: *mut libc::c_void,
        __src: *const libc::c_void,
        __n: libc::c_ulong,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn memccpy(
        __dest: *mut libc::c_void,
        __src: *const libc::c_void,
        __c: libc::c_int,
        __n: libc::c_ulong,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn memset(
        __s: *mut libc::c_void,
        __c: libc::c_int,
        __n: libc::c_ulong,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn memcmp(
        __s1: *const libc::c_void,
        __s2: *const libc::c_void,
        __n: libc::c_ulong,
    ) -> libc::c_int;
}
extern "C" {
    pub fn memchr(
        __s: *const libc::c_void,
        __c: libc::c_int,
        __n: libc::c_ulong,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn strcpy(__dest: *mut libc::c_char, __src: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn strncpy(
        __dest: *mut libc::c_char,
        __src: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn strcat(__dest: *mut libc::c_char, __src: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn strncat(
        __dest: *mut libc::c_char,
        __src: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn strcmp(__s1: *const libc::c_char, __s2: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn strncmp(
        __s1: *const libc::c_char,
        __s2: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> libc::c_int;
}
extern "C" {
    pub fn strcoll(__s1: *const libc::c_char, __s2: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn strxfrm(
        __dest: *mut libc::c_char,
        __src: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> libc::c_ulong;
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __locale_struct {
    pub __locales: [*mut __locale_data; 13usize],
    pub __ctype_b: *const libc::c_ushort,
    pub __ctype_tolower: *const libc::c_int,
    pub __ctype_toupper: *const libc::c_int,
    pub __names: [*const libc::c_char; 13usize],
}
#[test]
fn bindgen_test_layout___locale_struct() {
    assert_eq!(
        ::core::mem::size_of::<__locale_struct>(),
        232usize,
        concat!("Size of: ", stringify!(__locale_struct))
    );
    assert_eq!(
        ::core::mem::align_of::<__locale_struct>(),
        8usize,
        concat!("Alignment of ", stringify!(__locale_struct))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__locale_struct>())).__locales as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__locale_struct),
            "::",
            stringify!(__locales)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__locale_struct>())).__ctype_b as *const _ as usize },
        104usize,
        concat!(
            "Offset of field: ",
            stringify!(__locale_struct),
            "::",
            stringify!(__ctype_b)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__locale_struct>())).__ctype_tolower as *const _ as usize
        },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(__locale_struct),
            "::",
            stringify!(__ctype_tolower)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__locale_struct>())).__ctype_toupper as *const _ as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(__locale_struct),
            "::",
            stringify!(__ctype_toupper)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__locale_struct>())).__names as *const _ as usize },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(__locale_struct),
            "::",
            stringify!(__names)
        )
    );
}
pub type __locale_t = *mut __locale_struct;
pub type locale_t = __locale_t;
extern "C" {
    pub fn strcoll_l(
        __s1: *const libc::c_char,
        __s2: *const libc::c_char,
        __l: locale_t,
    ) -> libc::c_int;
}
extern "C" {
    pub fn strxfrm_l(
        __dest: *mut libc::c_char,
        __src: *const libc::c_char,
        __n: usize,
        __l: locale_t,
    ) -> usize;
}
extern "C" {
    pub fn strdup(__s: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn strndup(__string: *const libc::c_char, __n: libc::c_ulong) -> *mut libc::c_char;
}
extern "C" {
    pub fn strchr(__s: *const libc::c_char, __c: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    pub fn strrchr(__s: *const libc::c_char, __c: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    pub fn strcspn(__s: *const libc::c_char, __reject: *const libc::c_char) -> libc::c_ulong;
}
extern "C" {
    pub fn strspn(__s: *const libc::c_char, __accept: *const libc::c_char) -> libc::c_ulong;
}
extern "C" {
    pub fn strpbrk(__s: *const libc::c_char, __accept: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn strstr(
        __haystack: *const libc::c_char,
        __needle: *const libc::c_char,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn strtok(__s: *mut libc::c_char, __delim: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn strtok_r(
        __s: *mut libc::c_char,
        __delim: *const libc::c_char,
        __save_ptr: *mut *mut libc::c_char,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn strlen(__s: *const libc::c_char) -> libc::c_ulong;
}
extern "C" {
    pub fn strnlen(__string: *const libc::c_char, __maxlen: usize) -> usize;
}
extern "C" {
    pub fn strerror(__errnum: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    #[link_name = "\u{1}__xpg_strerror_r"]
    pub fn strerror_r(
        __errnum: libc::c_int,
        __buf: *mut libc::c_char,
        __buflen: usize,
    ) -> libc::c_int;
}
extern "C" {
    pub fn strerror_l(__errnum: libc::c_int, __l: locale_t) -> *mut libc::c_char;
}
extern "C" {
    pub fn bcmp(
        __s1: *const libc::c_void,
        __s2: *const libc::c_void,
        __n: libc::c_ulong,
    ) -> libc::c_int;
}
extern "C" {
    pub fn bcopy(__src: *const libc::c_void, __dest: *mut libc::c_void, __n: usize);
}
extern "C" {
    pub fn bzero(__s: *mut libc::c_void, __n: libc::c_ulong);
}
extern "C" {
    pub fn index(__s: *const libc::c_char, __c: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    pub fn rindex(__s: *const libc::c_char, __c: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    pub fn ffs(__i: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn ffsl(__l: libc::c_long) -> libc::c_int;
}
extern "C" {
    pub fn ffsll(__ll: libc::c_longlong) -> libc::c_int;
}
extern "C" {
    pub fn strcasecmp(__s1: *const libc::c_char, __s2: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn strncasecmp(
        __s1: *const libc::c_char,
        __s2: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> libc::c_int;
}
extern "C" {
    pub fn strcasecmp_l(
        __s1: *const libc::c_char,
        __s2: *const libc::c_char,
        __loc: locale_t,
    ) -> libc::c_int;
}
extern "C" {
    pub fn strncasecmp_l(
        __s1: *const libc::c_char,
        __s2: *const libc::c_char,
        __n: usize,
        __loc: locale_t,
    ) -> libc::c_int;
}
extern "C" {
    pub fn explicit_bzero(__s: *mut libc::c_void, __n: usize);
}
extern "C" {
    pub fn strsep(
        __stringp: *mut *mut libc::c_char,
        __delim: *const libc::c_char,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn strsignal(__sig: libc::c_int) -> *mut libc::c_char;
}
extern "C" {
    pub fn stpcpy(__dest: *mut libc::c_char, __src: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn stpncpy(
        __dest: *mut libc::c_char,
        __src: *const libc::c_char,
        __n: libc::c_ulong,
    ) -> *mut libc::c_char;
}
pub type va_list = __builtin_va_list;
pub type __gnuc_va_list = __builtin_va_list;
pub type __u_char = libc::c_uchar;
pub type __u_short = libc::c_ushort;
pub type __u_int = libc::c_uint;
pub type __u_long = libc::c_ulong;
pub type __int8_t = libc::c_schar;
pub type __uint8_t = libc::c_uchar;
pub type __int16_t = libc::c_short;
pub type __uint16_t = libc::c_ushort;
pub type __int32_t = libc::c_int;
pub type __uint32_t = libc::c_uint;
pub type __int64_t = libc::c_long;
pub type __uint64_t = libc::c_ulong;
pub type __int_least8_t = __int8_t;
pub type __uint_least8_t = __uint8_t;
pub type __int_least16_t = __int16_t;
pub type __uint_least16_t = __uint16_t;
pub type __int_least32_t = __int32_t;
pub type __uint_least32_t = __uint32_t;
pub type __int_least64_t = __int64_t;
pub type __uint_least64_t = __uint64_t;
pub type __quad_t = libc::c_long;
pub type __u_quad_t = libc::c_ulong;
pub type __intmax_t = libc::c_long;
pub type __uintmax_t = libc::c_ulong;
pub type __dev_t = libc::c_ulong;
pub type __uid_t = libc::c_uint;
pub type __gid_t = libc::c_uint;
pub type __ino_t = libc::c_ulong;
pub type __ino64_t = libc::c_ulong;
pub type __mode_t = libc::c_uint;
pub type __nlink_t = libc::c_ulong;
pub type __off_t = libc::c_long;
pub type __off64_t = libc::c_long;
pub type __pid_t = libc::c_int;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __fsid_t {
    pub __val: [libc::c_int; 2usize],
}
#[test]
fn bindgen_test_layout___fsid_t() {
    assert_eq!(
        ::core::mem::size_of::<__fsid_t>(),
        8usize,
        concat!("Size of: ", stringify!(__fsid_t))
    );
    assert_eq!(
        ::core::mem::align_of::<__fsid_t>(),
        4usize,
        concat!("Alignment of ", stringify!(__fsid_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__fsid_t>())).__val as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__fsid_t),
            "::",
            stringify!(__val)
        )
    );
}
pub type __clock_t = libc::c_long;
pub type __rlim_t = libc::c_ulong;
pub type __rlim64_t = libc::c_ulong;
pub type __id_t = libc::c_uint;
pub type __time_t = libc::c_long;
pub type __useconds_t = libc::c_uint;
pub type __suseconds_t = libc::c_long;
pub type __daddr_t = libc::c_int;
pub type __key_t = libc::c_int;
pub type __clockid_t = libc::c_int;
pub type __timer_t = *mut libc::c_void;
pub type __blksize_t = libc::c_long;
pub type __blkcnt_t = libc::c_long;
pub type __blkcnt64_t = libc::c_long;
pub type __fsblkcnt_t = libc::c_ulong;
pub type __fsblkcnt64_t = libc::c_ulong;
pub type __fsfilcnt_t = libc::c_ulong;
pub type __fsfilcnt64_t = libc::c_ulong;
pub type __fsword_t = libc::c_long;
pub type __ssize_t = libc::c_long;
pub type __syscall_slong_t = libc::c_long;
pub type __syscall_ulong_t = libc::c_ulong;
pub type __loff_t = __off64_t;
pub type __caddr_t = *mut libc::c_char;
pub type __intptr_t = libc::c_long;
pub type __socklen_t = libc::c_uint;
pub type __sig_atomic_t = libc::c_int;
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __mbstate_t {
    pub __count: libc::c_int,
    pub __value: __mbstate_t__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union __mbstate_t__bindgen_ty_1 {
    pub __wch: libc::c_uint,
    pub __wchb: [libc::c_char; 4usize],
    _bindgen_union_align: u32,
}
#[test]
fn bindgen_test_layout___mbstate_t__bindgen_ty_1() {
    assert_eq!(
        ::core::mem::size_of::<__mbstate_t__bindgen_ty_1>(),
        4usize,
        concat!("Size of: ", stringify!(__mbstate_t__bindgen_ty_1))
    );
    assert_eq!(
        ::core::mem::align_of::<__mbstate_t__bindgen_ty_1>(),
        4usize,
        concat!("Alignment of ", stringify!(__mbstate_t__bindgen_ty_1))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__mbstate_t__bindgen_ty_1>())).__wch as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__mbstate_t__bindgen_ty_1),
            "::",
            stringify!(__wch)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__mbstate_t__bindgen_ty_1>())).__wchb as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__mbstate_t__bindgen_ty_1),
            "::",
            stringify!(__wchb)
        )
    );
}
#[test]
fn bindgen_test_layout___mbstate_t() {
    assert_eq!(
        ::core::mem::size_of::<__mbstate_t>(),
        8usize,
        concat!("Size of: ", stringify!(__mbstate_t))
    );
    assert_eq!(
        ::core::mem::align_of::<__mbstate_t>(),
        4usize,
        concat!("Alignment of ", stringify!(__mbstate_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__mbstate_t>())).__count as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__mbstate_t),
            "::",
            stringify!(__count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__mbstate_t>())).__value as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__mbstate_t),
            "::",
            stringify!(__value)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct _G_fpos_t {
    pub __pos: __off_t,
    pub __state: __mbstate_t,
}
#[test]
fn bindgen_test_layout__G_fpos_t() {
    assert_eq!(
        ::core::mem::size_of::<_G_fpos_t>(),
        16usize,
        concat!("Size of: ", stringify!(_G_fpos_t))
    );
    assert_eq!(
        ::core::mem::align_of::<_G_fpos_t>(),
        8usize,
        concat!("Alignment of ", stringify!(_G_fpos_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_G_fpos_t>())).__pos as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(_G_fpos_t),
            "::",
            stringify!(__pos)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_G_fpos_t>())).__state as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(_G_fpos_t),
            "::",
            stringify!(__state)
        )
    );
}
pub type __fpos_t = _G_fpos_t;
#[repr(C)]
#[derive(Copy, Clone)]
pub struct _G_fpos64_t {
    pub __pos: __off64_t,
    pub __state: __mbstate_t,
}
#[test]
fn bindgen_test_layout__G_fpos64_t() {
    assert_eq!(
        ::core::mem::size_of::<_G_fpos64_t>(),
        16usize,
        concat!("Size of: ", stringify!(_G_fpos64_t))
    );
    assert_eq!(
        ::core::mem::align_of::<_G_fpos64_t>(),
        8usize,
        concat!("Alignment of ", stringify!(_G_fpos64_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_G_fpos64_t>())).__pos as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(_G_fpos64_t),
            "::",
            stringify!(__pos)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_G_fpos64_t>())).__state as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(_G_fpos64_t),
            "::",
            stringify!(__state)
        )
    );
}
pub type __fpos64_t = _G_fpos64_t;
pub type __FILE = _IO_FILE;
pub type FILE = _IO_FILE;
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _IO_marker {
    _unused: [u8; 0],
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _IO_codecvt {
    _unused: [u8; 0],
}
#[repr(C)]
#[derive(Debug, Copy, Clone)]
pub struct _IO_wide_data {
    _unused: [u8; 0],
}
pub type _IO_lock_t = libc::c_void;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct _IO_FILE {
    pub _flags: libc::c_int,
    pub _IO_read_ptr: *mut libc::c_char,
    pub _IO_read_end: *mut libc::c_char,
    pub _IO_read_base: *mut libc::c_char,
    pub _IO_write_base: *mut libc::c_char,
    pub _IO_write_ptr: *mut libc::c_char,
    pub _IO_write_end: *mut libc::c_char,
    pub _IO_buf_base: *mut libc::c_char,
    pub _IO_buf_end: *mut libc::c_char,
    pub _IO_save_base: *mut libc::c_char,
    pub _IO_backup_base: *mut libc::c_char,
    pub _IO_save_end: *mut libc::c_char,
    pub _markers: *mut _IO_marker,
    pub _chain: *mut _IO_FILE,
    pub _fileno: libc::c_int,
    pub _flags2: libc::c_int,
    pub _old_offset: __off_t,
    pub _cur_column: libc::c_ushort,
    pub _vtable_offset: libc::c_schar,
    pub _shortbuf: [libc::c_char; 1usize],
    pub _lock: *mut _IO_lock_t,
    pub _offset: __off64_t,
    pub _codecvt: *mut _IO_codecvt,
    pub _wide_data: *mut _IO_wide_data,
    pub _freeres_list: *mut _IO_FILE,
    pub _freeres_buf: *mut libc::c_void,
    pub __pad5: usize,
    pub _mode: libc::c_int,
    pub _unused2: [libc::c_char; 20usize],
}
#[test]
fn bindgen_test_layout__IO_FILE() {
    assert_eq!(
        ::core::mem::size_of::<_IO_FILE>(),
        216usize,
        concat!("Size of: ", stringify!(_IO_FILE))
    );
    assert_eq!(
        ::core::mem::align_of::<_IO_FILE>(),
        8usize,
        concat!("Alignment of ", stringify!(_IO_FILE))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._flags as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_flags)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_read_ptr as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_read_ptr)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_read_end as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_read_end)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_read_base as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_read_base)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_write_base as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_write_base)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_write_ptr as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_write_ptr)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_write_end as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_write_end)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_buf_base as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_buf_base)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_buf_end as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_buf_end)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_save_base as *const _ as usize },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_save_base)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_backup_base as *const _ as usize },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_backup_base)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._IO_save_end as *const _ as usize },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_IO_save_end)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._markers as *const _ as usize },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_markers)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._chain as *const _ as usize },
        104usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_chain)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._fileno as *const _ as usize },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_fileno)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._flags2 as *const _ as usize },
        116usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_flags2)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._old_offset as *const _ as usize },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_old_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._cur_column as *const _ as usize },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_cur_column)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._vtable_offset as *const _ as usize },
        130usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_vtable_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._shortbuf as *const _ as usize },
        131usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_shortbuf)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._lock as *const _ as usize },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_lock)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._offset as *const _ as usize },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._codecvt as *const _ as usize },
        152usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_codecvt)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._wide_data as *const _ as usize },
        160usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_wide_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._freeres_list as *const _ as usize },
        168usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_freeres_list)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._freeres_buf as *const _ as usize },
        176usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_freeres_buf)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>())).__pad5 as *const _ as usize },
        184usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(__pad5)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._mode as *const _ as usize },
        192usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_mode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<_IO_FILE>()))._unused2 as *const _ as usize },
        196usize,
        concat!(
            "Offset of field: ",
            stringify!(_IO_FILE),
            "::",
            stringify!(_unused2)
        )
    );
}
pub type off_t = __off64_t;
pub type fpos_t = __fpos64_t;
extern "C" {
    pub static mut stdin: *mut FILE;
}
extern "C" {
    pub static mut stdout: *mut FILE;
}
extern "C" {
    pub static mut stderr: *mut FILE;
}
extern "C" {
    pub fn remove(__filename: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn rename(__old: *const libc::c_char, __new: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn renameat(
        __oldfd: libc::c_int,
        __old: *const libc::c_char,
        __newfd: libc::c_int,
        __new: *const libc::c_char,
    ) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}tmpfile64"]
    pub fn tmpfile() -> *mut FILE;
}
extern "C" {
    pub fn tmpnam(__s: *mut libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn tmpnam_r(__s: *mut libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn tempnam(__dir: *const libc::c_char, __pfx: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn fclose(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fflush(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fflush_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}fopen64"]
    pub fn fopen(__filename: *const libc::c_char, __modes: *const libc::c_char) -> *mut FILE;
}
extern "C" {
    #[link_name = "\u{1}freopen64"]
    pub fn freopen(
        __filename: *const libc::c_char,
        __modes: *const libc::c_char,
        __stream: *mut FILE,
    ) -> *mut FILE;
}
extern "C" {
    pub fn fdopen(__fd: libc::c_int, __modes: *const libc::c_char) -> *mut FILE;
}
extern "C" {
    pub fn fmemopen(
        __s: *mut libc::c_void,
        __len: usize,
        __modes: *const libc::c_char,
    ) -> *mut FILE;
}
extern "C" {
    pub fn open_memstream(__bufloc: *mut *mut libc::c_char, __sizeloc: *mut usize) -> *mut FILE;
}
extern "C" {
    pub fn setbuf(__stream: *mut FILE, __buf: *mut libc::c_char);
}
extern "C" {
    pub fn setvbuf(
        __stream: *mut FILE,
        __buf: *mut libc::c_char,
        __modes: libc::c_int,
        __n: usize,
    ) -> libc::c_int;
}
extern "C" {
    pub fn setbuffer(__stream: *mut FILE, __buf: *mut libc::c_char, __size: usize);
}
extern "C" {
    pub fn setlinebuf(__stream: *mut FILE);
}
extern "C" {
    pub fn fprintf(__stream: *mut FILE, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn printf(__format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn sprintf(__s: *mut libc::c_char, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn vfprintf(
        __s: *mut FILE,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn vprintf(__format: *const libc::c_char, __arg: *mut __va_list_tag) -> libc::c_int;
}
extern "C" {
    pub fn vsprintf(
        __s: *mut libc::c_char,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn snprintf(
        __s: *mut libc::c_char,
        __maxlen: libc::c_ulong,
        __format: *const libc::c_char,
        ...
    ) -> libc::c_int;
}
extern "C" {
    pub fn vsnprintf(
        __s: *mut libc::c_char,
        __maxlen: libc::c_ulong,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn vdprintf(
        __fd: libc::c_int,
        __fmt: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn dprintf(__fd: libc::c_int, __fmt: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn fscanf(__stream: *mut FILE, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn scanf(__format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn sscanf(__s: *const libc::c_char, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_fscanf"]
    pub fn fscanf1(__stream: *mut FILE, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_scanf"]
    pub fn scanf1(__format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_sscanf"]
    pub fn sscanf1(__s: *const libc::c_char, __format: *const libc::c_char, ...) -> libc::c_int;
}
extern "C" {
    pub fn vfscanf(
        __s: *mut FILE,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn vscanf(__format: *const libc::c_char, __arg: *mut __va_list_tag) -> libc::c_int;
}
extern "C" {
    pub fn vsscanf(
        __s: *const libc::c_char,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_vfscanf"]
    pub fn vfscanf1(
        __s: *mut FILE,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_vscanf"]
    pub fn vscanf1(__format: *const libc::c_char, __arg: *mut __va_list_tag) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}__isoc99_vsscanf"]
    pub fn vsscanf1(
        __s: *const libc::c_char,
        __format: *const libc::c_char,
        __arg: *mut __va_list_tag,
    ) -> libc::c_int;
}
extern "C" {
    pub fn fgetc(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn getc(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn getchar() -> libc::c_int;
}
extern "C" {
    pub fn getc_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn getchar_unlocked() -> libc::c_int;
}
extern "C" {
    pub fn fgetc_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fputc(__c: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn putc(__c: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn putchar(__c: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn fputc_unlocked(__c: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn putc_unlocked(__c: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn putchar_unlocked(__c: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn getw(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn putw(__w: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fgets(
        __s: *mut libc::c_char,
        __n: libc::c_int,
        __stream: *mut FILE,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn getdelim(
        __lineptr: *mut *mut libc::c_char,
        __n: *mut usize,
        __delimiter: libc::c_int,
        __stream: *mut FILE,
    ) -> __ssize_t;
}
extern "C" {
    pub fn getline(
        __lineptr: *mut *mut libc::c_char,
        __n: *mut usize,
        __stream: *mut FILE,
    ) -> __ssize_t;
}
extern "C" {
    pub fn fputs(__s: *const libc::c_char, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn puts(__s: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn ungetc(__c: libc::c_int, __stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fread(
        __ptr: *mut libc::c_void,
        __size: libc::c_ulong,
        __n: libc::c_ulong,
        __stream: *mut FILE,
    ) -> libc::c_ulong;
}
extern "C" {
    pub fn fwrite(
        __ptr: *const libc::c_void,
        __size: libc::c_ulong,
        __n: libc::c_ulong,
        __s: *mut FILE,
    ) -> libc::c_ulong;
}
extern "C" {
    pub fn fread_unlocked(
        __ptr: *mut libc::c_void,
        __size: usize,
        __n: usize,
        __stream: *mut FILE,
    ) -> usize;
}
extern "C" {
    pub fn fwrite_unlocked(
        __ptr: *const libc::c_void,
        __size: usize,
        __n: usize,
        __stream: *mut FILE,
    ) -> usize;
}
extern "C" {
    pub fn fseek(__stream: *mut FILE, __off: libc::c_long, __whence: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn ftell(__stream: *mut FILE) -> libc::c_long;
}
extern "C" {
    pub fn rewind(__stream: *mut FILE);
}
extern "C" {
    #[link_name = "\u{1}fseeko64"]
    pub fn fseeko(__stream: *mut FILE, __off: __off64_t, __whence: libc::c_int) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}ftello64"]
    pub fn ftello(__stream: *mut FILE) -> __off64_t;
}
extern "C" {
    #[link_name = "\u{1}fgetpos64"]
    pub fn fgetpos(__stream: *mut FILE, __pos: *mut fpos_t) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}fsetpos64"]
    pub fn fsetpos(__stream: *mut FILE, __pos: *const fpos_t) -> libc::c_int;
}
extern "C" {
    pub fn clearerr(__stream: *mut FILE);
}
extern "C" {
    pub fn feof(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn ferror(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn clearerr_unlocked(__stream: *mut FILE);
}
extern "C" {
    pub fn feof_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn ferror_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn perror(__s: *const libc::c_char);
}
extern "C" {
    pub static mut sys_nerr: libc::c_int;
}
extern "C" {
    pub static mut sys_errlist: [*const libc::c_char; 0usize];
}
extern "C" {
    pub fn fileno(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn fileno_unlocked(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn popen(__command: *const libc::c_char, __modes: *const libc::c_char) -> *mut FILE;
}
extern "C" {
    pub fn pclose(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn ctermid(__s: *mut libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn flockfile(__stream: *mut FILE);
}
extern "C" {
    pub fn ftrylockfile(__stream: *mut FILE) -> libc::c_int;
}
extern "C" {
    pub fn funlockfile(__stream: *mut FILE);
}
pub type wchar_t = libc::c_int;
pub const idtype_t_P_ALL: idtype_t = 0;
pub const idtype_t_P_PID: idtype_t = 1;
pub const idtype_t_P_PGID: idtype_t = 2;
pub type idtype_t = libc::c_uint;
pub type _Float32 = f32;
pub type _Float64 = f64;
pub type _Float32x = f64;
pub type _Float64x = u128;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct div_t {
    pub quot: libc::c_int,
    pub rem: libc::c_int,
}
#[test]
fn bindgen_test_layout_div_t() {
    assert_eq!(
        ::core::mem::size_of::<div_t>(),
        8usize,
        concat!("Size of: ", stringify!(div_t))
    );
    assert_eq!(
        ::core::mem::align_of::<div_t>(),
        4usize,
        concat!("Alignment of ", stringify!(div_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<div_t>())).quot as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(div_t),
            "::",
            stringify!(quot)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<div_t>())).rem as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(div_t),
            "::",
            stringify!(rem)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct ldiv_t {
    pub quot: libc::c_long,
    pub rem: libc::c_long,
}
#[test]
fn bindgen_test_layout_ldiv_t() {
    assert_eq!(
        ::core::mem::size_of::<ldiv_t>(),
        16usize,
        concat!("Size of: ", stringify!(ldiv_t))
    );
    assert_eq!(
        ::core::mem::align_of::<ldiv_t>(),
        8usize,
        concat!("Alignment of ", stringify!(ldiv_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ldiv_t>())).quot as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(ldiv_t),
            "::",
            stringify!(quot)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ldiv_t>())).rem as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(ldiv_t),
            "::",
            stringify!(rem)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct lldiv_t {
    pub quot: libc::c_longlong,
    pub rem: libc::c_longlong,
}
#[test]
fn bindgen_test_layout_lldiv_t() {
    assert_eq!(
        ::core::mem::size_of::<lldiv_t>(),
        16usize,
        concat!("Size of: ", stringify!(lldiv_t))
    );
    assert_eq!(
        ::core::mem::align_of::<lldiv_t>(),
        8usize,
        concat!("Alignment of ", stringify!(lldiv_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<lldiv_t>())).quot as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(lldiv_t),
            "::",
            stringify!(quot)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<lldiv_t>())).rem as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(lldiv_t),
            "::",
            stringify!(rem)
        )
    );
}
extern "C" {
    pub fn atof(__nptr: *const libc::c_char) -> f64;
}
extern "C" {
    pub fn atoi(__nptr: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn atol(__nptr: *const libc::c_char) -> libc::c_long;
}
extern "C" {
    pub fn atoll(__nptr: *const libc::c_char) -> libc::c_longlong;
}
extern "C" {
    pub fn strtod(__nptr: *const libc::c_char, __endptr: *mut *mut libc::c_char) -> f64;
}
extern "C" {
    pub fn strtof(__nptr: *const libc::c_char, __endptr: *mut *mut libc::c_char) -> f32;
}
extern "C" {
    pub fn strtol(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_long;
}
extern "C" {
    pub fn strtoul(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_ulong;
}
extern "C" {
    pub fn strtoq(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_longlong;
}
extern "C" {
    pub fn strtouq(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_ulonglong;
}
extern "C" {
    pub fn strtoll(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_longlong;
}
extern "C" {
    pub fn strtoull(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> libc::c_ulonglong;
}
extern "C" {
    pub fn l64a(__n: libc::c_long) -> *mut libc::c_char;
}
extern "C" {
    pub fn a64l(__s: *const libc::c_char) -> libc::c_long;
}
pub type u_char = __u_char;
pub type u_short = __u_short;
pub type u_int = __u_int;
pub type u_long = __u_long;
pub type quad_t = __quad_t;
pub type u_quad_t = __u_quad_t;
pub type fsid_t = __fsid_t;
pub type loff_t = __loff_t;
pub type ino_t = __ino64_t;
pub type dev_t = __dev_t;
pub type gid_t = __gid_t;
pub type mode_t = __mode_t;
pub type nlink_t = __nlink_t;
pub type uid_t = __uid_t;
pub type pid_t = __pid_t;
pub type id_t = __id_t;
pub type daddr_t = __daddr_t;
pub type caddr_t = __caddr_t;
pub type key_t = __key_t;
pub type clock_t = __clock_t;
pub type clockid_t = __clockid_t;
pub type time_t = __time_t;
pub type timer_t = __timer_t;
pub type ulong = libc::c_ulong;
pub type ushort = libc::c_ushort;
pub type uint = libc::c_uint;
pub type u_int8_t = __uint8_t;
pub type u_int16_t = __uint16_t;
pub type u_int32_t = __uint32_t;
pub type u_int64_t = __uint64_t;
pub type register_t = libc::c_long;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __sigset_t {
    pub __val: [libc::c_ulong; 16usize],
}
#[test]
fn bindgen_test_layout___sigset_t() {
    assert_eq!(
        ::core::mem::size_of::<__sigset_t>(),
        128usize,
        concat!("Size of: ", stringify!(__sigset_t))
    );
    assert_eq!(
        ::core::mem::align_of::<__sigset_t>(),
        8usize,
        concat!("Alignment of ", stringify!(__sigset_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__sigset_t>())).__val as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__sigset_t),
            "::",
            stringify!(__val)
        )
    );
}
pub type sigset_t = __sigset_t;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct timeval {
    pub tv_sec: __time_t,
    pub tv_usec: __suseconds_t,
}
#[test]
fn bindgen_test_layout_timeval() {
    assert_eq!(
        ::core::mem::size_of::<timeval>(),
        16usize,
        concat!("Size of: ", stringify!(timeval))
    );
    assert_eq!(
        ::core::mem::align_of::<timeval>(),
        8usize,
        concat!("Alignment of ", stringify!(timeval))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timeval>())).tv_sec as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(timeval),
            "::",
            stringify!(tv_sec)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timeval>())).tv_usec as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(timeval),
            "::",
            stringify!(tv_usec)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct timespec {
    pub tv_sec: __time_t,
    pub tv_nsec: __syscall_slong_t,
}
#[test]
fn bindgen_test_layout_timespec() {
    assert_eq!(
        ::core::mem::size_of::<timespec>(),
        16usize,
        concat!("Size of: ", stringify!(timespec))
    );
    assert_eq!(
        ::core::mem::align_of::<timespec>(),
        8usize,
        concat!("Alignment of ", stringify!(timespec))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timespec>())).tv_sec as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(timespec),
            "::",
            stringify!(tv_sec)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timespec>())).tv_nsec as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(timespec),
            "::",
            stringify!(tv_nsec)
        )
    );
}
pub type suseconds_t = __suseconds_t;
pub type __fd_mask = libc::c_long;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct fd_set {
    pub __fds_bits: [__fd_mask; 16usize],
}
#[test]
fn bindgen_test_layout_fd_set() {
    assert_eq!(
        ::core::mem::size_of::<fd_set>(),
        128usize,
        concat!("Size of: ", stringify!(fd_set))
    );
    assert_eq!(
        ::core::mem::align_of::<fd_set>(),
        8usize,
        concat!("Alignment of ", stringify!(fd_set))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<fd_set>())).__fds_bits as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(fd_set),
            "::",
            stringify!(__fds_bits)
        )
    );
}
pub type fd_mask = __fd_mask;
extern "C" {
    pub fn select(
        __nfds: libc::c_int,
        __readfds: *mut fd_set,
        __writefds: *mut fd_set,
        __exceptfds: *mut fd_set,
        __timeout: *mut timeval,
    ) -> libc::c_int;
}
extern "C" {
    pub fn pselect(
        __nfds: libc::c_int,
        __readfds: *mut fd_set,
        __writefds: *mut fd_set,
        __exceptfds: *mut fd_set,
        __timeout: *const timespec,
        __sigmask: *const __sigset_t,
    ) -> libc::c_int;
}
pub type blksize_t = __blksize_t;
pub type blkcnt_t = __blkcnt64_t;
pub type fsblkcnt_t = __fsblkcnt64_t;
pub type fsfilcnt_t = __fsfilcnt64_t;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_internal_list {
    pub __prev: *mut __pthread_internal_list,
    pub __next: *mut __pthread_internal_list,
}
#[test]
fn bindgen_test_layout___pthread_internal_list() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_internal_list>(),
        16usize,
        concat!("Size of: ", stringify!(__pthread_internal_list))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_internal_list>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_internal_list))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_internal_list>())).__prev as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_internal_list),
            "::",
            stringify!(__prev)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_internal_list>())).__next as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_internal_list),
            "::",
            stringify!(__next)
        )
    );
}
pub type __pthread_list_t = __pthread_internal_list;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_internal_slist {
    pub __next: *mut __pthread_internal_slist,
}
#[test]
fn bindgen_test_layout___pthread_internal_slist() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_internal_slist>(),
        8usize,
        concat!("Size of: ", stringify!(__pthread_internal_slist))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_internal_slist>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_internal_slist))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_internal_slist>())).__next as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_internal_slist),
            "::",
            stringify!(__next)
        )
    );
}
pub type __pthread_slist_t = __pthread_internal_slist;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_mutex_s {
    pub __lock: libc::c_int,
    pub __count: libc::c_uint,
    pub __owner: libc::c_int,
    pub __nusers: libc::c_uint,
    pub __kind: libc::c_int,
    pub __spins: libc::c_short,
    pub __elision: libc::c_short,
    pub __list: __pthread_list_t,
}
#[test]
fn bindgen_test_layout___pthread_mutex_s() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_mutex_s>(),
        40usize,
        concat!("Size of: ", stringify!(__pthread_mutex_s))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_mutex_s>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_mutex_s))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__lock as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__lock)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__count as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__owner as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__owner)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__nusers as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__nusers)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__kind as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__kind)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__spins as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__spins)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__elision as *const _ as usize },
        22usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__elision)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_mutex_s>())).__list as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_mutex_s),
            "::",
            stringify!(__list)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_rwlock_arch_t {
    pub __readers: libc::c_uint,
    pub __writers: libc::c_uint,
    pub __wrphase_futex: libc::c_uint,
    pub __writers_futex: libc::c_uint,
    pub __pad3: libc::c_uint,
    pub __pad4: libc::c_uint,
    pub __cur_writer: libc::c_int,
    pub __shared: libc::c_int,
    pub __rwelision: libc::c_schar,
    pub __pad1: [libc::c_uchar; 7usize],
    pub __pad2: libc::c_ulong,
    pub __flags: libc::c_uint,
}
#[test]
fn bindgen_test_layout___pthread_rwlock_arch_t() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_rwlock_arch_t>(),
        56usize,
        concat!("Size of: ", stringify!(__pthread_rwlock_arch_t))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_rwlock_arch_t>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_rwlock_arch_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__readers as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__readers)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__writers as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__writers)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__wrphase_futex as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__wrphase_futex)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__writers_futex as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__writers_futex)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__pad3 as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__pad3)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__pad4 as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__pad4)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__cur_writer as *const _ as usize
        },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__cur_writer)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__shared as *const _ as usize
        },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__shared)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__rwelision as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__rwelision)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__pad1 as *const _ as usize },
        33usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__pad1)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__pad2 as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__pad2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_rwlock_arch_t>())).__flags as *const _ as usize
        },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_rwlock_arch_t),
            "::",
            stringify!(__flags)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct __pthread_cond_s {
    pub __bindgen_anon_1: __pthread_cond_s__bindgen_ty_1,
    pub __bindgen_anon_2: __pthread_cond_s__bindgen_ty_2,
    pub __g_refs: [libc::c_uint; 2usize],
    pub __g_size: [libc::c_uint; 2usize],
    pub __g1_orig_size: libc::c_uint,
    pub __wrefs: libc::c_uint,
    pub __g_signals: [libc::c_uint; 2usize],
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union __pthread_cond_s__bindgen_ty_1 {
    pub __wseq: libc::c_ulonglong,
    pub __wseq32: __pthread_cond_s__bindgen_ty_1__bindgen_ty_1,
    _bindgen_union_align: u64,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_cond_s__bindgen_ty_1__bindgen_ty_1 {
    pub __low: libc::c_uint,
    pub __high: libc::c_uint,
}
#[test]
fn bindgen_test_layout___pthread_cond_s__bindgen_ty_1__bindgen_ty_1() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_cond_s__bindgen_ty_1__bindgen_ty_1>(),
        8usize,
        concat!(
            "Size of: ",
            stringify!(__pthread_cond_s__bindgen_ty_1__bindgen_ty_1)
        )
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_cond_s__bindgen_ty_1__bindgen_ty_1>(),
        4usize,
        concat!(
            "Alignment of ",
            stringify!(__pthread_cond_s__bindgen_ty_1__bindgen_ty_1)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_1__bindgen_ty_1>())).__low
                as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_1__bindgen_ty_1),
            "::",
            stringify!(__low)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_1__bindgen_ty_1>())).__high
                as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_1__bindgen_ty_1),
            "::",
            stringify!(__high)
        )
    );
}
#[test]
fn bindgen_test_layout___pthread_cond_s__bindgen_ty_1() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_cond_s__bindgen_ty_1>(),
        8usize,
        concat!("Size of: ", stringify!(__pthread_cond_s__bindgen_ty_1))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_cond_s__bindgen_ty_1>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_cond_s__bindgen_ty_1))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_1>())).__wseq as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_1),
            "::",
            stringify!(__wseq)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_1>())).__wseq32 as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_1),
            "::",
            stringify!(__wseq32)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union __pthread_cond_s__bindgen_ty_2 {
    pub __g1_start: libc::c_ulonglong,
    pub __g1_start32: __pthread_cond_s__bindgen_ty_2__bindgen_ty_1,
    _bindgen_union_align: u64,
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __pthread_cond_s__bindgen_ty_2__bindgen_ty_1 {
    pub __low: libc::c_uint,
    pub __high: libc::c_uint,
}
#[test]
fn bindgen_test_layout___pthread_cond_s__bindgen_ty_2__bindgen_ty_1() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_cond_s__bindgen_ty_2__bindgen_ty_1>(),
        8usize,
        concat!(
            "Size of: ",
            stringify!(__pthread_cond_s__bindgen_ty_2__bindgen_ty_1)
        )
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_cond_s__bindgen_ty_2__bindgen_ty_1>(),
        4usize,
        concat!(
            "Alignment of ",
            stringify!(__pthread_cond_s__bindgen_ty_2__bindgen_ty_1)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_2__bindgen_ty_1>())).__low
                as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_2__bindgen_ty_1),
            "::",
            stringify!(__low)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_2__bindgen_ty_1>())).__high
                as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_2__bindgen_ty_1),
            "::",
            stringify!(__high)
        )
    );
}
#[test]
fn bindgen_test_layout___pthread_cond_s__bindgen_ty_2() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_cond_s__bindgen_ty_2>(),
        8usize,
        concat!("Size of: ", stringify!(__pthread_cond_s__bindgen_ty_2))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_cond_s__bindgen_ty_2>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_cond_s__bindgen_ty_2))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_2>())).__g1_start as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_2),
            "::",
            stringify!(__g1_start)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s__bindgen_ty_2>())).__g1_start32 as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s__bindgen_ty_2),
            "::",
            stringify!(__g1_start32)
        )
    );
}
#[test]
fn bindgen_test_layout___pthread_cond_s() {
    assert_eq!(
        ::core::mem::size_of::<__pthread_cond_s>(),
        48usize,
        concat!("Size of: ", stringify!(__pthread_cond_s))
    );
    assert_eq!(
        ::core::mem::align_of::<__pthread_cond_s>(),
        8usize,
        concat!("Alignment of ", stringify!(__pthread_cond_s))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_cond_s>())).__g_refs as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s),
            "::",
            stringify!(__g_refs)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_cond_s>())).__g_size as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s),
            "::",
            stringify!(__g_size)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__pthread_cond_s>())).__g1_orig_size as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s),
            "::",
            stringify!(__g1_orig_size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_cond_s>())).__wrefs as *const _ as usize },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s),
            "::",
            stringify!(__wrefs)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__pthread_cond_s>())).__g_signals as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(__pthread_cond_s),
            "::",
            stringify!(__g_signals)
        )
    );
}
pub type pthread_t = libc::c_ulong;
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_mutexattr_t {
    pub __size: [libc::c_char; 4usize],
    pub __align: libc::c_int,
    _bindgen_union_align: u32,
}
#[test]
fn bindgen_test_layout_pthread_mutexattr_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_mutexattr_t>(),
        4usize,
        concat!("Size of: ", stringify!(pthread_mutexattr_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_mutexattr_t>(),
        4usize,
        concat!("Alignment of ", stringify!(pthread_mutexattr_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_mutexattr_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_mutexattr_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_mutexattr_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_mutexattr_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_condattr_t {
    pub __size: [libc::c_char; 4usize],
    pub __align: libc::c_int,
    _bindgen_union_align: u32,
}
#[test]
fn bindgen_test_layout_pthread_condattr_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_condattr_t>(),
        4usize,
        concat!("Size of: ", stringify!(pthread_condattr_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_condattr_t>(),
        4usize,
        concat!("Alignment of ", stringify!(pthread_condattr_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_condattr_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_condattr_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_condattr_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_condattr_t),
            "::",
            stringify!(__align)
        )
    );
}
pub type pthread_key_t = libc::c_uint;
pub type pthread_once_t = libc::c_int;
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_attr_t {
    pub __size: [libc::c_char; 56usize],
    pub __align: libc::c_long,
    _bindgen_union_align: [u64; 7usize],
}
#[test]
fn bindgen_test_layout_pthread_attr_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_attr_t>(),
        56usize,
        concat!("Size of: ", stringify!(pthread_attr_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_attr_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_attr_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_attr_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_attr_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_attr_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_attr_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_mutex_t {
    pub __data: __pthread_mutex_s,
    pub __size: [libc::c_char; 40usize],
    pub __align: libc::c_long,
    _bindgen_union_align: [u64; 5usize],
}
#[test]
fn bindgen_test_layout_pthread_mutex_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_mutex_t>(),
        40usize,
        concat!("Size of: ", stringify!(pthread_mutex_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_mutex_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_mutex_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_mutex_t>())).__data as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_mutex_t),
            "::",
            stringify!(__data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_mutex_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_mutex_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_mutex_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_mutex_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_cond_t {
    pub __data: __pthread_cond_s,
    pub __size: [libc::c_char; 48usize],
    pub __align: libc::c_longlong,
    _bindgen_union_align: [u64; 6usize],
}
#[test]
fn bindgen_test_layout_pthread_cond_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_cond_t>(),
        48usize,
        concat!("Size of: ", stringify!(pthread_cond_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_cond_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_cond_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_cond_t>())).__data as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_cond_t),
            "::",
            stringify!(__data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_cond_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_cond_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_cond_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_cond_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_rwlock_t {
    pub __data: __pthread_rwlock_arch_t,
    pub __size: [libc::c_char; 56usize],
    pub __align: libc::c_long,
    _bindgen_union_align: [u64; 7usize],
}
#[test]
fn bindgen_test_layout_pthread_rwlock_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_rwlock_t>(),
        56usize,
        concat!("Size of: ", stringify!(pthread_rwlock_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_rwlock_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_rwlock_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_rwlock_t>())).__data as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_rwlock_t),
            "::",
            stringify!(__data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_rwlock_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_rwlock_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_rwlock_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_rwlock_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_rwlockattr_t {
    pub __size: [libc::c_char; 8usize],
    pub __align: libc::c_long,
    _bindgen_union_align: u64,
}
#[test]
fn bindgen_test_layout_pthread_rwlockattr_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_rwlockattr_t>(),
        8usize,
        concat!("Size of: ", stringify!(pthread_rwlockattr_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_rwlockattr_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_rwlockattr_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_rwlockattr_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_rwlockattr_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_rwlockattr_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_rwlockattr_t),
            "::",
            stringify!(__align)
        )
    );
}
pub type pthread_spinlock_t = libc::c_int;
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_barrier_t {
    pub __size: [libc::c_char; 32usize],
    pub __align: libc::c_long,
    _bindgen_union_align: [u64; 4usize],
}
#[test]
fn bindgen_test_layout_pthread_barrier_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_barrier_t>(),
        32usize,
        concat!("Size of: ", stringify!(pthread_barrier_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_barrier_t>(),
        8usize,
        concat!("Alignment of ", stringify!(pthread_barrier_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_barrier_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_barrier_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_barrier_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_barrier_t),
            "::",
            stringify!(__align)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union pthread_barrierattr_t {
    pub __size: [libc::c_char; 4usize],
    pub __align: libc::c_int,
    _bindgen_union_align: u32,
}
#[test]
fn bindgen_test_layout_pthread_barrierattr_t() {
    assert_eq!(
        ::core::mem::size_of::<pthread_barrierattr_t>(),
        4usize,
        concat!("Size of: ", stringify!(pthread_barrierattr_t))
    );
    assert_eq!(
        ::core::mem::align_of::<pthread_barrierattr_t>(),
        4usize,
        concat!("Alignment of ", stringify!(pthread_barrierattr_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_barrierattr_t>())).__size as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_barrierattr_t),
            "::",
            stringify!(__size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<pthread_barrierattr_t>())).__align as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(pthread_barrierattr_t),
            "::",
            stringify!(__align)
        )
    );
}
extern "C" {
    pub fn random() -> libc::c_long;
}
extern "C" {
    pub fn srandom(__seed: libc::c_uint);
}
extern "C" {
    pub fn initstate(
        __seed: libc::c_uint,
        __statebuf: *mut libc::c_char,
        __statelen: usize,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn setstate(__statebuf: *mut libc::c_char) -> *mut libc::c_char;
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct random_data {
    pub fptr: *mut i32,
    pub rptr: *mut i32,
    pub state: *mut i32,
    pub rand_type: libc::c_int,
    pub rand_deg: libc::c_int,
    pub rand_sep: libc::c_int,
    pub end_ptr: *mut i32,
}
#[test]
fn bindgen_test_layout_random_data() {
    assert_eq!(
        ::core::mem::size_of::<random_data>(),
        48usize,
        concat!("Size of: ", stringify!(random_data))
    );
    assert_eq!(
        ::core::mem::align_of::<random_data>(),
        8usize,
        concat!("Alignment of ", stringify!(random_data))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).fptr as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(fptr)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).rptr as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(rptr)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).state as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(state)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).rand_type as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(rand_type)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).rand_deg as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(rand_deg)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).rand_sep as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(rand_sep)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<random_data>())).end_ptr as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(random_data),
            "::",
            stringify!(end_ptr)
        )
    );
}
extern "C" {
    pub fn random_r(__buf: *mut random_data, __result: *mut i32) -> libc::c_int;
}
extern "C" {
    pub fn srandom_r(__seed: libc::c_uint, __buf: *mut random_data) -> libc::c_int;
}
extern "C" {
    pub fn initstate_r(
        __seed: libc::c_uint,
        __statebuf: *mut libc::c_char,
        __statelen: usize,
        __buf: *mut random_data,
    ) -> libc::c_int;
}
extern "C" {
    pub fn setstate_r(__statebuf: *mut libc::c_char, __buf: *mut random_data) -> libc::c_int;
}
extern "C" {
    pub fn rand() -> libc::c_int;
}
extern "C" {
    pub fn srand(__seed: libc::c_uint);
}
extern "C" {
    pub fn rand_r(__seed: *mut libc::c_uint) -> libc::c_int;
}
extern "C" {
    pub fn drand48() -> f64;
}
extern "C" {
    pub fn erand48(__xsubi: *mut libc::c_ushort) -> f64;
}
extern "C" {
    pub fn lrand48() -> libc::c_long;
}
extern "C" {
    pub fn nrand48(__xsubi: *mut libc::c_ushort) -> libc::c_long;
}
extern "C" {
    pub fn mrand48() -> libc::c_long;
}
extern "C" {
    pub fn jrand48(__xsubi: *mut libc::c_ushort) -> libc::c_long;
}
extern "C" {
    pub fn srand48(__seedval: libc::c_long);
}
extern "C" {
    pub fn seed48(__seed16v: *mut libc::c_ushort) -> *mut libc::c_ushort;
}
extern "C" {
    pub fn lcong48(__param: *mut libc::c_ushort);
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct drand48_data {
    pub __x: [libc::c_ushort; 3usize],
    pub __old_x: [libc::c_ushort; 3usize],
    pub __c: libc::c_ushort,
    pub __init: libc::c_ushort,
    pub __a: libc::c_ulonglong,
}
#[test]
fn bindgen_test_layout_drand48_data() {
    assert_eq!(
        ::core::mem::size_of::<drand48_data>(),
        24usize,
        concat!("Size of: ", stringify!(drand48_data))
    );
    assert_eq!(
        ::core::mem::align_of::<drand48_data>(),
        8usize,
        concat!("Alignment of ", stringify!(drand48_data))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<drand48_data>())).__x as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(drand48_data),
            "::",
            stringify!(__x)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<drand48_data>())).__old_x as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(drand48_data),
            "::",
            stringify!(__old_x)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<drand48_data>())).__c as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(drand48_data),
            "::",
            stringify!(__c)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<drand48_data>())).__init as *const _ as usize },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(drand48_data),
            "::",
            stringify!(__init)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<drand48_data>())).__a as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(drand48_data),
            "::",
            stringify!(__a)
        )
    );
}
extern "C" {
    pub fn drand48_r(__buffer: *mut drand48_data, __result: *mut f64) -> libc::c_int;
}
extern "C" {
    pub fn erand48_r(
        __xsubi: *mut libc::c_ushort,
        __buffer: *mut drand48_data,
        __result: *mut f64,
    ) -> libc::c_int;
}
extern "C" {
    pub fn lrand48_r(__buffer: *mut drand48_data, __result: *mut libc::c_long) -> libc::c_int;
}
extern "C" {
    pub fn nrand48_r(
        __xsubi: *mut libc::c_ushort,
        __buffer: *mut drand48_data,
        __result: *mut libc::c_long,
    ) -> libc::c_int;
}
extern "C" {
    pub fn mrand48_r(__buffer: *mut drand48_data, __result: *mut libc::c_long) -> libc::c_int;
}
extern "C" {
    pub fn jrand48_r(
        __xsubi: *mut libc::c_ushort,
        __buffer: *mut drand48_data,
        __result: *mut libc::c_long,
    ) -> libc::c_int;
}
extern "C" {
    pub fn srand48_r(__seedval: libc::c_long, __buffer: *mut drand48_data) -> libc::c_int;
}
extern "C" {
    pub fn seed48_r(__seed16v: *mut libc::c_ushort, __buffer: *mut drand48_data) -> libc::c_int;
}
extern "C" {
    pub fn lcong48_r(__param: *mut libc::c_ushort, __buffer: *mut drand48_data) -> libc::c_int;
}
extern "C" {
    pub fn malloc(__size: libc::c_ulong) -> *mut libc::c_void;
}
extern "C" {
    pub fn calloc(__nmemb: libc::c_ulong, __size: libc::c_ulong) -> *mut libc::c_void;
}
extern "C" {
    pub fn realloc(__ptr: *mut libc::c_void, __size: libc::c_ulong) -> *mut libc::c_void;
}
extern "C" {
    pub fn reallocarray(
        __ptr: *mut libc::c_void,
        __nmemb: usize,
        __size: usize,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn free(__ptr: *mut libc::c_void);
}
extern "C" {
    pub fn alloca(__size: libc::c_ulong) -> *mut libc::c_void;
}
extern "C" {
    pub fn valloc(__size: usize) -> *mut libc::c_void;
}
extern "C" {
    pub fn posix_memalign(
        __memptr: *mut *mut libc::c_void,
        __alignment: usize,
        __size: usize,
    ) -> libc::c_int;
}
extern "C" {
    pub fn aligned_alloc(__alignment: usize, __size: usize) -> *mut libc::c_void;
}
extern "C" {
    pub fn abort();
}
extern "C" {
    pub fn atexit(__func: ::core::option::Option<unsafe extern "C" fn()>) -> libc::c_int;
}
extern "C" {
    pub fn at_quick_exit(__func: ::core::option::Option<unsafe extern "C" fn()>) -> libc::c_int;
}
extern "C" {
    pub fn on_exit(
        __func: ::core::option::Option<
            unsafe extern "C" fn(__status: libc::c_int, __arg: *mut libc::c_void),
        >,
        __arg: *mut libc::c_void,
    ) -> libc::c_int;
}
extern "C" {
    pub fn exit(__status: libc::c_int);
}
extern "C" {
    pub fn quick_exit(__status: libc::c_int);
}
extern "C" {
    pub fn getenv(__name: *const libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn putenv(__string: *mut libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn setenv(
        __name: *const libc::c_char,
        __value: *const libc::c_char,
        __replace: libc::c_int,
    ) -> libc::c_int;
}
extern "C" {
    pub fn unsetenv(__name: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn clearenv() -> libc::c_int;
}
extern "C" {
    pub fn mktemp(__template: *mut libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    #[link_name = "\u{1}mkstemp64"]
    pub fn mkstemp(__template: *mut libc::c_char) -> libc::c_int;
}
extern "C" {
    #[link_name = "\u{1}mkstemps64"]
    pub fn mkstemps(__template: *mut libc::c_char, __suffixlen: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn mkdtemp(__template: *mut libc::c_char) -> *mut libc::c_char;
}
extern "C" {
    pub fn system(__command: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn realpath(
        __name: *const libc::c_char,
        __resolved: *mut libc::c_char,
    ) -> *mut libc::c_char;
}
pub type __compar_fn_t = ::core::option::Option<
    unsafe extern "C" fn(arg1: *const libc::c_void, arg2: *const libc::c_void) -> libc::c_int,
>;
extern "C" {
    pub fn bsearch(
        __key: *const libc::c_void,
        __base: *const libc::c_void,
        __nmemb: usize,
        __size: usize,
        __compar: __compar_fn_t,
    ) -> *mut libc::c_void;
}
extern "C" {
    pub fn qsort(__base: *mut libc::c_void, __nmemb: usize, __size: usize, __compar: __compar_fn_t);
}
extern "C" {
    pub fn abs(__x: libc::c_int) -> libc::c_int;
}
extern "C" {
    pub fn labs(__x: libc::c_long) -> libc::c_long;
}
extern "C" {
    pub fn llabs(__x: libc::c_longlong) -> libc::c_longlong;
}
extern "C" {
    pub fn div(__numer: libc::c_int, __denom: libc::c_int) -> div_t;
}
extern "C" {
    pub fn ldiv(__numer: libc::c_long, __denom: libc::c_long) -> ldiv_t;
}
extern "C" {
    pub fn lldiv(__numer: libc::c_longlong, __denom: libc::c_longlong) -> lldiv_t;
}
extern "C" {
    pub fn ecvt(
        __value: f64,
        __ndigit: libc::c_int,
        __decpt: *mut libc::c_int,
        __sign: *mut libc::c_int,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn fcvt(
        __value: f64,
        __ndigit: libc::c_int,
        __decpt: *mut libc::c_int,
        __sign: *mut libc::c_int,
    ) -> *mut libc::c_char;
}
extern "C" {
    pub fn gcvt(__value: f64, __ndigit: libc::c_int, __buf: *mut libc::c_char)
        -> *mut libc::c_char;
}
extern "C" {
    pub fn mblen(__s: *const libc::c_char, __n: usize) -> libc::c_int;
}
extern "C" {
    pub fn mbtowc(__pwc: *mut wchar_t, __s: *const libc::c_char, __n: usize) -> libc::c_int;
}
extern "C" {
    pub fn wctomb(__s: *mut libc::c_char, __wchar: wchar_t) -> libc::c_int;
}
extern "C" {
    pub fn mbstowcs(__pwcs: *mut wchar_t, __s: *const libc::c_char, __n: usize) -> usize;
}
extern "C" {
    pub fn wcstombs(__s: *mut libc::c_char, __pwcs: *const wchar_t, __n: usize) -> usize;
}
extern "C" {
    pub fn rpmatch(__response: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn getsubopt(
        __optionp: *mut *mut libc::c_char,
        __tokens: *const *mut libc::c_char,
        __valuep: *mut *mut libc::c_char,
    ) -> libc::c_int;
}
extern "C" {
    pub fn getloadavg(__loadavg: *mut f64, __nelem: libc::c_int) -> libc::c_int;
}
pub type float_t = f32;
pub type double_t = f64;
extern "C" {
    pub fn acos(__x: f64) -> f64;
}
extern "C" {
    pub fn asin(__x: f64) -> f64;
}
extern "C" {
    pub fn atan(__x: f64) -> f64;
}
extern "C" {
    pub fn atan2(__y: f64, __x: f64) -> f64;
}
extern "C" {
    pub fn cos(__x: f64) -> f64;
}
extern "C" {
    pub fn sin(__x: f64) -> f64;
}
extern "C" {
    pub fn tan(__x: f64) -> f64;
}
extern "C" {
    pub fn cosh(__x: f64) -> f64;
}
extern "C" {
    pub fn sinh(__x: f64) -> f64;
}
extern "C" {
    pub fn tanh(__x: f64) -> f64;
}
extern "C" {
    pub fn acosh(__x: f64) -> f64;
}
extern "C" {
    pub fn asinh(__x: f64) -> f64;
}
extern "C" {
    pub fn atanh(__x: f64) -> f64;
}
extern "C" {
    pub fn exp(__x: f64) -> f64;
}
extern "C" {
    pub fn frexp(__x: f64, __exponent: *mut libc::c_int) -> f64;
}
extern "C" {
    pub fn ldexp(__x: f64, __exponent: libc::c_int) -> f64;
}
extern "C" {
    pub fn log(__x: f64) -> f64;
}
extern "C" {
    pub fn log10(__x: f64) -> f64;
}
extern "C" {
    pub fn modf(__x: f64, __iptr: *mut f64) -> f64;
}
extern "C" {
    pub fn expm1(__x: f64) -> f64;
}
extern "C" {
    pub fn log1p(__x: f64) -> f64;
}
extern "C" {
    pub fn logb(__x: f64) -> f64;
}
extern "C" {
    pub fn exp2(__x: f64) -> f64;
}
extern "C" {
    pub fn log2(__x: f64) -> f64;
}
extern "C" {
    pub fn pow(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn sqrt(__x: f64) -> f64;
}
extern "C" {
    pub fn hypot(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn cbrt(__x: f64) -> f64;
}
extern "C" {
    pub fn ceil(__x: f64) -> f64;
}
extern "C" {
    pub fn fabs(__x: f64) -> f64;
}
extern "C" {
    pub fn floor(__x: f64) -> f64;
}
extern "C" {
    pub fn fmod(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn isinf(__value: f64) -> libc::c_int;
}
extern "C" {
    pub fn finite(__value: f64) -> libc::c_int;
}
extern "C" {
    pub fn drem(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn significand(__x: f64) -> f64;
}
extern "C" {
    pub fn copysign(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn nan(__tagb: *const libc::c_char) -> f64;
}
extern "C" {
    pub fn isnan(__value: f64) -> libc::c_int;
}
extern "C" {
    pub fn j0(arg1: f64) -> f64;
}
extern "C" {
    pub fn j1(arg1: f64) -> f64;
}
extern "C" {
    pub fn jn(arg1: libc::c_int, arg2: f64) -> f64;
}
extern "C" {
    pub fn y0(arg1: f64) -> f64;
}
extern "C" {
    pub fn y1(arg1: f64) -> f64;
}
extern "C" {
    pub fn yn(arg1: libc::c_int, arg2: f64) -> f64;
}
extern "C" {
    pub fn erf(arg1: f64) -> f64;
}
extern "C" {
    pub fn erfc(arg1: f64) -> f64;
}
extern "C" {
    pub fn lgamma(arg1: f64) -> f64;
}
extern "C" {
    pub fn tgamma(arg1: f64) -> f64;
}
extern "C" {
    pub fn gamma(arg1: f64) -> f64;
}
extern "C" {
    pub fn lgamma_r(arg1: f64, __signgamp: *mut libc::c_int) -> f64;
}
extern "C" {
    pub fn rint(__x: f64) -> f64;
}
extern "C" {
    pub fn nextafter(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn remainder(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn scalbn(__x: f64, __n: libc::c_int) -> f64;
}
extern "C" {
    pub fn ilogb(__x: f64) -> libc::c_int;
}
extern "C" {
    pub fn scalbln(__x: f64, __n: libc::c_long) -> f64;
}
extern "C" {
    pub fn nearbyint(__x: f64) -> f64;
}
extern "C" {
    pub fn round(__x: f64) -> f64;
}
extern "C" {
    pub fn trunc(__x: f64) -> f64;
}
extern "C" {
    pub fn remquo(__x: f64, __y: f64, __quo: *mut libc::c_int) -> f64;
}
extern "C" {
    pub fn lrint(__x: f64) -> libc::c_long;
}
extern "C" {
    pub fn llrint(__x: f64) -> libc::c_longlong;
}
extern "C" {
    pub fn lround(__x: f64) -> libc::c_long;
}
extern "C" {
    pub fn llround(__x: f64) -> libc::c_longlong;
}
extern "C" {
    pub fn fdim(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn fmax(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn fmin(__x: f64, __y: f64) -> f64;
}
extern "C" {
    pub fn fma(__x: f64, __y: f64, __z: f64) -> f64;
}
extern "C" {
    pub fn scalb(__x: f64, __n: f64) -> f64;
}
extern "C" {
    pub fn acosf(__x: f32) -> f32;
}
extern "C" {
    pub fn asinf(__x: f32) -> f32;
}
extern "C" {
    pub fn atanf(__x: f32) -> f32;
}
extern "C" {
    pub fn atan2f(__y: f32, __x: f32) -> f32;
}
extern "C" {
    pub fn cosf(__x: f32) -> f32;
}
extern "C" {
    pub fn sinf(__x: f32) -> f32;
}
extern "C" {
    pub fn tanf(__x: f32) -> f32;
}
extern "C" {
    pub fn coshf(__x: f32) -> f32;
}
extern "C" {
    pub fn sinhf(__x: f32) -> f32;
}
extern "C" {
    pub fn tanhf(__x: f32) -> f32;
}
extern "C" {
    pub fn acoshf(__x: f32) -> f32;
}
extern "C" {
    pub fn asinhf(__x: f32) -> f32;
}
extern "C" {
    pub fn atanhf(__x: f32) -> f32;
}
extern "C" {
    pub fn expf(__x: f32) -> f32;
}
extern "C" {
    pub fn frexpf(__x: f32, __exponent: *mut libc::c_int) -> f32;
}
extern "C" {
    pub fn ldexpf(__x: f32, __exponent: libc::c_int) -> f32;
}
extern "C" {
    pub fn logf(__x: f32) -> f32;
}
extern "C" {
    pub fn log10f(__x: f32) -> f32;
}
extern "C" {
    pub fn modff(__x: f32, __iptr: *mut f32) -> f32;
}
extern "C" {
    pub fn expm1f(__x: f32) -> f32;
}
extern "C" {
    pub fn log1pf(__x: f32) -> f32;
}
extern "C" {
    pub fn logbf(__x: f32) -> f32;
}
extern "C" {
    pub fn exp2f(__x: f32) -> f32;
}
extern "C" {
    pub fn log2f(__x: f32) -> f32;
}
extern "C" {
    pub fn powf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn sqrtf(__x: f32) -> f32;
}
extern "C" {
    pub fn hypotf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn cbrtf(__x: f32) -> f32;
}
extern "C" {
    pub fn ceilf(__x: f32) -> f32;
}
extern "C" {
    pub fn fabsf(__x: f32) -> f32;
}
extern "C" {
    pub fn floorf(__x: f32) -> f32;
}
extern "C" {
    pub fn fmodf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn isinff(__value: f32) -> libc::c_int;
}
extern "C" {
    pub fn finitef(__value: f32) -> libc::c_int;
}
extern "C" {
    pub fn dremf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn significandf(__x: f32) -> f32;
}
extern "C" {
    pub fn copysignf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn nanf(__tagb: *const libc::c_char) -> f32;
}
extern "C" {
    pub fn isnanf(__value: f32) -> libc::c_int;
}
extern "C" {
    pub fn j0f(arg1: f32) -> f32;
}
extern "C" {
    pub fn j1f(arg1: f32) -> f32;
}
extern "C" {
    pub fn jnf(arg1: libc::c_int, arg2: f32) -> f32;
}
extern "C" {
    pub fn y0f(arg1: f32) -> f32;
}
extern "C" {
    pub fn y1f(arg1: f32) -> f32;
}
extern "C" {
    pub fn ynf(arg1: libc::c_int, arg2: f32) -> f32;
}
extern "C" {
    pub fn erff(arg1: f32) -> f32;
}
extern "C" {
    pub fn erfcf(arg1: f32) -> f32;
}
extern "C" {
    pub fn lgammaf(arg1: f32) -> f32;
}
extern "C" {
    pub fn tgammaf(arg1: f32) -> f32;
}
extern "C" {
    pub fn gammaf(arg1: f32) -> f32;
}
extern "C" {
    pub fn lgammaf_r(arg1: f32, __signgamp: *mut libc::c_int) -> f32;
}
extern "C" {
    pub fn rintf(__x: f32) -> f32;
}
extern "C" {
    pub fn nextafterf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn remainderf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn scalbnf(__x: f32, __n: libc::c_int) -> f32;
}
extern "C" {
    pub fn ilogbf(__x: f32) -> libc::c_int;
}
extern "C" {
    pub fn scalblnf(__x: f32, __n: libc::c_long) -> f32;
}
extern "C" {
    pub fn nearbyintf(__x: f32) -> f32;
}
extern "C" {
    pub fn roundf(__x: f32) -> f32;
}
extern "C" {
    pub fn truncf(__x: f32) -> f32;
}
extern "C" {
    pub fn remquof(__x: f32, __y: f32, __quo: *mut libc::c_int) -> f32;
}
extern "C" {
    pub fn lrintf(__x: f32) -> libc::c_long;
}
extern "C" {
    pub fn llrintf(__x: f32) -> libc::c_longlong;
}
extern "C" {
    pub fn lroundf(__x: f32) -> libc::c_long;
}
extern "C" {
    pub fn llroundf(__x: f32) -> libc::c_longlong;
}
extern "C" {
    pub fn fdimf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn fmaxf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn fminf(__x: f32, __y: f32) -> f32;
}
extern "C" {
    pub fn fmaf(__x: f32, __y: f32, __z: f32) -> f32;
}
extern "C" {
    pub fn scalbf(__x: f32, __n: f32) -> f32;
}
extern "C" {
    pub static mut signgam: libc::c_int;
}
pub const FP_NAN: libc::c_uint = 0;
pub const FP_INFINITE: libc::c_uint = 1;
pub const FP_ZERO: libc::c_uint = 2;
pub const FP_SUBNORMAL: libc::c_uint = 3;
pub const FP_NORMAL: libc::c_uint = 4;
pub type _bindgen_ty_1 = libc::c_uint;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct timezone {
    pub tz_minuteswest: libc::c_int,
    pub tz_dsttime: libc::c_int,
}
#[test]
fn bindgen_test_layout_timezone() {
    assert_eq!(
        ::core::mem::size_of::<timezone>(),
        8usize,
        concat!("Size of: ", stringify!(timezone))
    );
    assert_eq!(
        ::core::mem::align_of::<timezone>(),
        4usize,
        concat!("Alignment of ", stringify!(timezone))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timezone>())).tz_minuteswest as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(timezone),
            "::",
            stringify!(tz_minuteswest)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<timezone>())).tz_dsttime as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(timezone),
            "::",
            stringify!(tz_dsttime)
        )
    );
}
extern "C" {
    pub fn gettimeofday(__tv: *mut timeval, __tz: *mut libc::c_void) -> libc::c_int;
}
extern "C" {
    pub fn settimeofday(__tv: *const timeval, __tz: *const timezone) -> libc::c_int;
}
extern "C" {
    pub fn adjtime(__delta: *const timeval, __olddelta: *mut timeval) -> libc::c_int;
}
pub const __itimer_which_ITIMER_REAL: __itimer_which = 0;
pub const __itimer_which_ITIMER_VIRTUAL: __itimer_which = 1;
pub const __itimer_which_ITIMER_PROF: __itimer_which = 2;
pub type __itimer_which = libc::c_uint;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct itimerval {
    pub it_interval: timeval,
    pub it_value: timeval,
}
#[test]
fn bindgen_test_layout_itimerval() {
    assert_eq!(
        ::core::mem::size_of::<itimerval>(),
        32usize,
        concat!("Size of: ", stringify!(itimerval))
    );
    assert_eq!(
        ::core::mem::align_of::<itimerval>(),
        8usize,
        concat!("Alignment of ", stringify!(itimerval))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<itimerval>())).it_interval as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(itimerval),
            "::",
            stringify!(it_interval)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<itimerval>())).it_value as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(itimerval),
            "::",
            stringify!(it_value)
        )
    );
}
pub type __itimer_which_t = libc::c_int;
extern "C" {
    pub fn getitimer(__which: __itimer_which_t, __value: *mut itimerval) -> libc::c_int;
}
extern "C" {
    pub fn setitimer(
        __which: __itimer_which_t,
        __new: *const itimerval,
        __old: *mut itimerval,
    ) -> libc::c_int;
}
extern "C" {
    pub fn utimes(__file: *const libc::c_char, __tvp: *const timeval) -> libc::c_int;
}
extern "C" {
    pub fn lutimes(__file: *const libc::c_char, __tvp: *const timeval) -> libc::c_int;
}
extern "C" {
    pub fn futimes(__fd: libc::c_int, __tvp: *const timeval) -> libc::c_int;
}
pub type int_least8_t = __int_least8_t;
pub type int_least16_t = __int_least16_t;
pub type int_least32_t = __int_least32_t;
pub type int_least64_t = __int_least64_t;
pub type uint_least8_t = __uint_least8_t;
pub type uint_least16_t = __uint_least16_t;
pub type uint_least32_t = __uint_least32_t;
pub type uint_least64_t = __uint_least64_t;
pub type int_fast8_t = libc::c_schar;
pub type int_fast16_t = libc::c_long;
pub type int_fast32_t = libc::c_long;
pub type int_fast64_t = libc::c_long;
pub type uint_fast8_t = libc::c_uchar;
pub type uint_fast16_t = libc::c_ulong;
pub type uint_fast32_t = libc::c_ulong;
pub type uint_fast64_t = libc::c_ulong;
pub type intmax_t = __intmax_t;
pub type uintmax_t = __uintmax_t;
pub type __gwchar_t = libc::c_int;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct imaxdiv_t {
    pub quot: libc::c_long,
    pub rem: libc::c_long,
}
#[test]
fn bindgen_test_layout_imaxdiv_t() {
    assert_eq!(
        ::core::mem::size_of::<imaxdiv_t>(),
        16usize,
        concat!("Size of: ", stringify!(imaxdiv_t))
    );
    assert_eq!(
        ::core::mem::align_of::<imaxdiv_t>(),
        8usize,
        concat!("Alignment of ", stringify!(imaxdiv_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<imaxdiv_t>())).quot as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(imaxdiv_t),
            "::",
            stringify!(quot)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<imaxdiv_t>())).rem as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(imaxdiv_t),
            "::",
            stringify!(rem)
        )
    );
}
extern "C" {
    pub fn imaxabs(__n: intmax_t) -> intmax_t;
}
extern "C" {
    pub fn imaxdiv(__numer: intmax_t, __denom: intmax_t) -> imaxdiv_t;
}
extern "C" {
    pub fn strtoimax(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> intmax_t;
}
extern "C" {
    pub fn strtoumax(
        __nptr: *const libc::c_char,
        __endptr: *mut *mut libc::c_char,
        __base: libc::c_int,
    ) -> uintmax_t;
}
extern "C" {
    pub fn wcstoimax(
        __nptr: *const __gwchar_t,
        __endptr: *mut *mut __gwchar_t,
        __base: libc::c_int,
    ) -> intmax_t;
}
extern "C" {
    pub fn wcstoumax(
        __nptr: *const __gwchar_t,
        __endptr: *mut *mut __gwchar_t,
        __base: libc::c_int,
    ) -> uintmax_t;
}
pub const LibRaw_open_flags_LIBRAW_OPEN_BIGFILE: LibRaw_open_flags = 1;
pub const LibRaw_open_flags_LIBRAW_OPEN_FILE: LibRaw_open_flags = 2;
pub type LibRaw_open_flags = libc::c_uint;
pub const LibRaw_openbayer_patterns_LIBRAW_OPENBAYER_RGGB: LibRaw_openbayer_patterns = 148;
pub const LibRaw_openbayer_patterns_LIBRAW_OPENBAYER_BGGR: LibRaw_openbayer_patterns = 22;
pub const LibRaw_openbayer_patterns_LIBRAW_OPENBAYER_GRBG: LibRaw_openbayer_patterns = 97;
pub const LibRaw_openbayer_patterns_LIBRAW_OPENBAYER_GBRG: LibRaw_openbayer_patterns = 73;
pub type LibRaw_openbayer_patterns = libc::c_uint;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_FORWARDMATRIX: LibRaw_dngfields_marks = 1;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_ILLUMINANT: LibRaw_dngfields_marks = 2;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_COLORMATRIX: LibRaw_dngfields_marks = 4;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_CALIBRATION: LibRaw_dngfields_marks = 8;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_ANALOGBALANCE: LibRaw_dngfields_marks = 16;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_BLACK: LibRaw_dngfields_marks = 32;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_WHITE: LibRaw_dngfields_marks = 64;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_OPCODE2: LibRaw_dngfields_marks = 128;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_LINTABLE: LibRaw_dngfields_marks = 256;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_CROPORIGIN: LibRaw_dngfields_marks = 512;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_CROPSIZE: LibRaw_dngfields_marks = 1024;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_PREVIEWCS: LibRaw_dngfields_marks = 2048;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_ASSHOTNEUTRAL: LibRaw_dngfields_marks = 4096;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_BASELINEEXPOSURE: LibRaw_dngfields_marks = 8192;
pub const LibRaw_dngfields_marks_LIBRAW_DNGFM_LINEARRESPONSELIMIT: LibRaw_dngfields_marks = 16384;
pub type LibRaw_dngfields_marks = libc::c_uint;
pub const LibRaw_As_Shot_WB_Applied_codes_LIBRAW_ASWB_APPLIED: LibRaw_As_Shot_WB_Applied_codes = 1;
pub const LibRaw_As_Shot_WB_Applied_codes_LIBRAW_ASWB_CANON: LibRaw_As_Shot_WB_Applied_codes = 2;
pub const LibRaw_As_Shot_WB_Applied_codes_LIBRAW_ASWB_NIKON: LibRaw_As_Shot_WB_Applied_codes = 4;
pub const LibRaw_As_Shot_WB_Applied_codes_LIBRAW_ASWB_NIKON_SRAW: LibRaw_As_Shot_WB_Applied_codes =
    8;
pub const LibRaw_As_Shot_WB_Applied_codes_LIBRAW_ASWB_PENTAX: LibRaw_As_Shot_WB_Applied_codes = 16;
pub type LibRaw_As_Shot_WB_Applied_codes = libc::c_uint;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_UNKNOWN: LibRaw_ExifTagTypes = 0;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_BYTE: LibRaw_ExifTagTypes = 1;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_ASCII: LibRaw_ExifTagTypes = 2;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SHORT: LibRaw_ExifTagTypes = 3;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_LONG: LibRaw_ExifTagTypes = 4;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_RATIONAL: LibRaw_ExifTagTypes = 5;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SBYTE: LibRaw_ExifTagTypes = 6;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_UNDEFINED: LibRaw_ExifTagTypes = 7;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SSHORT: LibRaw_ExifTagTypes = 8;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SLONG: LibRaw_ExifTagTypes = 9;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SRATIONAL: LibRaw_ExifTagTypes = 10;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_FLOAT: LibRaw_ExifTagTypes = 11;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_DOUBLE: LibRaw_ExifTagTypes = 12;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_IFD: LibRaw_ExifTagTypes = 13;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_UNICODE: LibRaw_ExifTagTypes = 14;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_COMPLEX: LibRaw_ExifTagTypes = 15;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_LONG8: LibRaw_ExifTagTypes = 16;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_SLONG8: LibRaw_ExifTagTypes = 17;
pub const LibRaw_ExifTagTypes_LIBRAW_EXIFTAG_TYPE_IFD8: LibRaw_ExifTagTypes = 18;
pub type LibRaw_ExifTagTypes = libc::c_uint;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Unknown: LibRaw_whitebalance_code = 0;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Daylight: LibRaw_whitebalance_code = 1;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Fluorescent: LibRaw_whitebalance_code = 2;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Tungsten: LibRaw_whitebalance_code = 3;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Flash: LibRaw_whitebalance_code = 4;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FineWeather: LibRaw_whitebalance_code = 9;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Cloudy: LibRaw_whitebalance_code = 10;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Shade: LibRaw_whitebalance_code = 11;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FL_D: LibRaw_whitebalance_code = 12;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FL_N: LibRaw_whitebalance_code = 13;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FL_W: LibRaw_whitebalance_code = 14;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FL_WW: LibRaw_whitebalance_code = 15;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FL_L: LibRaw_whitebalance_code = 16;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Ill_A: LibRaw_whitebalance_code = 17;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Ill_B: LibRaw_whitebalance_code = 18;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Ill_C: LibRaw_whitebalance_code = 19;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_D55: LibRaw_whitebalance_code = 20;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_D65: LibRaw_whitebalance_code = 21;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_D75: LibRaw_whitebalance_code = 22;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_D50: LibRaw_whitebalance_code = 23;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_StudioTungsten: LibRaw_whitebalance_code = 24;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Sunset: LibRaw_whitebalance_code = 64;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Underwater: LibRaw_whitebalance_code = 65;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_FluorescentHigh: LibRaw_whitebalance_code = 66;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_HT_Mercury: LibRaw_whitebalance_code = 67;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_AsShot: LibRaw_whitebalance_code = 81;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Auto: LibRaw_whitebalance_code = 82;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom: LibRaw_whitebalance_code = 83;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Auto1: LibRaw_whitebalance_code = 85;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Auto2: LibRaw_whitebalance_code = 86;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Auto3: LibRaw_whitebalance_code = 87;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Auto4: LibRaw_whitebalance_code = 88;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom1: LibRaw_whitebalance_code = 90;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom2: LibRaw_whitebalance_code = 91;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom3: LibRaw_whitebalance_code = 92;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom4: LibRaw_whitebalance_code = 93;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom5: LibRaw_whitebalance_code = 94;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Custom6: LibRaw_whitebalance_code = 95;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_PC_Set1: LibRaw_whitebalance_code = 96;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_PC_Set2: LibRaw_whitebalance_code = 97;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_PC_Set3: LibRaw_whitebalance_code = 98;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_PC_Set4: LibRaw_whitebalance_code = 99;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_PC_Set5: LibRaw_whitebalance_code = 100;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Measured: LibRaw_whitebalance_code = 110;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_BW: LibRaw_whitebalance_code = 120;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Kelvin: LibRaw_whitebalance_code = 254;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_Other: LibRaw_whitebalance_code = 255;
pub const LibRaw_whitebalance_code_LIBRAW_WBI_None: LibRaw_whitebalance_code = 65535;
pub type LibRaw_whitebalance_code = libc::c_uint;
pub const LibRaw_MultiExposure_related_LIBRAW_ME_NONE: LibRaw_MultiExposure_related = 0;
pub const LibRaw_MultiExposure_related_LIBRAW_ME_SIMPLE: LibRaw_MultiExposure_related = 1;
pub const LibRaw_MultiExposure_related_LIBRAW_ME_OVERLAY: LibRaw_MultiExposure_related = 2;
pub const LibRaw_MultiExposure_related_LIBRAW_ME_HDR: LibRaw_MultiExposure_related = 3;
pub type LibRaw_MultiExposure_related = libc::c_uint;
pub const LibRaw_dng_processing_LIBRAW_DNG_NONE: LibRaw_dng_processing = 0;
pub const LibRaw_dng_processing_LIBRAW_DNG_FLOAT: LibRaw_dng_processing = 1;
pub const LibRaw_dng_processing_LIBRAW_DNG_LINEAR: LibRaw_dng_processing = 2;
pub const LibRaw_dng_processing_LIBRAW_DNG_DEFLATE: LibRaw_dng_processing = 4;
pub const LibRaw_dng_processing_LIBRAW_DNG_XTRANS: LibRaw_dng_processing = 8;
pub const LibRaw_dng_processing_LIBRAW_DNG_OTHER: LibRaw_dng_processing = 16;
pub const LibRaw_dng_processing_LIBRAW_DNG_8BIT: LibRaw_dng_processing = 32;
pub const LibRaw_dng_processing_LIBRAW_DNG_ALL: LibRaw_dng_processing = 59;
pub const LibRaw_dng_processing_LIBRAW_DNG_DEFAULT: LibRaw_dng_processing = 39;
pub type LibRaw_dng_processing = libc::c_uint;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_RAWSPEED: LibRaw_runtime_capabilities = 1;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_DNGSDK: LibRaw_runtime_capabilities = 2;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_GPRSDK: LibRaw_runtime_capabilities = 4;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_UNICODEPATHS: LibRaw_runtime_capabilities = 8;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_X3FTOOLS: LibRaw_runtime_capabilities = 16;
pub const LibRaw_runtime_capabilities_LIBRAW_CAPS_RPI6BY9: LibRaw_runtime_capabilities = 32;
pub type LibRaw_runtime_capabilities = libc::c_uint;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_NotFound: LibRaw_colorspace = 0;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_sRGB: LibRaw_colorspace = 1;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_AdobeRGB: LibRaw_colorspace = 2;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_WideGamutRGB: LibRaw_colorspace = 3;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_ProPhotoRGB: LibRaw_colorspace = 4;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_ICC: LibRaw_colorspace = 5;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_Uncalibrated: LibRaw_colorspace = 6;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_CameraLinearUniWB: LibRaw_colorspace = 7;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_CameraLinear: LibRaw_colorspace = 8;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_CameraGammaUniWB: LibRaw_colorspace = 9;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_CameraGamma: LibRaw_colorspace = 10;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_MonochromeLinear: LibRaw_colorspace = 11;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_MonochromeGamma: LibRaw_colorspace = 12;
pub const LibRaw_colorspace_LIBRAW_COLORSPACE_Unknown: LibRaw_colorspace = 255;
pub type LibRaw_colorspace = libc::c_uint;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Unknown: LibRaw_cameramaker_index = 0;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Agfa: LibRaw_cameramaker_index = 1;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Alcatel: LibRaw_cameramaker_index = 2;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Apple: LibRaw_cameramaker_index = 3;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Aptina: LibRaw_cameramaker_index = 4;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_AVT: LibRaw_cameramaker_index = 5;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Baumer: LibRaw_cameramaker_index = 6;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Broadcom: LibRaw_cameramaker_index = 7;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Canon: LibRaw_cameramaker_index = 8;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Casio: LibRaw_cameramaker_index = 9;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_CINE: LibRaw_cameramaker_index = 10;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Clauss: LibRaw_cameramaker_index = 11;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Contax: LibRaw_cameramaker_index = 12;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Creative: LibRaw_cameramaker_index = 13;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_DJI: LibRaw_cameramaker_index = 14;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_DXO: LibRaw_cameramaker_index = 15;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Epson: LibRaw_cameramaker_index = 16;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Foculus: LibRaw_cameramaker_index = 17;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Fujifilm: LibRaw_cameramaker_index = 18;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Generic: LibRaw_cameramaker_index = 19;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Gione: LibRaw_cameramaker_index = 20;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_GITUP: LibRaw_cameramaker_index = 21;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Google: LibRaw_cameramaker_index = 22;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_GoPro: LibRaw_cameramaker_index = 23;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Hasselblad: LibRaw_cameramaker_index = 24;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_HTC: LibRaw_cameramaker_index = 25;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_I_Mobile: LibRaw_cameramaker_index = 26;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Imacon: LibRaw_cameramaker_index = 27;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_JK_Imaging: LibRaw_cameramaker_index = 28;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Kodak: LibRaw_cameramaker_index = 29;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Konica: LibRaw_cameramaker_index = 30;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Leaf: LibRaw_cameramaker_index = 31;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Leica: LibRaw_cameramaker_index = 32;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Lenovo: LibRaw_cameramaker_index = 33;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_LG: LibRaw_cameramaker_index = 34;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Logitech: LibRaw_cameramaker_index = 35;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Mamiya: LibRaw_cameramaker_index = 36;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Matrix: LibRaw_cameramaker_index = 37;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Meizu: LibRaw_cameramaker_index = 38;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Micron: LibRaw_cameramaker_index = 39;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Minolta: LibRaw_cameramaker_index = 40;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Motorola: LibRaw_cameramaker_index = 41;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_NGM: LibRaw_cameramaker_index = 42;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Nikon: LibRaw_cameramaker_index = 43;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Nokia: LibRaw_cameramaker_index = 44;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Olympus: LibRaw_cameramaker_index = 45;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_OmniVison: LibRaw_cameramaker_index = 46;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Panasonic: LibRaw_cameramaker_index = 47;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Parrot: LibRaw_cameramaker_index = 48;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Pentax: LibRaw_cameramaker_index = 49;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_PhaseOne: LibRaw_cameramaker_index = 50;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_PhotoControl: LibRaw_cameramaker_index = 51;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Photron: LibRaw_cameramaker_index = 52;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Pixelink: LibRaw_cameramaker_index = 53;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Polaroid: LibRaw_cameramaker_index = 54;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_RED: LibRaw_cameramaker_index = 55;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Ricoh: LibRaw_cameramaker_index = 56;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Rollei: LibRaw_cameramaker_index = 57;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_RoverShot: LibRaw_cameramaker_index = 58;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Samsung: LibRaw_cameramaker_index = 59;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Sigma: LibRaw_cameramaker_index = 60;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Sinar: LibRaw_cameramaker_index = 61;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_SMaL: LibRaw_cameramaker_index = 62;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Sony: LibRaw_cameramaker_index = 63;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_ST_Micro: LibRaw_cameramaker_index = 64;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_THL: LibRaw_cameramaker_index = 65;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_VLUU: LibRaw_cameramaker_index = 66;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Xiaomi: LibRaw_cameramaker_index = 67;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_XIAOYI: LibRaw_cameramaker_index = 68;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_YI: LibRaw_cameramaker_index = 69;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Yuneec: LibRaw_cameramaker_index = 70;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_Zeiss: LibRaw_cameramaker_index = 71;
pub const LibRaw_cameramaker_index_LIBRAW_CAMERAMAKER_TheLastOne: LibRaw_cameramaker_index = 72;
pub type LibRaw_cameramaker_index = libc::c_uint;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Unknown: LibRaw_camera_mounts = 0;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Alpa: LibRaw_camera_mounts = 1;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_C: LibRaw_camera_mounts = 2;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Canon_EF_M: LibRaw_camera_mounts = 3;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Canon_EF_S: LibRaw_camera_mounts = 4;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Canon_EF: LibRaw_camera_mounts = 5;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Canon_RF: LibRaw_camera_mounts = 6;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Contax_N: LibRaw_camera_mounts = 7;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Contax645: LibRaw_camera_mounts = 8;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_FT: LibRaw_camera_mounts = 9;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_mFT: LibRaw_camera_mounts = 10;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Fuji_GF: LibRaw_camera_mounts = 11;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Fuji_GX: LibRaw_camera_mounts = 12;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Fuji_X: LibRaw_camera_mounts = 13;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Hasselblad_H: LibRaw_camera_mounts = 14;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Hasselblad_V: LibRaw_camera_mounts = 15;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Hasselblad_XCD: LibRaw_camera_mounts = 16;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Leica_M: LibRaw_camera_mounts = 17;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Leica_R: LibRaw_camera_mounts = 18;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Leica_S: LibRaw_camera_mounts = 19;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Leica_SL: LibRaw_camera_mounts = 20;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Leica_TL: LibRaw_camera_mounts = 21;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_LPS_L: LibRaw_camera_mounts = 22;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Mamiya67: LibRaw_camera_mounts = 23;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Mamiya645: LibRaw_camera_mounts = 24;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Minolta_A: LibRaw_camera_mounts = 25;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Nikon_CX: LibRaw_camera_mounts = 26;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Nikon_F: LibRaw_camera_mounts = 27;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Nikon_Z: LibRaw_camera_mounts = 28;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Pentax_645: LibRaw_camera_mounts = 29;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Pentax_K: LibRaw_camera_mounts = 30;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Pentax_Q: LibRaw_camera_mounts = 31;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_RicohModule: LibRaw_camera_mounts = 32;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Rollei_bayonet: LibRaw_camera_mounts = 33;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Samsung_NX_M: LibRaw_camera_mounts = 34;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Samsung_NX: LibRaw_camera_mounts = 35;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Sigma_X3F: LibRaw_camera_mounts = 36;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_Sony_E: LibRaw_camera_mounts = 37;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_LF: LibRaw_camera_mounts = 38;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_DigitalBack: LibRaw_camera_mounts = 39;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_FixedLens: LibRaw_camera_mounts = 40;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_IL_UM: LibRaw_camera_mounts = 41;
pub const LibRaw_camera_mounts_LIBRAW_MOUNT_TheLastOne: LibRaw_camera_mounts = 42;
pub type LibRaw_camera_mounts = libc::c_uint;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_Unknown: LibRaw_camera_formats = 0;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_APSC: LibRaw_camera_formats = 1;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_FF: LibRaw_camera_formats = 2;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_MF: LibRaw_camera_formats = 3;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_APSH: LibRaw_camera_formats = 4;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_1INCH: LibRaw_camera_formats = 5;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_1div2p3INCH: LibRaw_camera_formats = 6;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_1div1p7INCH: LibRaw_camera_formats = 7;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_FT: LibRaw_camera_formats = 8;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_CROP645: LibRaw_camera_formats = 9;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_LeicaS: LibRaw_camera_formats = 10;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_645: LibRaw_camera_formats = 11;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_66: LibRaw_camera_formats = 12;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_69: LibRaw_camera_formats = 13;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_LF: LibRaw_camera_formats = 14;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_Leica_DMR: LibRaw_camera_formats = 15;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_67: LibRaw_camera_formats = 16;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_SigmaAPSC: LibRaw_camera_formats = 17;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_SigmaMerrill: LibRaw_camera_formats = 18;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_SigmaAPSH: LibRaw_camera_formats = 19;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_3648: LibRaw_camera_formats = 20;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_68: LibRaw_camera_formats = 21;
pub const LibRaw_camera_formats_LIBRAW_FORMAT_TheLastOne: LibRaw_camera_formats = 22;
pub type LibRaw_camera_formats = libc::c_uint;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_UNKNOWN: LibRawImageAspects = 0;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_3to2: LibRawImageAspects = 1;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_1to1: LibRawImageAspects = 2;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_4to3: LibRawImageAspects = 3;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_16to9: LibRawImageAspects = 4;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_5to4: LibRawImageAspects = 5;
pub const LibRawImageAspects_LIBRAW_IMAGE_ASPECT_OTHER: LibRawImageAspects = 6;
pub type LibRawImageAspects = libc::c_uint;
pub const LibRaw_lens_focal_types_LIBRAW_FT_UNDEFINED: LibRaw_lens_focal_types = 0;
pub const LibRaw_lens_focal_types_LIBRAW_FT_PRIME_LENS: LibRaw_lens_focal_types = 1;
pub const LibRaw_lens_focal_types_LIBRAW_FT_ZOOM_LENS: LibRaw_lens_focal_types = 2;
pub const LibRaw_lens_focal_types_LIBRAW_FT_ZOOM_LENS_CONSTANT_APERTURE: LibRaw_lens_focal_types =
    3;
pub const LibRaw_lens_focal_types_LIBRAW_FT_ZOOM_LENS_VARIABLE_APERTURE: LibRaw_lens_focal_types =
    4;
pub type LibRaw_lens_focal_types = libc::c_uint;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_UNDEFINED: LibRaw_Canon_RecordModes = 0;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_JPEG: LibRaw_Canon_RecordModes = 1;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CRW_THM: LibRaw_Canon_RecordModes = 2;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_AVI_THM: LibRaw_Canon_RecordModes = 3;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_TIF: LibRaw_Canon_RecordModes = 4;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_TIF_JPEG: LibRaw_Canon_RecordModes = 5;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CR2: LibRaw_Canon_RecordModes = 6;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CR2_JPEG: LibRaw_Canon_RecordModes = 7;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_UNKNOWN: LibRaw_Canon_RecordModes = 8;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_MOV: LibRaw_Canon_RecordModes = 9;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_MP4: LibRaw_Canon_RecordModes = 10;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CRM: LibRaw_Canon_RecordModes = 11;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CR3: LibRaw_Canon_RecordModes = 12;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CR3_JPEG: LibRaw_Canon_RecordModes = 13;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_HEIF: LibRaw_Canon_RecordModes = 14;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_CR3_HEIF: LibRaw_Canon_RecordModes = 15;
pub const LibRaw_Canon_RecordModes_LIBRAW_Canon_RecordMode_TheLastOne: LibRaw_Canon_RecordModes =
    16;
pub type LibRaw_Canon_RecordModes = libc::c_uint;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_DSC: LibRaw_sony_cameratypes = 1;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_DSLR: LibRaw_sony_cameratypes = 2;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_NEX: LibRaw_sony_cameratypes = 3;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_SLT: LibRaw_sony_cameratypes = 4;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_ILCE: LibRaw_sony_cameratypes = 5;
pub const LibRaw_sony_cameratypes_LIBRAW_SONY_ILCA: LibRaw_sony_cameratypes = 6;
pub type LibRaw_sony_cameratypes = libc::c_uint;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_UnknownSensor: LibRaw_KodakSensors = 0;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M1: LibRaw_KodakSensors = 1;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M15: LibRaw_KodakSensors = 2;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M16: LibRaw_KodakSensors = 3;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M17: LibRaw_KodakSensors = 4;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M2: LibRaw_KodakSensors = 5;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M23: LibRaw_KodakSensors = 6;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M24: LibRaw_KodakSensors = 7;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M3: LibRaw_KodakSensors = 8;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M5: LibRaw_KodakSensors = 9;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M6: LibRaw_KodakSensors = 10;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_C14: LibRaw_KodakSensors = 11;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_X14: LibRaw_KodakSensors = 12;
pub const LibRaw_KodakSensors_LIBRAW_Kodak_M11: LibRaw_KodakSensors = 13;
pub type LibRaw_KodakSensors = libc::c_uint;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_Unknown: LibRaw_HasselbladFormatCodes = 0;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_3FR: LibRaw_HasselbladFormatCodes = 1;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_FFF: LibRaw_HasselbladFormatCodes = 2;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_Imacon: LibRaw_HasselbladFormatCodes = 3;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_HasselbladDNG: LibRaw_HasselbladFormatCodes = 4;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_AdobeDNG: LibRaw_HasselbladFormatCodes = 5;
pub const LibRaw_HasselbladFormatCodes_LIBRAW_HF_AdobeDNG_fromPhocusDNG:
    LibRaw_HasselbladFormatCodes = 6;
pub type LibRaw_HasselbladFormatCodes = libc::c_uint;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_NONE: LibRaw_processing_options = 0;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_BASEONLY: LibRaw_processing_options =
    1;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_DELTAONLY:
    LibRaw_processing_options = 2;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_DELTAZEROBASE:
    LibRaw_processing_options = 4;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_DELTATOVALUE:
    LibRaw_processing_options = 8;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SONYARW2_ALLFLAGS: LibRaw_processing_options =
    15;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DP2Q_INTERPOLATERG:
    LibRaw_processing_options = 16;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DP2Q_INTERPOLATEAF:
    LibRaw_processing_options = 32;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_PENTAX_PS_ALLFRAMES:
    LibRaw_processing_options = 64;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_CONVERTFLOAT_TO_INT:
    LibRaw_processing_options = 128;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SRAW_NO_RGB: LibRaw_processing_options = 256;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SRAW_NO_INTERPOLATE:
    LibRaw_processing_options = 512;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_ARQ_SKIP_CHANNEL_SWAP:
    LibRaw_processing_options = 1024;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_NO_ROTATE_FOR_KODAK_THUMBNAILS:
    LibRaw_processing_options = 2048;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_USE_DNG_DEFAULT_CROP:
    LibRaw_processing_options = 4096;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_USE_PPM16_THUMBS: LibRaw_processing_options =
    8192;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_SKIP_MAKERNOTES: LibRaw_processing_options =
    16384;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DONT_CHECK_DNG_ILLUMINANT:
    LibRaw_processing_options = 32768;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNGSDK_ZEROCOPY: LibRaw_processing_options =
    65536;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_ZEROFILTERS_FOR_MONOCHROMETIFFS:
    LibRaw_processing_options = 131072;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_ADD_ENHANCED: LibRaw_processing_options =
    262144;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_ADD_PREVIEWS: LibRaw_processing_options =
    524288;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_PREFER_LARGEST_IMAGE:
    LibRaw_processing_options = 1048576;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_STAGE2: LibRaw_processing_options =
    2097152;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_STAGE3: LibRaw_processing_options =
    4194304;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_ALLOWSIZECHANGE:
    LibRaw_processing_options = 8388608;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_DNG_DISABLEWBADJUST:
    LibRaw_processing_options = 16777216;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_PROVIDE_NONSTANDARD_WB:
    LibRaw_processing_options = 33554432;
pub const LibRaw_processing_options_LIBRAW_PROCESSING_CAMERAWB_FALLBACK_TO_DAYLIGHT:
    LibRaw_processing_options = 67108864;
pub type LibRaw_processing_options = libc::c_uint;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_HASCURVE: LibRaw_decoder_flags = 16;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_SONYARW2: LibRaw_decoder_flags = 32;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_TRYRAWSPEED: LibRaw_decoder_flags = 64;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_OWNALLOC: LibRaw_decoder_flags = 128;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_FIXEDMAXC: LibRaw_decoder_flags = 256;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_ADOBECOPYPIXEL: LibRaw_decoder_flags = 512;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_LEGACY_WITH_MARGINS: LibRaw_decoder_flags = 1024;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_3CHANNEL: LibRaw_decoder_flags = 2048;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_SINAR4SHOT: LibRaw_decoder_flags = 2048;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_FLATDATA: LibRaw_decoder_flags = 4096;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_FLAT_BG2_SWAPPED: LibRaw_decoder_flags = 8192;
pub const LibRaw_decoder_flags_LIBRAW_DECODER_NOTSET: LibRaw_decoder_flags = 32768;
pub type LibRaw_decoder_flags = libc::c_uint;
pub const LibRaw_constructor_flags_LIBRAW_OPTIONS_NONE: LibRaw_constructor_flags = 0;
pub const LibRaw_constructor_flags_LIBRAW_OPIONS_NO_MEMERR_CALLBACK: LibRaw_constructor_flags = 1;
pub const LibRaw_constructor_flags_LIBRAW_OPIONS_NO_DATAERR_CALLBACK: LibRaw_constructor_flags = 2;
pub type LibRaw_constructor_flags = libc::c_uint;
pub const LibRaw_warnings_LIBRAW_WARN_NONE: LibRaw_warnings = 0;
pub const LibRaw_warnings_LIBRAW_WARN_BAD_CAMERA_WB: LibRaw_warnings = 4;
pub const LibRaw_warnings_LIBRAW_WARN_NO_METADATA: LibRaw_warnings = 8;
pub const LibRaw_warnings_LIBRAW_WARN_NO_JPEGLIB: LibRaw_warnings = 16;
pub const LibRaw_warnings_LIBRAW_WARN_NO_EMBEDDED_PROFILE: LibRaw_warnings = 32;
pub const LibRaw_warnings_LIBRAW_WARN_NO_INPUT_PROFILE: LibRaw_warnings = 64;
pub const LibRaw_warnings_LIBRAW_WARN_BAD_OUTPUT_PROFILE: LibRaw_warnings = 128;
pub const LibRaw_warnings_LIBRAW_WARN_NO_BADPIXELMAP: LibRaw_warnings = 256;
pub const LibRaw_warnings_LIBRAW_WARN_BAD_DARKFRAME_FILE: LibRaw_warnings = 512;
pub const LibRaw_warnings_LIBRAW_WARN_BAD_DARKFRAME_DIM: LibRaw_warnings = 1024;
pub const LibRaw_warnings_LIBRAW_WARN_NO_JASPER: LibRaw_warnings = 2048;
pub const LibRaw_warnings_LIBRAW_WARN_RAWSPEED_PROBLEM: LibRaw_warnings = 4096;
pub const LibRaw_warnings_LIBRAW_WARN_RAWSPEED_UNSUPPORTED: LibRaw_warnings = 8192;
pub const LibRaw_warnings_LIBRAW_WARN_RAWSPEED_PROCESSED: LibRaw_warnings = 16384;
pub const LibRaw_warnings_LIBRAW_WARN_FALLBACK_TO_AHD: LibRaw_warnings = 32768;
pub const LibRaw_warnings_LIBRAW_WARN_PARSEFUJI_PROCESSED: LibRaw_warnings = 65536;
pub const LibRaw_warnings_LIBRAW_WARN_DNGSDK_PROCESSED: LibRaw_warnings = 131072;
pub const LibRaw_warnings_LIBRAW_WARN_DNG_IMAGES_REORDERED: LibRaw_warnings = 262144;
pub const LibRaw_warnings_LIBRAW_WARN_DNG_STAGE2_APPLIED: LibRaw_warnings = 524288;
pub const LibRaw_warnings_LIBRAW_WARN_DNG_STAGE3_APPLIED: LibRaw_warnings = 1048576;
pub type LibRaw_warnings = libc::c_uint;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_NONE: LibRaw_exceptions = 0;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_ALLOC: LibRaw_exceptions = 1;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_DECODE_RAW: LibRaw_exceptions = 2;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_DECODE_JPEG: LibRaw_exceptions = 3;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_IO_EOF: LibRaw_exceptions = 4;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_IO_CORRUPT: LibRaw_exceptions = 5;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_CANCELLED_BY_CALLBACK: LibRaw_exceptions = 6;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_BAD_CROP: LibRaw_exceptions = 7;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_IO_BADFILE: LibRaw_exceptions = 8;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_DECODE_JPEG2000: LibRaw_exceptions = 9;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_TOOBIG: LibRaw_exceptions = 10;
pub const LibRaw_exceptions_LIBRAW_EXCEPTION_MEMPOOL: LibRaw_exceptions = 11;
pub type LibRaw_exceptions = libc::c_uint;
pub const LibRaw_progress_LIBRAW_PROGRESS_START: LibRaw_progress = 0;
pub const LibRaw_progress_LIBRAW_PROGRESS_OPEN: LibRaw_progress = 1;
pub const LibRaw_progress_LIBRAW_PROGRESS_IDENTIFY: LibRaw_progress = 2;
pub const LibRaw_progress_LIBRAW_PROGRESS_SIZE_ADJUST: LibRaw_progress = 4;
pub const LibRaw_progress_LIBRAW_PROGRESS_LOAD_RAW: LibRaw_progress = 8;
pub const LibRaw_progress_LIBRAW_PROGRESS_RAW2_IMAGE: LibRaw_progress = 16;
pub const LibRaw_progress_LIBRAW_PROGRESS_REMOVE_ZEROES: LibRaw_progress = 32;
pub const LibRaw_progress_LIBRAW_PROGRESS_BAD_PIXELS: LibRaw_progress = 64;
pub const LibRaw_progress_LIBRAW_PROGRESS_DARK_FRAME: LibRaw_progress = 128;
pub const LibRaw_progress_LIBRAW_PROGRESS_FOVEON_INTERPOLATE: LibRaw_progress = 256;
pub const LibRaw_progress_LIBRAW_PROGRESS_SCALE_COLORS: LibRaw_progress = 512;
pub const LibRaw_progress_LIBRAW_PROGRESS_PRE_INTERPOLATE: LibRaw_progress = 1024;
pub const LibRaw_progress_LIBRAW_PROGRESS_INTERPOLATE: LibRaw_progress = 2048;
pub const LibRaw_progress_LIBRAW_PROGRESS_MIX_GREEN: LibRaw_progress = 4096;
pub const LibRaw_progress_LIBRAW_PROGRESS_MEDIAN_FILTER: LibRaw_progress = 8192;
pub const LibRaw_progress_LIBRAW_PROGRESS_HIGHLIGHTS: LibRaw_progress = 16384;
pub const LibRaw_progress_LIBRAW_PROGRESS_FUJI_ROTATE: LibRaw_progress = 32768;
pub const LibRaw_progress_LIBRAW_PROGRESS_FLIP: LibRaw_progress = 65536;
pub const LibRaw_progress_LIBRAW_PROGRESS_APPLY_PROFILE: LibRaw_progress = 131072;
pub const LibRaw_progress_LIBRAW_PROGRESS_CONVERT_RGB: LibRaw_progress = 262144;
pub const LibRaw_progress_LIBRAW_PROGRESS_STRETCH: LibRaw_progress = 524288;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE20: LibRaw_progress = 1048576;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE21: LibRaw_progress = 2097152;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE22: LibRaw_progress = 4194304;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE23: LibRaw_progress = 8388608;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE24: LibRaw_progress = 16777216;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE25: LibRaw_progress = 33554432;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE26: LibRaw_progress = 67108864;
pub const LibRaw_progress_LIBRAW_PROGRESS_STAGE27: LibRaw_progress = 134217728;
pub const LibRaw_progress_LIBRAW_PROGRESS_THUMB_LOAD: LibRaw_progress = 268435456;
pub const LibRaw_progress_LIBRAW_PROGRESS_TRESERVED1: LibRaw_progress = 536870912;
pub const LibRaw_progress_LIBRAW_PROGRESS_TRESERVED2: LibRaw_progress = 1073741824;
pub type LibRaw_progress = libc::c_uint;
pub const LibRaw_errors_LIBRAW_SUCCESS: LibRaw_errors = 0;
pub const LibRaw_errors_LIBRAW_UNSPECIFIED_ERROR: LibRaw_errors = -1;
pub const LibRaw_errors_LIBRAW_FILE_UNSUPPORTED: LibRaw_errors = -2;
pub const LibRaw_errors_LIBRAW_REQUEST_FOR_NONEXISTENT_IMAGE: LibRaw_errors = -3;
pub const LibRaw_errors_LIBRAW_OUT_OF_ORDER_CALL: LibRaw_errors = -4;
pub const LibRaw_errors_LIBRAW_NO_THUMBNAIL: LibRaw_errors = -5;
pub const LibRaw_errors_LIBRAW_UNSUPPORTED_THUMBNAIL: LibRaw_errors = -6;
pub const LibRaw_errors_LIBRAW_INPUT_CLOSED: LibRaw_errors = -7;
pub const LibRaw_errors_LIBRAW_NOT_IMPLEMENTED: LibRaw_errors = -8;
pub const LibRaw_errors_LIBRAW_UNSUFFICIENT_MEMORY: LibRaw_errors = -100007;
pub const LibRaw_errors_LIBRAW_DATA_ERROR: LibRaw_errors = -100008;
pub const LibRaw_errors_LIBRAW_IO_ERROR: LibRaw_errors = -100009;
pub const LibRaw_errors_LIBRAW_CANCELLED_BY_CALLBACK: LibRaw_errors = -100010;
pub const LibRaw_errors_LIBRAW_BAD_CROP: LibRaw_errors = -100011;
pub const LibRaw_errors_LIBRAW_TOO_BIG: LibRaw_errors = -100012;
pub const LibRaw_errors_LIBRAW_MEMPOOL_OVERFLOW: LibRaw_errors = -100013;
pub type LibRaw_errors = libc::c_int;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_UNKNOWN: LibRaw_thumbnail_formats = 0;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_JPEG: LibRaw_thumbnail_formats = 1;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_BITMAP: LibRaw_thumbnail_formats = 2;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_BITMAP16: LibRaw_thumbnail_formats = 3;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_LAYER: LibRaw_thumbnail_formats = 4;
pub const LibRaw_thumbnail_formats_LIBRAW_THUMBNAIL_ROLLEI: LibRaw_thumbnail_formats = 5;
pub type LibRaw_thumbnail_formats = libc::c_uint;
pub const LibRaw_image_formats_LIBRAW_IMAGE_JPEG: LibRaw_image_formats = 1;
pub const LibRaw_image_formats_LIBRAW_IMAGE_BITMAP: LibRaw_image_formats = 2;
pub type LibRaw_image_formats = libc::c_uint;
pub type INT64 = libc::c_longlong;
pub type UINT64 = libc::c_ulonglong;
pub type uchar = libc::c_uchar;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_decoder_info_t {
    pub decoder_name: *const libc::c_char,
    pub decoder_flags: libc::c_uint,
}
#[test]
fn bindgen_test_layout_libraw_decoder_info_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_decoder_info_t>(),
        16usize,
        concat!("Size of: ", stringify!(libraw_decoder_info_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_decoder_info_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_decoder_info_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_decoder_info_t>())).decoder_name as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_decoder_info_t),
            "::",
            stringify!(decoder_name)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_decoder_info_t>())).decoder_flags as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_decoder_info_t),
            "::",
            stringify!(decoder_flags)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_internal_output_params_t {
    pub mix_green: libc::c_uint,
    pub raw_color: libc::c_uint,
    pub zero_is_bad: libc::c_uint,
    pub shrink: ushort,
    pub fuji_width: ushort,
}
#[test]
fn bindgen_test_layout_libraw_internal_output_params_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_internal_output_params_t>(),
        16usize,
        concat!("Size of: ", stringify!(libraw_internal_output_params_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_internal_output_params_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_internal_output_params_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_output_params_t>())).mix_green as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_output_params_t),
            "::",
            stringify!(mix_green)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_output_params_t>())).raw_color as *const _
                as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_output_params_t),
            "::",
            stringify!(raw_color)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_output_params_t>())).zero_is_bad as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_output_params_t),
            "::",
            stringify!(zero_is_bad)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_output_params_t>())).shrink as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_output_params_t),
            "::",
            stringify!(shrink)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_output_params_t>())).fuji_width as *const _
                as usize
        },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_output_params_t),
            "::",
            stringify!(fuji_width)
        )
    );
}
pub type memory_callback = ::core::option::Option<
    unsafe extern "C" fn(
        data: *mut libc::c_void,
        file: *const libc::c_char,
        where_: *const libc::c_char,
    ),
>;
pub type exif_parser_callback = ::core::option::Option<
    unsafe extern "C" fn(
        context: *mut libc::c_void,
        tag: libc::c_int,
        type_: libc::c_int,
        len: libc::c_int,
        ord: libc::c_uint,
        ifp: *mut libc::c_void,
        base: INT64,
    ),
>;
extern "C" {
    pub fn default_memory_callback(
        data: *mut libc::c_void,
        file: *const libc::c_char,
        where_: *const libc::c_char,
    );
}
pub type data_callback = ::core::option::Option<
    unsafe extern "C" fn(data: *mut libc::c_void, file: *const libc::c_char, offset: libc::c_int),
>;
extern "C" {
    pub fn default_data_callback(
        data: *mut libc::c_void,
        file: *const libc::c_char,
        offset: libc::c_int,
    );
}
pub type progress_callback = ::core::option::Option<
    unsafe extern "C" fn(
        data: *mut libc::c_void,
        stage: LibRaw_progress,
        iteration: libc::c_int,
        expected: libc::c_int,
    ) -> libc::c_int,
>;
pub type pre_identify_callback =
    ::core::option::Option<unsafe extern "C" fn(ctx: *mut libc::c_void) -> libc::c_int>;
pub type post_identify_callback =
    ::core::option::Option<unsafe extern "C" fn(ctx: *mut libc::c_void)>;
pub type process_step_callback =
    ::core::option::Option<unsafe extern "C" fn(ctx: *mut libc::c_void)>;
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_callbacks_t {
    pub mem_cb: memory_callback,
    pub memcb_data: *mut libc::c_void,
    pub data_cb: data_callback,
    pub datacb_data: *mut libc::c_void,
    pub progress_cb: progress_callback,
    pub progresscb_data: *mut libc::c_void,
    pub exif_cb: exif_parser_callback,
    pub exifparser_data: *mut libc::c_void,
    pub pre_identify_cb: pre_identify_callback,
    pub post_identify_cb: post_identify_callback,
    pub pre_subtractblack_cb: process_step_callback,
    pub pre_scalecolors_cb: process_step_callback,
    pub pre_preinterpolate_cb: process_step_callback,
    pub pre_interpolate_cb: process_step_callback,
    pub interpolate_bayer_cb: process_step_callback,
    pub interpolate_xtrans_cb: process_step_callback,
    pub post_interpolate_cb: process_step_callback,
    pub pre_converttorgb_cb: process_step_callback,
    pub post_converttorgb_cb: process_step_callback,
}
#[test]
fn bindgen_test_layout_libraw_callbacks_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_callbacks_t>(),
        152usize,
        concat!("Size of: ", stringify!(libraw_callbacks_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_callbacks_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_callbacks_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).mem_cb as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(mem_cb)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).memcb_data as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(memcb_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).data_cb as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(data_cb)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).datacb_data as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(datacb_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).progress_cb as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(progress_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).progresscb_data as *const _ as usize
        },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(progresscb_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_callbacks_t>())).exif_cb as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(exif_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).exifparser_data as *const _ as usize
        },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(exifparser_data)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_identify_cb as *const _ as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_identify_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).post_identify_cb as *const _ as usize
        },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(post_identify_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_subtractblack_cb as *const _
                as usize
        },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_subtractblack_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_scalecolors_cb as *const _ as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_scalecolors_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_preinterpolate_cb as *const _
                as usize
        },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_preinterpolate_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_interpolate_cb as *const _ as usize
        },
        104usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_interpolate_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).interpolate_bayer_cb as *const _
                as usize
        },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(interpolate_bayer_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).interpolate_xtrans_cb as *const _
                as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(interpolate_xtrans_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).post_interpolate_cb as *const _ as usize
        },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(post_interpolate_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).pre_converttorgb_cb as *const _ as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(pre_converttorgb_cb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_callbacks_t>())).post_converttorgb_cb as *const _
                as usize
        },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_callbacks_t),
            "::",
            stringify!(post_converttorgb_cb)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_processed_image_t {
    pub type_: LibRaw_image_formats,
    pub height: ushort,
    pub width: ushort,
    pub colors: ushort,
    pub bits: ushort,
    pub data_size: libc::c_uint,
    pub data: [libc::c_uchar; 1usize],
}
#[test]
fn bindgen_test_layout_libraw_processed_image_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_processed_image_t>(),
        20usize,
        concat!("Size of: ", stringify!(libraw_processed_image_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_processed_image_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_processed_image_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_processed_image_t>())).type_ as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(type_)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_processed_image_t>())).height as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(height)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_processed_image_t>())).width as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(width)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_processed_image_t>())).colors as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(colors)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_processed_image_t>())).bits as *const _ as usize },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(bits)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_processed_image_t>())).data_size as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(data_size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_processed_image_t>())).data as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_processed_image_t),
            "::",
            stringify!(data)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_iparams_t {
    pub guard: [libc::c_char; 4usize],
    pub make: [libc::c_char; 64usize],
    pub model: [libc::c_char; 64usize],
    pub software: [libc::c_char; 64usize],
    pub normalized_make: [libc::c_char; 64usize],
    pub normalized_model: [libc::c_char; 64usize],
    pub maker_index: libc::c_uint,
    pub raw_count: libc::c_uint,
    pub dng_version: libc::c_uint,
    pub is_foveon: libc::c_uint,
    pub colors: libc::c_int,
    pub filters: libc::c_uint,
    pub xtrans: [[libc::c_char; 6usize]; 6usize],
    pub xtrans_abs: [[libc::c_char; 6usize]; 6usize],
    pub cdesc: [libc::c_char; 5usize],
    pub xmplen: libc::c_uint,
    pub xmpdata: *mut libc::c_char,
}
#[test]
fn bindgen_test_layout_libraw_iparams_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_iparams_t>(),
        440usize,
        concat!("Size of: ", stringify!(libraw_iparams_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_iparams_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_iparams_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).guard as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(guard)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).make as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(make)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).model as *const _ as usize },
        68usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(model)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).software as *const _ as usize },
        132usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(software)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_iparams_t>())).normalized_make as *const _ as usize
        },
        196usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(normalized_make)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_iparams_t>())).normalized_model as *const _ as usize
        },
        260usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(normalized_model)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).maker_index as *const _ as usize },
        324usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(maker_index)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).raw_count as *const _ as usize },
        328usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(raw_count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).dng_version as *const _ as usize },
        332usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(dng_version)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).is_foveon as *const _ as usize },
        336usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(is_foveon)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).colors as *const _ as usize },
        340usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(colors)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).filters as *const _ as usize },
        344usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(filters)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).xtrans as *const _ as usize },
        348usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(xtrans)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).xtrans_abs as *const _ as usize },
        384usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(xtrans_abs)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).cdesc as *const _ as usize },
        420usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(cdesc)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).xmplen as *const _ as usize },
        428usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(xmplen)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_iparams_t>())).xmpdata as *const _ as usize },
        432usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_iparams_t),
            "::",
            stringify!(xmpdata)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_raw_inset_crop_t {
    pub cleft: ushort,
    pub ctop: ushort,
    pub cwidth: ushort,
    pub cheight: ushort,
    pub aspect: ushort,
}
#[test]
fn bindgen_test_layout_libraw_raw_inset_crop_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_raw_inset_crop_t>(),
        10usize,
        concat!("Size of: ", stringify!(libraw_raw_inset_crop_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_raw_inset_crop_t>(),
        2usize,
        concat!("Alignment of ", stringify!(libraw_raw_inset_crop_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_raw_inset_crop_t>())).cleft as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_raw_inset_crop_t),
            "::",
            stringify!(cleft)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_raw_inset_crop_t>())).ctop as *const _ as usize },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_raw_inset_crop_t),
            "::",
            stringify!(ctop)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_raw_inset_crop_t>())).cwidth as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_raw_inset_crop_t),
            "::",
            stringify!(cwidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_raw_inset_crop_t>())).cheight as *const _ as usize
        },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_raw_inset_crop_t),
            "::",
            stringify!(cheight)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_raw_inset_crop_t>())).aspect as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_raw_inset_crop_t),
            "::",
            stringify!(aspect)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_image_sizes_t {
    pub raw_height: ushort,
    pub raw_width: ushort,
    pub height: ushort,
    pub width: ushort,
    pub top_margin: ushort,
    pub left_margin: ushort,
    pub iheight: ushort,
    pub iwidth: ushort,
    pub raw_pitch: libc::c_uint,
    pub pixel_aspect: f64,
    pub flip: libc::c_int,
    pub mask: [[libc::c_int; 4usize]; 8usize],
    pub raw_inset_crop: libraw_raw_inset_crop_t,
}
#[test]
fn bindgen_test_layout_libraw_image_sizes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_image_sizes_t>(),
        176usize,
        concat!("Size of: ", stringify!(libraw_image_sizes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_image_sizes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_image_sizes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_image_sizes_t>())).raw_height as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(raw_height)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).raw_width as *const _ as usize },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(raw_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).height as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(height)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).width as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(width)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_image_sizes_t>())).top_margin as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(top_margin)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_image_sizes_t>())).left_margin as *const _ as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(left_margin)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).iheight as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(iheight)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).iwidth as *const _ as usize },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(iwidth)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).raw_pitch as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(raw_pitch)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_image_sizes_t>())).pixel_aspect as *const _ as usize
        },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(pixel_aspect)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).flip as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(flip)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_image_sizes_t>())).mask as *const _ as usize },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(mask)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_image_sizes_t>())).raw_inset_crop as *const _ as usize
        },
        164usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_image_sizes_t),
            "::",
            stringify!(raw_inset_crop)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct ph1_t {
    pub format: libc::c_int,
    pub key_off: libc::c_int,
    pub tag_21a: libc::c_int,
    pub t_black: libc::c_int,
    pub split_col: libc::c_int,
    pub black_col: libc::c_int,
    pub split_row: libc::c_int,
    pub black_row: libc::c_int,
    pub tag_210: f32,
}
#[test]
fn bindgen_test_layout_ph1_t() {
    assert_eq!(
        ::core::mem::size_of::<ph1_t>(),
        36usize,
        concat!("Size of: ", stringify!(ph1_t))
    );
    assert_eq!(
        ::core::mem::align_of::<ph1_t>(),
        4usize,
        concat!("Alignment of ", stringify!(ph1_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).format as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(format)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).key_off as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(key_off)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).tag_21a as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(tag_21a)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).t_black as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(t_black)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).split_col as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(split_col)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).black_col as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(black_col)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).split_row as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(split_row)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).black_row as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(black_row)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<ph1_t>())).tag_210 as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(ph1_t),
            "::",
            stringify!(tag_210)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_dng_color_t {
    pub parsedfields: libc::c_uint,
    pub illuminant: ushort,
    pub calibration: [[f32; 4usize]; 4usize],
    pub colormatrix: [[f32; 3usize]; 4usize],
    pub forwardmatrix: [[f32; 4usize]; 3usize],
}
#[test]
fn bindgen_test_layout_libraw_dng_color_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_dng_color_t>(),
        168usize,
        concat!("Size of: ", stringify!(libraw_dng_color_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_dng_color_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_dng_color_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_color_t>())).parsedfields as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_color_t),
            "::",
            stringify!(parsedfields)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_color_t>())).illuminant as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_color_t),
            "::",
            stringify!(illuminant)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_color_t>())).calibration as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_color_t),
            "::",
            stringify!(calibration)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_color_t>())).colormatrix as *const _ as usize },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_color_t),
            "::",
            stringify!(colormatrix)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_color_t>())).forwardmatrix as *const _ as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_color_t),
            "::",
            stringify!(forwardmatrix)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_dng_levels_t {
    pub parsedfields: libc::c_uint,
    pub dng_cblack: [libc::c_uint; 4104usize],
    pub dng_black: libc::c_uint,
    pub dng_fcblack: [f32; 4104usize],
    pub dng_fblack: f32,
    pub dng_whitelevel: [libc::c_uint; 4usize],
    pub default_crop: [libc::c_uint; 4usize],
    pub preview_colorspace: libc::c_uint,
    pub analogbalance: [f32; 4usize],
    pub asshotneutral: [f32; 4usize],
    pub baseline_exposure: f32,
    pub LinearResponseLimit: f32,
}
#[test]
fn bindgen_test_layout_libraw_dng_levels_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_dng_levels_t>(),
        32920usize,
        concat!("Size of: ", stringify!(libraw_dng_levels_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_dng_levels_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_dng_levels_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).parsedfields as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(parsedfields)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_levels_t>())).dng_cblack as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(dng_cblack)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_levels_t>())).dng_black as *const _ as usize },
        16420usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(dng_black)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).dng_fcblack as *const _ as usize
        },
        16424usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(dng_fcblack)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dng_levels_t>())).dng_fblack as *const _ as usize },
        32840usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(dng_fblack)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).dng_whitelevel as *const _ as usize
        },
        32844usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(dng_whitelevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).default_crop as *const _ as usize
        },
        32860usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(default_crop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).preview_colorspace as *const _ as usize
        },
        32876usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(preview_colorspace)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).analogbalance as *const _ as usize
        },
        32880usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(analogbalance)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).asshotneutral as *const _ as usize
        },
        32896usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(asshotneutral)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).baseline_exposure as *const _ as usize
        },
        32912usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(baseline_exposure)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dng_levels_t>())).LinearResponseLimit as *const _
                as usize
        },
        32916usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dng_levels_t),
            "::",
            stringify!(LinearResponseLimit)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_P1_color_t {
    pub romm_cam: [f32; 9usize],
}
#[test]
fn bindgen_test_layout_libraw_P1_color_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_P1_color_t>(),
        36usize,
        concat!("Size of: ", stringify!(libraw_P1_color_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_P1_color_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_P1_color_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_P1_color_t>())).romm_cam as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_P1_color_t),
            "::",
            stringify!(romm_cam)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_canon_makernotes_t {
    pub ColorDataVer: libc::c_int,
    pub ColorDataSubVer: libc::c_int,
    pub SpecularWhiteLevel: libc::c_int,
    pub NormalWhiteLevel: libc::c_int,
    pub ChannelBlackLevel: [libc::c_int; 4usize],
    pub AverageBlackLevel: libc::c_int,
    pub multishot: [libc::c_uint; 4usize],
    pub MeteringMode: libc::c_short,
    pub SpotMeteringMode: libc::c_short,
    pub FlashMeteringMode: uchar,
    pub FlashExposureLock: libc::c_short,
    pub ExposureMode: libc::c_short,
    pub AESetting: libc::c_short,
    pub HighlightTonePriority: uchar,
    pub ImageStabilization: libc::c_short,
    pub FocusMode: libc::c_short,
    pub AFPoint: libc::c_short,
    pub FocusContinuous: libc::c_short,
    pub AFPointsInFocus30D: libc::c_short,
    pub AFPointsInFocus1D: [uchar; 8usize],
    pub AFPointsInFocus5D: ushort,
    pub AFAreaMode: ushort,
    pub NumAFPoints: ushort,
    pub ValidAFPoints: ushort,
    pub AFImageWidth: ushort,
    pub AFImageHeight: ushort,
    pub AFAreaWidths: [libc::c_short; 61usize],
    pub AFAreaHeights: [libc::c_short; 61usize],
    pub AFAreaXPositions: [libc::c_short; 61usize],
    pub AFAreaYPositions: [libc::c_short; 61usize],
    pub AFPointsInFocus: [libc::c_short; 4usize],
    pub AFPointsSelected: [libc::c_short; 4usize],
    pub PrimaryAFPoint: ushort,
    pub FlashMode: libc::c_short,
    pub FlashActivity: libc::c_short,
    pub FlashBits: libc::c_short,
    pub ManualFlashOutput: libc::c_short,
    pub FlashOutput: libc::c_short,
    pub FlashGuideNumber: libc::c_short,
    pub ContinuousDrive: libc::c_short,
    pub SensorWidth: libc::c_short,
    pub SensorHeight: libc::c_short,
    pub SensorLeftBorder: libc::c_short,
    pub SensorTopBorder: libc::c_short,
    pub SensorRightBorder: libc::c_short,
    pub SensorBottomBorder: libc::c_short,
    pub BlackMaskLeftBorder: libc::c_short,
    pub BlackMaskTopBorder: libc::c_short,
    pub BlackMaskRightBorder: libc::c_short,
    pub BlackMaskBottomBorder: libc::c_short,
    pub AFMicroAdjMode: libc::c_int,
    pub AFMicroAdjValue: f32,
    pub MakernotesFlip: libc::c_short,
    pub RecordMode: libc::c_short,
    pub SRAWQuality: libc::c_short,
    pub wbi: libc::c_uint,
    pub firmware: f32,
    pub RF_lensID: libc::c_short,
}
#[test]
fn bindgen_test_layout_libraw_canon_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_canon_makernotes_t>(),
        664usize,
        concat!("Size of: ", stringify!(libraw_canon_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_canon_makernotes_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_canon_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ColorDataVer as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ColorDataVer)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ColorDataSubVer as *const _
                as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ColorDataSubVer)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SpecularWhiteLevel as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SpecularWhiteLevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).NormalWhiteLevel as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(NormalWhiteLevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ChannelBlackLevel as *const _
                as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ChannelBlackLevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AverageBlackLevel as *const _
                as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AverageBlackLevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).multishot as *const _ as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(multishot)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).MeteringMode as *const _ as usize
        },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(MeteringMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SpotMeteringMode as *const _
                as usize
        },
        54usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SpotMeteringMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashMeteringMode as *const _
                as usize
        },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashMeteringMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashExposureLock as *const _
                as usize
        },
        58usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashExposureLock)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ExposureMode as *const _ as usize
        },
        60usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ExposureMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AESetting as *const _ as usize
        },
        62usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AESetting)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).HighlightTonePriority as *const _
                as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(HighlightTonePriority)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ImageStabilization as *const _
                as usize
        },
        66usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ImageStabilization)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FocusMode as *const _ as usize
        },
        68usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPoint as *const _ as usize
        },
        70usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FocusContinuous as *const _
                as usize
        },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FocusContinuous)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPointsInFocus30D as *const _
                as usize
        },
        74usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPointsInFocus30D)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPointsInFocus1D as *const _
                as usize
        },
        76usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPointsInFocus1D)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPointsInFocus5D as *const _
                as usize
        },
        84usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPointsInFocus5D)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFAreaMode as *const _ as usize
        },
        86usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFAreaMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).NumAFPoints as *const _ as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(NumAFPoints)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ValidAFPoints as *const _
                as usize
        },
        90usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ValidAFPoints)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFImageWidth as *const _ as usize
        },
        92usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFImageWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFImageHeight as *const _
                as usize
        },
        94usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFImageHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFAreaWidths as *const _ as usize
        },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFAreaWidths)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFAreaHeights as *const _
                as usize
        },
        218usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFAreaHeights)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFAreaXPositions as *const _
                as usize
        },
        340usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFAreaXPositions)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFAreaYPositions as *const _
                as usize
        },
        462usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFAreaYPositions)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPointsInFocus as *const _
                as usize
        },
        584usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPointsInFocus)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFPointsSelected as *const _
                as usize
        },
        592usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFPointsSelected)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).PrimaryAFPoint as *const _
                as usize
        },
        600usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(PrimaryAFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashMode as *const _ as usize
        },
        602usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashActivity as *const _
                as usize
        },
        604usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashActivity)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashBits as *const _ as usize
        },
        606usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashBits)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ManualFlashOutput as *const _
                as usize
        },
        608usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ManualFlashOutput)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashOutput as *const _ as usize
        },
        610usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashOutput)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).FlashGuideNumber as *const _
                as usize
        },
        612usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(FlashGuideNumber)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).ContinuousDrive as *const _
                as usize
        },
        614usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(ContinuousDrive)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorWidth as *const _ as usize
        },
        616usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorHeight as *const _ as usize
        },
        618usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorLeftBorder as *const _
                as usize
        },
        620usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorLeftBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorTopBorder as *const _
                as usize
        },
        622usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorTopBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorRightBorder as *const _
                as usize
        },
        624usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorRightBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SensorBottomBorder as *const _
                as usize
        },
        626usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SensorBottomBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).BlackMaskLeftBorder as *const _
                as usize
        },
        628usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(BlackMaskLeftBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).BlackMaskTopBorder as *const _
                as usize
        },
        630usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(BlackMaskTopBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).BlackMaskRightBorder as *const _
                as usize
        },
        632usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(BlackMaskRightBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).BlackMaskBottomBorder as *const _
                as usize
        },
        634usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(BlackMaskBottomBorder)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFMicroAdjMode as *const _
                as usize
        },
        636usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFMicroAdjMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).AFMicroAdjValue as *const _
                as usize
        },
        640usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(AFMicroAdjValue)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).MakernotesFlip as *const _
                as usize
        },
        644usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(MakernotesFlip)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).RecordMode as *const _ as usize
        },
        646usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(RecordMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).SRAWQuality as *const _ as usize
        },
        648usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(SRAWQuality)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).wbi as *const _ as usize },
        652usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(wbi)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).firmware as *const _ as usize
        },
        656usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(firmware)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_canon_makernotes_t>())).RF_lensID as *const _ as usize
        },
        660usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_canon_makernotes_t),
            "::",
            stringify!(RF_lensID)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_hasselblad_makernotes_t {
    pub BaseISO: libc::c_int,
    pub Gain: f64,
    pub Sensor: [libc::c_char; 8usize],
    pub SensorUnit: [libc::c_char; 64usize],
    pub HostBody: [libc::c_char; 64usize],
    pub SensorCode: libc::c_int,
    pub SensorSubCode: libc::c_int,
    pub CoatingCode: libc::c_int,
    pub uncropped: libc::c_int,
    pub CaptureSequenceInitiator: [libc::c_char; 32usize],
    pub SensorUnitConnector: [libc::c_char; 64usize],
    pub format: libc::c_int,
    pub nIFD_CM: [libc::c_int; 2usize],
    pub RecommendedCrop: [libc::c_int; 2usize],
    pub mnColorMatrix: [[f64; 3usize]; 4usize],
}
#[test]
fn bindgen_test_layout_libraw_hasselblad_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_hasselblad_makernotes_t>(),
        384usize,
        concat!("Size of: ", stringify!(libraw_hasselblad_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_hasselblad_makernotes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_hasselblad_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).BaseISO as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(BaseISO)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).Gain as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(Gain)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).Sensor as *const _ as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(Sensor)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).SensorUnit as *const _
                as usize
        },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(SensorUnit)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).HostBody as *const _
                as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(HostBody)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).SensorCode as *const _
                as usize
        },
        152usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(SensorCode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).SensorSubCode as *const _
                as usize
        },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(SensorSubCode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).CoatingCode as *const _
                as usize
        },
        160usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(CoatingCode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).uncropped as *const _
                as usize
        },
        164usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(uncropped)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).CaptureSequenceInitiator
                as *const _ as usize
        },
        168usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(CaptureSequenceInitiator)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).SensorUnitConnector
                as *const _ as usize
        },
        200usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(SensorUnitConnector)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).format as *const _ as usize
        },
        264usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(format)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).nIFD_CM as *const _ as usize
        },
        268usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(nIFD_CM)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).RecommendedCrop as *const _
                as usize
        },
        276usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(RecommendedCrop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_hasselblad_makernotes_t>())).mnColorMatrix as *const _
                as usize
        },
        288usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_hasselblad_makernotes_t),
            "::",
            stringify!(mnColorMatrix)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_fuji_info_t {
    pub ExpoMidPointShift: f32,
    pub DynamicRange: ushort,
    pub FilmMode: ushort,
    pub DynamicRangeSetting: ushort,
    pub DevelopmentDynamicRange: ushort,
    pub AutoDynamicRange: ushort,
    pub DRangePriority: ushort,
    pub DRangePriorityAuto: ushort,
    pub DRangePriorityFixed: ushort,
    pub BrightnessCompensation: f32,
    pub FocusMode: ushort,
    pub AFMode: ushort,
    pub FocusPixel: [ushort; 2usize],
    pub ImageStabilization: [ushort; 3usize],
    pub FlashMode: ushort,
    pub WB_Preset: ushort,
    pub ShutterType: ushort,
    pub ExrMode: ushort,
    pub Macro: ushort,
    pub Rating: libc::c_uint,
    pub CropMode: ushort,
    pub FrameRate: ushort,
    pub FrameWidth: ushort,
    pub FrameHeight: ushort,
    pub SerialSignature: [libc::c_char; 13usize],
    pub RAFVersion: [libc::c_char; 5usize],
    pub RAFDataVersion: ushort,
    pub isTSNERDTS: libc::c_int,
    pub DriveMode: libc::c_short,
}
#[test]
fn bindgen_test_layout_libraw_fuji_info_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_fuji_info_t>(),
        88usize,
        concat!("Size of: ", stringify!(libraw_fuji_info_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_fuji_info_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_fuji_info_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).ExpoMidPointShift as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(ExpoMidPointShift)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DynamicRange as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DynamicRange)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FilmMode as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FilmMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DynamicRangeSetting as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DynamicRangeSetting)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DevelopmentDynamicRange as *const _
                as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DevelopmentDynamicRange)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).AutoDynamicRange as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(AutoDynamicRange)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DRangePriority as *const _ as usize
        },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DRangePriority)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DRangePriorityAuto as *const _ as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DRangePriorityAuto)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).DRangePriorityFixed as *const _ as usize
        },
        18usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DRangePriorityFixed)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).BrightnessCompensation as *const _
                as usize
        },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(BrightnessCompensation)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FocusMode as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).AFMode as *const _ as usize },
        26usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(AFMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FocusPixel as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FocusPixel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).ImageStabilization as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(ImageStabilization)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FlashMode as *const _ as usize },
        38usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FlashMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).WB_Preset as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(WB_Preset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).ShutterType as *const _ as usize },
        42usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(ShutterType)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).ExrMode as *const _ as usize },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(ExrMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).Macro as *const _ as usize },
        46usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(Macro)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).Rating as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(Rating)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).CropMode as *const _ as usize },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(CropMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FrameRate as *const _ as usize },
        54usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FrameRate)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FrameWidth as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FrameWidth)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).FrameHeight as *const _ as usize },
        58usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(FrameHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).SerialSignature as *const _ as usize
        },
        60usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(SerialSignature)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).RAFVersion as *const _ as usize },
        73usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(RAFVersion)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_fuji_info_t>())).RAFDataVersion as *const _ as usize
        },
        78usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(RAFDataVersion)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).isTSNERDTS as *const _ as usize },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(isTSNERDTS)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_fuji_info_t>())).DriveMode as *const _ as usize },
        84usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_fuji_info_t),
            "::",
            stringify!(DriveMode)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_sensor_highspeed_crop_t {
    pub cleft: ushort,
    pub ctop: ushort,
    pub cwidth: ushort,
    pub cheight: ushort,
}
#[test]
fn bindgen_test_layout_libraw_sensor_highspeed_crop_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_sensor_highspeed_crop_t>(),
        8usize,
        concat!("Size of: ", stringify!(libraw_sensor_highspeed_crop_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_sensor_highspeed_crop_t>(),
        2usize,
        concat!("Alignment of ", stringify!(libraw_sensor_highspeed_crop_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sensor_highspeed_crop_t>())).cleft as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sensor_highspeed_crop_t),
            "::",
            stringify!(cleft)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sensor_highspeed_crop_t>())).ctop as *const _ as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sensor_highspeed_crop_t),
            "::",
            stringify!(ctop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sensor_highspeed_crop_t>())).cwidth as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sensor_highspeed_crop_t),
            "::",
            stringify!(cwidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sensor_highspeed_crop_t>())).cheight as *const _ as usize
        },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sensor_highspeed_crop_t),
            "::",
            stringify!(cheight)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_nikon_makernotes_t {
    pub ExposureBracketValue: f64,
    pub ActiveDLighting: ushort,
    pub ShootingMode: ushort,
    pub ImageStabilization: [uchar; 7usize],
    pub VibrationReduction: uchar,
    pub VRMode: uchar,
    pub FocusMode: [libc::c_char; 7usize],
    pub AFPoint: uchar,
    pub AFPointsInFocus: ushort,
    pub ContrastDetectAF: uchar,
    pub AFAreaMode: uchar,
    pub PhaseDetectAF: uchar,
    pub PrimaryAFPoint: uchar,
    pub AFPointsUsed: [uchar; 29usize],
    pub AFImageWidth: ushort,
    pub AFImageHeight: ushort,
    pub AFAreaXPposition: ushort,
    pub AFAreaYPosition: ushort,
    pub AFAreaWidth: ushort,
    pub AFAreaHeight: ushort,
    pub ContrastDetectAFInFocus: uchar,
    pub FlashSetting: [libc::c_char; 13usize],
    pub FlashType: [libc::c_char; 20usize],
    pub FlashExposureCompensation: [uchar; 4usize],
    pub ExternalFlashExposureComp: [uchar; 4usize],
    pub FlashExposureBracketValue: [uchar; 4usize],
    pub FlashMode: uchar,
    pub FlashExposureCompensation2: libc::c_schar,
    pub FlashExposureCompensation3: libc::c_schar,
    pub FlashExposureCompensation4: libc::c_schar,
    pub FlashSource: uchar,
    pub FlashFirmware: [uchar; 2usize],
    pub ExternalFlashFlags: uchar,
    pub FlashControlCommanderMode: uchar,
    pub FlashOutputAndCompensation: uchar,
    pub FlashFocalLength: uchar,
    pub FlashGNDistance: uchar,
    pub FlashGroupControlMode: [uchar; 4usize],
    pub FlashGroupOutputAndCompensation: [uchar; 4usize],
    pub FlashColorFilter: uchar,
    pub NEFCompression: ushort,
    pub ExposureMode: libc::c_int,
    pub ExposureProgram: libc::c_int,
    pub nMEshots: libc::c_int,
    pub MEgainOn: libc::c_int,
    pub ME_WB: [f64; 4usize],
    pub AFFineTune: uchar,
    pub AFFineTuneIndex: uchar,
    pub AFFineTuneAdj: i8,
    pub LensDataVersion: libc::c_uint,
    pub FlashInfoVersion: libc::c_uint,
    pub ColorBalanceVersion: libc::c_uint,
    pub key: uchar,
    pub NEFBitDepth: [ushort; 4usize],
    pub HighSpeedCropFormat: ushort,
    pub SensorHighSpeedCrop: libraw_sensor_highspeed_crop_t,
    pub SensorWidth: ushort,
    pub SensorHeight: ushort,
}
#[test]
fn bindgen_test_layout_libraw_nikon_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_nikon_makernotes_t>(),
        240usize,
        concat!("Size of: ", stringify!(libraw_nikon_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_nikon_makernotes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_nikon_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ExposureBracketValue as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ExposureBracketValue)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ActiveDLighting as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ActiveDLighting)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ShootingMode as *const _ as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ShootingMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ImageStabilization as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ImageStabilization)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).VibrationReduction as *const _
                as usize
        },
        19usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(VibrationReduction)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).VRMode as *const _ as usize
        },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(VRMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FocusMode as *const _ as usize
        },
        21usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFPoint as *const _ as usize
        },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFPointsInFocus as *const _
                as usize
        },
        30usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFPointsInFocus)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ContrastDetectAF as *const _
                as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ContrastDetectAF)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFAreaMode as *const _ as usize
        },
        33usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFAreaMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).PhaseDetectAF as *const _
                as usize
        },
        34usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(PhaseDetectAF)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).PrimaryAFPoint as *const _
                as usize
        },
        35usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(PrimaryAFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFPointsUsed as *const _ as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFPointsUsed)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFImageWidth as *const _ as usize
        },
        66usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFImageWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFImageHeight as *const _
                as usize
        },
        68usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFImageHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFAreaXPposition as *const _
                as usize
        },
        70usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFAreaXPposition)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFAreaYPosition as *const _
                as usize
        },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFAreaYPosition)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFAreaWidth as *const _ as usize
        },
        74usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFAreaWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFAreaHeight as *const _ as usize
        },
        76usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFAreaHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ContrastDetectAFInFocus
                as *const _ as usize
        },
        78usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ContrastDetectAFInFocus)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashSetting as *const _ as usize
        },
        79usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashSetting)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashType as *const _ as usize
        },
        92usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashExposureCompensation
                as *const _ as usize
        },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashExposureCompensation)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ExternalFlashExposureComp
                as *const _ as usize
        },
        116usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ExternalFlashExposureComp)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashExposureBracketValue
                as *const _ as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashExposureBracketValue)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashMode as *const _ as usize
        },
        124usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashExposureCompensation2
                as *const _ as usize
        },
        125usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashExposureCompensation2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashExposureCompensation3
                as *const _ as usize
        },
        126usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashExposureCompensation3)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashExposureCompensation4
                as *const _ as usize
        },
        127usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashExposureCompensation4)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashSource as *const _ as usize
        },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashSource)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashFirmware as *const _
                as usize
        },
        129usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashFirmware)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ExternalFlashFlags as *const _
                as usize
        },
        131usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ExternalFlashFlags)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashControlCommanderMode
                as *const _ as usize
        },
        132usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashControlCommanderMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashOutputAndCompensation
                as *const _ as usize
        },
        133usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashOutputAndCompensation)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashFocalLength as *const _
                as usize
        },
        134usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashFocalLength)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashGNDistance as *const _
                as usize
        },
        135usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashGNDistance)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashGroupControlMode as *const _
                as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashGroupControlMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashGroupOutputAndCompensation
                as *const _ as usize
        },
        140usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashGroupOutputAndCompensation)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashColorFilter as *const _
                as usize
        },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashColorFilter)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).NEFCompression as *const _
                as usize
        },
        146usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(NEFCompression)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ExposureMode as *const _ as usize
        },
        148usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ExposureMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ExposureProgram as *const _
                as usize
        },
        152usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ExposureProgram)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).nMEshots as *const _ as usize
        },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(nMEshots)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).MEgainOn as *const _ as usize
        },
        160usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(MEgainOn)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ME_WB as *const _ as usize
        },
        168usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ME_WB)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFFineTune as *const _ as usize
        },
        200usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFFineTune)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFFineTuneIndex as *const _
                as usize
        },
        201usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFFineTuneIndex)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).AFFineTuneAdj as *const _
                as usize
        },
        202usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(AFFineTuneAdj)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).LensDataVersion as *const _
                as usize
        },
        204usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(LensDataVersion)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).FlashInfoVersion as *const _
                as usize
        },
        208usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(FlashInfoVersion)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).ColorBalanceVersion as *const _
                as usize
        },
        212usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(ColorBalanceVersion)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).key as *const _ as usize },
        216usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(key)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).NEFBitDepth as *const _ as usize
        },
        218usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(NEFBitDepth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).HighSpeedCropFormat as *const _
                as usize
        },
        226usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(HighSpeedCropFormat)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).SensorHighSpeedCrop as *const _
                as usize
        },
        228usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(SensorHighSpeedCrop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).SensorWidth as *const _ as usize
        },
        236usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(SensorWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikon_makernotes_t>())).SensorHeight as *const _ as usize
        },
        238usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikon_makernotes_t),
            "::",
            stringify!(SensorHeight)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_olympus_makernotes_t {
    pub SensorCalibration: [libc::c_int; 2usize],
    pub FocusMode: [ushort; 2usize],
    pub AutoFocus: ushort,
    pub AFPoint: ushort,
    pub AFAreas: [libc::c_uint; 64usize],
    pub AFPointSelected: [f64; 5usize],
    pub AFResult: ushort,
    pub DriveMode: [ushort; 5usize],
    pub ColorSpace: ushort,
    pub AFFineTune: uchar,
    pub AFFineTuneAdj: [libc::c_short; 3usize],
    pub CameraType2: [libc::c_char; 6usize],
}
#[test]
fn bindgen_test_layout_libraw_olympus_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_olympus_makernotes_t>(),
        344usize,
        concat!("Size of: ", stringify!(libraw_olympus_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_olympus_makernotes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_olympus_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).SensorCalibration as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(SensorCalibration)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).FocusMode as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AutoFocus as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AutoFocus)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFPoint as *const _ as usize
        },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFAreas as *const _ as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFAreas)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFPointSelected as *const _
                as usize
        },
        272usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFPointSelected)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFResult as *const _ as usize
        },
        312usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFResult)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).DriveMode as *const _ as usize
        },
        314usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(DriveMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).ColorSpace as *const _ as usize
        },
        324usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(ColorSpace)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFFineTune as *const _ as usize
        },
        326usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFFineTune)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).AFFineTuneAdj as *const _
                as usize
        },
        328usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(AFFineTuneAdj)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_olympus_makernotes_t>())).CameraType2 as *const _
                as usize
        },
        334usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_olympus_makernotes_t),
            "::",
            stringify!(CameraType2)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_panasonic_makernotes_t {
    pub Compression: ushort,
    pub BlackLevelDim: ushort,
    pub BlackLevel: [f32; 8usize],
    pub Multishot: libc::c_uint,
    pub gamma: f32,
    pub HighISOMultiplier: [libc::c_int; 3usize],
}
#[test]
fn bindgen_test_layout_libraw_panasonic_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_panasonic_makernotes_t>(),
        56usize,
        concat!("Size of: ", stringify!(libraw_panasonic_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_panasonic_makernotes_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_panasonic_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).Compression as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(Compression)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).BlackLevelDim as *const _
                as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(BlackLevelDim)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).BlackLevel as *const _
                as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(BlackLevel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).Multishot as *const _
                as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(Multishot)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).gamma as *const _ as usize
        },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(gamma)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_panasonic_makernotes_t>())).HighISOMultiplier as *const _
                as usize
        },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_panasonic_makernotes_t),
            "::",
            stringify!(HighISOMultiplier)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_pentax_makernotes_t {
    pub FocusMode: ushort,
    pub AFPointSelected: ushort,
    pub AFPointsInFocus: libc::c_uint,
    pub FocusPosition: ushort,
    pub DriveMode: [uchar; 4usize],
    pub AFAdjustment: libc::c_short,
    pub MultiExposure: uchar,
    pub Quality: ushort,
}
#[test]
fn bindgen_test_layout_libraw_pentax_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_pentax_makernotes_t>(),
        20usize,
        concat!("Size of: ", stringify!(libraw_pentax_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_pentax_makernotes_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_pentax_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).FocusMode as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).AFPointSelected as *const _
                as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(AFPointSelected)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).AFPointsInFocus as *const _
                as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(AFPointsInFocus)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).FocusPosition as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(FocusPosition)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).DriveMode as *const _ as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(DriveMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).AFAdjustment as *const _
                as usize
        },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(AFAdjustment)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).MultiExposure as *const _
                as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(MultiExposure)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_pentax_makernotes_t>())).Quality as *const _ as usize
        },
        18usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_pentax_makernotes_t),
            "::",
            stringify!(Quality)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_samsung_makernotes_t {
    pub ImageSizeFull: [libc::c_uint; 4usize],
    pub ImageSizeCrop: [libc::c_uint; 4usize],
    pub ColorSpace: [libc::c_int; 2usize],
    pub key: [libc::c_uint; 11usize],
    pub DigitalGain: f64,
    pub DeviceType: libc::c_int,
    pub LensFirmware: [libc::c_char; 32usize],
}
#[test]
fn bindgen_test_layout_libraw_samsung_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_samsung_makernotes_t>(),
        136usize,
        concat!("Size of: ", stringify!(libraw_samsung_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_samsung_makernotes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_samsung_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).ImageSizeFull as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(ImageSizeFull)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).ImageSizeCrop as *const _
                as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(ImageSizeCrop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).ColorSpace as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(ColorSpace)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).key as *const _ as usize
        },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(key)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).DigitalGain as *const _
                as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(DigitalGain)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).DeviceType as *const _ as usize
        },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(DeviceType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_samsung_makernotes_t>())).LensFirmware as *const _
                as usize
        },
        100usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_samsung_makernotes_t),
            "::",
            stringify!(LensFirmware)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_kodak_makernotes_t {
    pub BlackLevelTop: ushort,
    pub BlackLevelBottom: ushort,
    pub offset_left: libc::c_short,
    pub offset_top: libc::c_short,
    pub clipBlack: ushort,
    pub clipWhite: ushort,
    pub romm_camDaylight: [[f32; 3usize]; 3usize],
    pub romm_camTungsten: [[f32; 3usize]; 3usize],
    pub romm_camFluorescent: [[f32; 3usize]; 3usize],
    pub romm_camFlash: [[f32; 3usize]; 3usize],
    pub romm_camCustom: [[f32; 3usize]; 3usize],
    pub romm_camAuto: [[f32; 3usize]; 3usize],
    pub val018percent: ushort,
    pub val100percent: ushort,
    pub val170percent: ushort,
    pub MakerNoteKodak8a: libc::c_short,
    pub ISOCalibrationGain: f32,
    pub AnalogISO: f32,
}
#[test]
fn bindgen_test_layout_libraw_kodak_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_kodak_makernotes_t>(),
        244usize,
        concat!("Size of: ", stringify!(libraw_kodak_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_kodak_makernotes_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_kodak_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).BlackLevelTop as *const _
                as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(BlackLevelTop)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).BlackLevelBottom as *const _
                as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(BlackLevelBottom)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).offset_left as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(offset_left)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).offset_top as *const _ as usize
        },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(offset_top)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).clipBlack as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(clipBlack)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).clipWhite as *const _ as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(clipWhite)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camDaylight as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camDaylight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camTungsten as *const _
                as usize
        },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camTungsten)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camFluorescent as *const _
                as usize
        },
        84usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camFluorescent)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camFlash as *const _
                as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camFlash)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camCustom as *const _
                as usize
        },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camCustom)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).romm_camAuto as *const _ as usize
        },
        192usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(romm_camAuto)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).val018percent as *const _
                as usize
        },
        228usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(val018percent)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).val100percent as *const _
                as usize
        },
        230usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(val100percent)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).val170percent as *const _
                as usize
        },
        232usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(val170percent)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).MakerNoteKodak8a as *const _
                as usize
        },
        234usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(MakerNoteKodak8a)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).ISOCalibrationGain as *const _
                as usize
        },
        236usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(ISOCalibrationGain)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_kodak_makernotes_t>())).AnalogISO as *const _ as usize
        },
        240usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_kodak_makernotes_t),
            "::",
            stringify!(AnalogISO)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_p1_makernotes_t {
    pub Software: [libc::c_char; 64usize],
    pub SystemType: [libc::c_char; 64usize],
    pub FirmwareString: [libc::c_char; 256usize],
    pub SystemModel: [libc::c_char; 64usize],
}
#[test]
fn bindgen_test_layout_libraw_p1_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_p1_makernotes_t>(),
        448usize,
        concat!("Size of: ", stringify!(libraw_p1_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_p1_makernotes_t>(),
        1usize,
        concat!("Alignment of ", stringify!(libraw_p1_makernotes_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_p1_makernotes_t>())).Software as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_p1_makernotes_t),
            "::",
            stringify!(Software)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_p1_makernotes_t>())).SystemType as *const _ as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_p1_makernotes_t),
            "::",
            stringify!(SystemType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_p1_makernotes_t>())).FirmwareString as *const _ as usize
        },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_p1_makernotes_t),
            "::",
            stringify!(FirmwareString)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_p1_makernotes_t>())).SystemModel as *const _ as usize
        },
        384usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_p1_makernotes_t),
            "::",
            stringify!(SystemModel)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_sony_info_t {
    pub CameraType: ushort,
    pub Sony0x9400_version: uchar,
    pub Sony0x9400_ReleaseMode2: uchar,
    pub Sony0x9400_SequenceImageNumber: libc::c_uint,
    pub Sony0x9400_SequenceLength1: uchar,
    pub Sony0x9400_SequenceFileNumber: libc::c_uint,
    pub Sony0x9400_SequenceLength2: uchar,
    pub AFAreaModeSetting: u8,
    pub FlexibleSpotPosition: [ushort; 2usize],
    pub AFPointSelected: u8,
    pub AFPointsUsed: [u8; 10usize],
    pub AFTracking: u8,
    pub AFType: u8,
    pub FocusLocation: [ushort; 4usize],
    pub AFMicroAdjValue: i8,
    pub AFMicroAdjOn: i8,
    pub AFMicroAdjRegisteredLenses: uchar,
    pub VariableLowPassFilter: ushort,
    pub LongExposureNoiseReduction: libc::c_uint,
    pub HighISONoiseReduction: ushort,
    pub HDR: [ushort; 2usize],
    pub group2010: ushort,
    pub real_iso_offset: ushort,
    pub MeteringMode_offset: ushort,
    pub ExposureProgram_offset: ushort,
    pub ReleaseMode2_offset: ushort,
    pub MinoltaCamID: libc::c_uint,
    pub firmware: f32,
    pub ImageCount3_offset: ushort,
    pub ImageCount3: libc::c_uint,
    pub ElectronicFrontCurtainShutter: libc::c_uint,
    pub MeteringMode2: ushort,
    pub SonyDateTime: [libc::c_char; 20usize],
    pub ShotNumberSincePowerUp: libc::c_uint,
    pub PixelShiftGroupPrefix: ushort,
    pub PixelShiftGroupID: libc::c_uint,
    pub nShotsInPixelShiftGroup: libc::c_char,
    pub numInPixelShiftGroup: libc::c_char,
    pub prd_ImageHeight: ushort,
    pub prd_ImageWidth: ushort,
    pub prd_RawBitDepth: ushort,
    pub prd_StorageMethod: ushort,
    pub prd_BayerPattern: ushort,
    pub SonyRawFileType: ushort,
    pub RAWFileType: ushort,
    pub Quality: libc::c_uint,
    pub FileFormat: ushort,
}
#[test]
fn bindgen_test_layout_libraw_sony_info_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_sony_info_t>(),
        152usize,
        concat!("Size of: ", stringify!(libraw_sony_info_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_sony_info_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_sony_info_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).CameraType as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(CameraType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_version as *const _ as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_version)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_ReleaseMode2 as *const _
                as usize
        },
        3usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_ReleaseMode2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_SequenceImageNumber
                as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_SequenceImageNumber)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_SequenceLength1 as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_SequenceLength1)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_SequenceFileNumber
                as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_SequenceFileNumber)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).Sony0x9400_SequenceLength2 as *const _
                as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Sony0x9400_SequenceLength2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFAreaModeSetting as *const _ as usize
        },
        17usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFAreaModeSetting)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).FlexibleSpotPosition as *const _
                as usize
        },
        18usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(FlexibleSpotPosition)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFPointSelected as *const _ as usize
        },
        22usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFPointSelected)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFPointsUsed as *const _ as usize
        },
        23usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFPointsUsed)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).AFTracking as *const _ as usize },
        33usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFTracking)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).AFType as *const _ as usize },
        34usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).FocusLocation as *const _ as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(FocusLocation)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFMicroAdjValue as *const _ as usize
        },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFMicroAdjValue)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFMicroAdjOn as *const _ as usize
        },
        45usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFMicroAdjOn)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).AFMicroAdjRegisteredLenses as *const _
                as usize
        },
        46usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(AFMicroAdjRegisteredLenses)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).VariableLowPassFilter as *const _
                as usize
        },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(VariableLowPassFilter)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).LongExposureNoiseReduction as *const _
                as usize
        },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(LongExposureNoiseReduction)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).HighISONoiseReduction as *const _
                as usize
        },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(HighISONoiseReduction)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).HDR as *const _ as usize },
        58usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(HDR)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).group2010 as *const _ as usize },
        62usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(group2010)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).real_iso_offset as *const _ as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(real_iso_offset)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).MeteringMode_offset as *const _ as usize
        },
        66usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(MeteringMode_offset)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).ExposureProgram_offset as *const _
                as usize
        },
        68usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ExposureProgram_offset)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).ReleaseMode2_offset as *const _ as usize
        },
        70usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ReleaseMode2_offset)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).MinoltaCamID as *const _ as usize
        },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(MinoltaCamID)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).firmware as *const _ as usize },
        76usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(firmware)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).ImageCount3_offset as *const _ as usize
        },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ImageCount3_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).ImageCount3 as *const _ as usize },
        84usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ImageCount3)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).ElectronicFrontCurtainShutter
                as *const _ as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ElectronicFrontCurtainShutter)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).MeteringMode2 as *const _ as usize
        },
        92usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(MeteringMode2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).SonyDateTime as *const _ as usize
        },
        94usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(SonyDateTime)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).ShotNumberSincePowerUp as *const _
                as usize
        },
        116usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(ShotNumberSincePowerUp)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).PixelShiftGroupPrefix as *const _
                as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(PixelShiftGroupPrefix)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).PixelShiftGroupID as *const _ as usize
        },
        124usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(PixelShiftGroupID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).nShotsInPixelShiftGroup as *const _
                as usize
        },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(nShotsInPixelShiftGroup)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).numInPixelShiftGroup as *const _
                as usize
        },
        129usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(numInPixelShiftGroup)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).prd_ImageHeight as *const _ as usize
        },
        130usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(prd_ImageHeight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).prd_ImageWidth as *const _ as usize
        },
        132usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(prd_ImageWidth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).prd_RawBitDepth as *const _ as usize
        },
        134usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(prd_RawBitDepth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).prd_StorageMethod as *const _ as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(prd_StorageMethod)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).prd_BayerPattern as *const _ as usize
        },
        138usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(prd_BayerPattern)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_sony_info_t>())).SonyRawFileType as *const _ as usize
        },
        140usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(SonyRawFileType)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).RAWFileType as *const _ as usize },
        142usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(RAWFileType)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).Quality as *const _ as usize },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(Quality)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_sony_info_t>())).FileFormat as *const _ as usize },
        148usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_sony_info_t),
            "::",
            stringify!(FileFormat)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_colordata_t {
    pub curve: [ushort; 65536usize],
    pub cblack: [libc::c_uint; 4104usize],
    pub black: libc::c_uint,
    pub data_maximum: libc::c_uint,
    pub maximum: libc::c_uint,
    pub linear_max: [libc::c_long; 4usize],
    pub fmaximum: f32,
    pub fnorm: f32,
    pub white: [[ushort; 8usize]; 8usize],
    pub cam_mul: [f32; 4usize],
    pub pre_mul: [f32; 4usize],
    pub cmatrix: [[f32; 4usize]; 3usize],
    pub ccm: [[f32; 4usize]; 3usize],
    pub rgb_cam: [[f32; 4usize]; 3usize],
    pub cam_xyz: [[f32; 3usize]; 4usize],
    pub phase_one_data: ph1_t,
    pub flash_used: f32,
    pub canon_ev: f32,
    pub model2: [libc::c_char; 64usize],
    pub UniqueCameraModel: [libc::c_char; 64usize],
    pub LocalizedCameraModel: [libc::c_char; 64usize],
    pub ImageUniqueID: [libc::c_char; 64usize],
    pub RawDataUniqueID: [libc::c_char; 17usize],
    pub OriginalRawFileName: [libc::c_char; 64usize],
    pub profile: *mut libc::c_void,
    pub profile_length: libc::c_uint,
    pub black_stat: [libc::c_uint; 8usize],
    pub dng_color: [libraw_dng_color_t; 2usize],
    pub dng_levels: libraw_dng_levels_t,
    pub WB_Coeffs: [[libc::c_int; 4usize]; 256usize],
    pub WBCT_Coeffs: [[f32; 5usize]; 64usize],
    pub as_shot_wb_applied: libc::c_int,
    pub P1_color: [libraw_P1_color_t; 2usize],
    pub raw_bps: libc::c_uint,
    pub ExifColorSpace: libc::c_int,
}
#[test]
fn bindgen_test_layout_libraw_colordata_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_colordata_t>(),
        187040usize,
        concat!("Size of: ", stringify!(libraw_colordata_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_colordata_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_colordata_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).curve as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(curve)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).cblack as *const _ as usize },
        131072usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(cblack)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).black as *const _ as usize },
        147488usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(black)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).data_maximum as *const _ as usize
        },
        147492usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(data_maximum)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).maximum as *const _ as usize },
        147496usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(maximum)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).linear_max as *const _ as usize },
        147504usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(linear_max)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).fmaximum as *const _ as usize },
        147536usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(fmaximum)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).fnorm as *const _ as usize },
        147540usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(fnorm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).white as *const _ as usize },
        147544usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(white)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).cam_mul as *const _ as usize },
        147672usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(cam_mul)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).pre_mul as *const _ as usize },
        147688usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(pre_mul)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).cmatrix as *const _ as usize },
        147704usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(cmatrix)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).ccm as *const _ as usize },
        147752usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(ccm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).rgb_cam as *const _ as usize },
        147800usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(rgb_cam)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).cam_xyz as *const _ as usize },
        147848usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(cam_xyz)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).phase_one_data as *const _ as usize
        },
        147896usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(phase_one_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).flash_used as *const _ as usize },
        147932usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(flash_used)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).canon_ev as *const _ as usize },
        147936usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(canon_ev)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).model2 as *const _ as usize },
        147940usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(model2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).UniqueCameraModel as *const _ as usize
        },
        148004usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(UniqueCameraModel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).LocalizedCameraModel as *const _
                as usize
        },
        148068usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(LocalizedCameraModel)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).ImageUniqueID as *const _ as usize
        },
        148132usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(ImageUniqueID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).RawDataUniqueID as *const _ as usize
        },
        148196usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(RawDataUniqueID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).OriginalRawFileName as *const _ as usize
        },
        148213usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(OriginalRawFileName)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).profile as *const _ as usize },
        148280usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(profile)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).profile_length as *const _ as usize
        },
        148288usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(profile_length)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).black_stat as *const _ as usize },
        148292usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(black_stat)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).dng_color as *const _ as usize },
        148324usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(dng_color)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).dng_levels as *const _ as usize },
        148660usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(dng_levels)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).WB_Coeffs as *const _ as usize },
        181580usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(WB_Coeffs)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).WBCT_Coeffs as *const _ as usize },
        185676usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(WBCT_Coeffs)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).as_shot_wb_applied as *const _ as usize
        },
        186956usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(as_shot_wb_applied)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).P1_color as *const _ as usize },
        186960usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(P1_color)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_colordata_t>())).raw_bps as *const _ as usize },
        187032usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(raw_bps)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_colordata_t>())).ExifColorSpace as *const _ as usize
        },
        187036usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_colordata_t),
            "::",
            stringify!(ExifColorSpace)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_thumbnail_t {
    pub tformat: LibRaw_thumbnail_formats,
    pub twidth: ushort,
    pub theight: ushort,
    pub tlength: libc::c_uint,
    pub tcolors: libc::c_int,
    pub thumb: *mut libc::c_char,
}
#[test]
fn bindgen_test_layout_libraw_thumbnail_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_thumbnail_t>(),
        24usize,
        concat!("Size of: ", stringify!(libraw_thumbnail_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_thumbnail_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_thumbnail_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).tformat as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(tformat)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).twidth as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(twidth)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).theight as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(theight)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).tlength as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(tlength)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).tcolors as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(tcolors)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_thumbnail_t>())).thumb as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_thumbnail_t),
            "::",
            stringify!(thumb)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_gps_info_t {
    pub latitude: [f32; 3usize],
    pub longitude: [f32; 3usize],
    pub gpstimestamp: [f32; 3usize],
    pub altitude: f32,
    pub altref: libc::c_char,
    pub latref: libc::c_char,
    pub longref: libc::c_char,
    pub gpsstatus: libc::c_char,
    pub gpsparsed: libc::c_char,
}
#[test]
fn bindgen_test_layout_libraw_gps_info_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_gps_info_t>(),
        48usize,
        concat!("Size of: ", stringify!(libraw_gps_info_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_gps_info_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_gps_info_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).latitude as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(latitude)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).longitude as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(longitude)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).gpstimestamp as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(gpstimestamp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).altitude as *const _ as usize },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(altitude)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).altref as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(altref)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).latref as *const _ as usize },
        41usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(latref)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).longref as *const _ as usize },
        42usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(longref)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).gpsstatus as *const _ as usize },
        43usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(gpsstatus)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_gps_info_t>())).gpsparsed as *const _ as usize },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_gps_info_t),
            "::",
            stringify!(gpsparsed)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_imgother_t {
    pub iso_speed: f32,
    pub shutter: f32,
    pub aperture: f32,
    pub focal_len: f32,
    pub timestamp: time_t,
    pub shot_order: libc::c_uint,
    pub gpsdata: [libc::c_uint; 32usize],
    pub parsed_gps: libraw_gps_info_t,
    pub desc: [libc::c_char; 512usize],
    pub artist: [libc::c_char; 64usize],
    pub analogbalance: [f32; 4usize],
}
#[test]
fn bindgen_test_layout_libraw_imgother_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_imgother_t>(),
        800usize,
        concat!("Size of: ", stringify!(libraw_imgother_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_imgother_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_imgother_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).iso_speed as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(iso_speed)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).shutter as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(shutter)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).aperture as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(aperture)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).focal_len as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(focal_len)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).timestamp as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(timestamp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).shot_order as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(shot_order)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).gpsdata as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(gpsdata)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).parsed_gps as *const _ as usize },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(parsed_gps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).desc as *const _ as usize },
        204usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(desc)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_imgother_t>())).artist as *const _ as usize },
        716usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(artist)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_imgother_t>())).analogbalance as *const _ as usize
        },
        780usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_imgother_t),
            "::",
            stringify!(analogbalance)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_metadata_common_t {
    pub FlashEC: f32,
    pub FlashGN: f32,
    pub CameraTemperature: f32,
    pub SensorTemperature: f32,
    pub SensorTemperature2: f32,
    pub LensTemperature: f32,
    pub AmbientTemperature: f32,
    pub BatteryTemperature: f32,
    pub exifAmbientTemperature: f32,
    pub exifHumidity: f32,
    pub exifPressure: f32,
    pub exifWaterDepth: f32,
    pub exifAcceleration: f32,
    pub exifCameraElevationAngle: f32,
    pub real_ISO: f32,
    pub exifExposureIndex: f32,
    pub ColorSpace: ushort,
    pub firmware: [libc::c_char; 128usize],
}
#[test]
fn bindgen_test_layout_libraw_metadata_common_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_metadata_common_t>(),
        196usize,
        concat!("Size of: ", stringify!(libraw_metadata_common_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_metadata_common_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_metadata_common_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).FlashEC as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(FlashEC)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).FlashGN as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(FlashGN)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).CameraTemperature as *const _
                as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(CameraTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).SensorTemperature as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(SensorTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).SensorTemperature2 as *const _
                as usize
        },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(SensorTemperature2)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).LensTemperature as *const _
                as usize
        },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(LensTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).AmbientTemperature as *const _
                as usize
        },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(AmbientTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).BatteryTemperature as *const _
                as usize
        },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(BatteryTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifAmbientTemperature as *const _
                as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifAmbientTemperature)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifHumidity as *const _ as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifHumidity)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifPressure as *const _ as usize
        },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifPressure)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifWaterDepth as *const _
                as usize
        },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifWaterDepth)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifAcceleration as *const _
                as usize
        },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifAcceleration)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifCameraElevationAngle
                as *const _ as usize
        },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifCameraElevationAngle)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).real_ISO as *const _ as usize
        },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(real_ISO)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).exifExposureIndex as *const _
                as usize
        },
        60usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(exifExposureIndex)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).ColorSpace as *const _ as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(ColorSpace)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_metadata_common_t>())).firmware as *const _ as usize
        },
        66usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_metadata_common_t),
            "::",
            stringify!(firmware)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_output_params_t {
    pub greybox: [libc::c_uint; 4usize],
    pub cropbox: [libc::c_uint; 4usize],
    pub aber: [f64; 4usize],
    pub gamm: [f64; 6usize],
    pub user_mul: [f32; 4usize],
    pub shot_select: libc::c_uint,
    pub bright: f32,
    pub threshold: f32,
    pub half_size: libc::c_int,
    pub four_color_rgb: libc::c_int,
    pub highlight: libc::c_int,
    pub use_auto_wb: libc::c_int,
    pub use_camera_wb: libc::c_int,
    pub use_camera_matrix: libc::c_int,
    pub output_color: libc::c_int,
    pub output_profile: *mut libc::c_char,
    pub camera_profile: *mut libc::c_char,
    pub bad_pixels: *mut libc::c_char,
    pub dark_frame: *mut libc::c_char,
    pub output_bps: libc::c_int,
    pub output_tiff: libc::c_int,
    pub user_flip: libc::c_int,
    pub user_qual: libc::c_int,
    pub user_black: libc::c_int,
    pub user_cblack: [libc::c_int; 4usize],
    pub user_sat: libc::c_int,
    pub med_passes: libc::c_int,
    pub auto_bright_thr: f32,
    pub adjust_maximum_thr: f32,
    pub no_auto_bright: libc::c_int,
    pub use_fuji_rotate: libc::c_int,
    pub green_matching: libc::c_int,
    pub dcb_iterations: libc::c_int,
    pub dcb_enhance_fl: libc::c_int,
    pub fbdd_noiserd: libc::c_int,
    pub exp_correc: libc::c_int,
    pub exp_shift: f32,
    pub exp_preser: f32,
    pub use_rawspeed: libc::c_int,
    pub use_dngsdk: libc::c_int,
    pub no_auto_scale: libc::c_int,
    pub no_interpolation: libc::c_int,
    pub raw_processing_options: libc::c_uint,
    pub max_raw_memory_mb: libc::c_uint,
    pub sony_arw2_posterization_thr: libc::c_int,
    pub coolscan_nef_gamma: f32,
    pub p4shot_order: [libc::c_char; 5usize],
    pub custom_camera_strings: *mut *mut libc::c_char,
}
#[test]
fn bindgen_test_layout_libraw_output_params_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_output_params_t>(),
        336usize,
        concat!("Size of: ", stringify!(libraw_output_params_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_output_params_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_output_params_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_output_params_t>())).greybox as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(greybox)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_output_params_t>())).cropbox as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(cropbox)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_output_params_t>())).aber as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(aber)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_output_params_t>())).gamm as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(gamm)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_mul as *const _ as usize
        },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_mul)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).shot_select as *const _ as usize
        },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(shot_select)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_output_params_t>())).bright as *const _ as usize },
        132usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(bright)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).threshold as *const _ as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(threshold)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).half_size as *const _ as usize
        },
        140usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(half_size)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).four_color_rgb as *const _ as usize
        },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(four_color_rgb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).highlight as *const _ as usize
        },
        148usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(highlight)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_auto_wb as *const _ as usize
        },
        152usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_auto_wb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_camera_wb as *const _ as usize
        },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_camera_wb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_camera_matrix as *const _
                as usize
        },
        160usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_camera_matrix)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).output_color as *const _ as usize
        },
        164usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(output_color)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).output_profile as *const _ as usize
        },
        168usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(output_profile)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).camera_profile as *const _ as usize
        },
        176usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(camera_profile)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).bad_pixels as *const _ as usize
        },
        184usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(bad_pixels)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).dark_frame as *const _ as usize
        },
        192usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(dark_frame)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).output_bps as *const _ as usize
        },
        200usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(output_bps)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).output_tiff as *const _ as usize
        },
        204usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(output_tiff)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_flip as *const _ as usize
        },
        208usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_flip)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_qual as *const _ as usize
        },
        212usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_qual)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_black as *const _ as usize
        },
        216usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_black)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_cblack as *const _ as usize
        },
        220usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_cblack)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).user_sat as *const _ as usize
        },
        236usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(user_sat)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).med_passes as *const _ as usize
        },
        240usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(med_passes)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).auto_bright_thr as *const _ as usize
        },
        244usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(auto_bright_thr)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).adjust_maximum_thr as *const _
                as usize
        },
        248usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(adjust_maximum_thr)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).no_auto_bright as *const _ as usize
        },
        252usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(no_auto_bright)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_fuji_rotate as *const _ as usize
        },
        256usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_fuji_rotate)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).green_matching as *const _ as usize
        },
        260usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(green_matching)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).dcb_iterations as *const _ as usize
        },
        264usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(dcb_iterations)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).dcb_enhance_fl as *const _ as usize
        },
        268usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(dcb_enhance_fl)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).fbdd_noiserd as *const _ as usize
        },
        272usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(fbdd_noiserd)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).exp_correc as *const _ as usize
        },
        276usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(exp_correc)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).exp_shift as *const _ as usize
        },
        280usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(exp_shift)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).exp_preser as *const _ as usize
        },
        284usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(exp_preser)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_rawspeed as *const _ as usize
        },
        288usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_rawspeed)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).use_dngsdk as *const _ as usize
        },
        292usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(use_dngsdk)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).no_auto_scale as *const _ as usize
        },
        296usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(no_auto_scale)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).no_interpolation as *const _
                as usize
        },
        300usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(no_interpolation)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).raw_processing_options as *const _
                as usize
        },
        304usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(raw_processing_options)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).max_raw_memory_mb as *const _
                as usize
        },
        308usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(max_raw_memory_mb)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).sony_arw2_posterization_thr
                as *const _ as usize
        },
        312usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(sony_arw2_posterization_thr)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).coolscan_nef_gamma as *const _
                as usize
        },
        316usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(coolscan_nef_gamma)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).p4shot_order as *const _ as usize
        },
        320usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(p4shot_order)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_output_params_t>())).custom_camera_strings as *const _
                as usize
        },
        328usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_output_params_t),
            "::",
            stringify!(custom_camera_strings)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_rawdata_t {
    pub raw_alloc: *mut libc::c_void,
    pub raw_image: *mut ushort,
    pub color4_image: *mut [ushort; 4usize],
    pub color3_image: *mut [ushort; 3usize],
    pub float_image: *mut f32,
    pub float3_image: *mut [f32; 3usize],
    pub float4_image: *mut [f32; 4usize],
    pub ph1_cblack: *mut [libc::c_short; 2usize],
    pub ph1_rblack: *mut [libc::c_short; 2usize],
    pub iparams: libraw_iparams_t,
    pub sizes: libraw_image_sizes_t,
    pub ioparams: libraw_internal_output_params_t,
    pub color: libraw_colordata_t,
}
#[test]
fn bindgen_test_layout_libraw_rawdata_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_rawdata_t>(),
        187744usize,
        concat!("Size of: ", stringify!(libraw_rawdata_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_rawdata_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_rawdata_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).raw_alloc as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(raw_alloc)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).raw_image as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(raw_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).color4_image as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(color4_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).color3_image as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(color3_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).float_image as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(float_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).float3_image as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(float3_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).float4_image as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(float4_image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).ph1_cblack as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(ph1_cblack)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).ph1_rblack as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(ph1_rblack)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).iparams as *const _ as usize },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(iparams)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).sizes as *const _ as usize },
        512usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(sizes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).ioparams as *const _ as usize },
        688usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(ioparams)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_rawdata_t>())).color as *const _ as usize },
        704usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_rawdata_t),
            "::",
            stringify!(color)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_makernotes_lens_t {
    pub LensID: libc::c_ulonglong,
    pub Lens: [libc::c_char; 128usize],
    pub LensFormat: ushort,
    pub LensMount: ushort,
    pub CamID: libc::c_ulonglong,
    pub CameraFormat: ushort,
    pub CameraMount: ushort,
    pub body: [libc::c_char; 64usize],
    pub FocalType: libc::c_short,
    pub LensFeatures_pre: [libc::c_char; 16usize],
    pub LensFeatures_suf: [libc::c_char; 16usize],
    pub MinFocal: f32,
    pub MaxFocal: f32,
    pub MaxAp4MinFocal: f32,
    pub MaxAp4MaxFocal: f32,
    pub MinAp4MinFocal: f32,
    pub MinAp4MaxFocal: f32,
    pub MaxAp: f32,
    pub MinAp: f32,
    pub CurFocal: f32,
    pub CurAp: f32,
    pub MaxAp4CurFocal: f32,
    pub MinAp4CurFocal: f32,
    pub MinFocusDistance: f32,
    pub FocusRangeIndex: f32,
    pub LensFStops: f32,
    pub TeleconverterID: libc::c_ulonglong,
    pub Teleconverter: [libc::c_char; 128usize],
    pub AdapterID: libc::c_ulonglong,
    pub Adapter: [libc::c_char; 128usize],
    pub AttachmentID: libc::c_ulonglong,
    pub Attachment: [libc::c_char; 128usize],
    pub FocalUnits: ushort,
    pub FocalLengthIn35mmFormat: f32,
}
#[test]
fn bindgen_test_layout_libraw_makernotes_lens_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_makernotes_lens_t>(),
        736usize,
        concat!("Size of: ", stringify!(libraw_makernotes_lens_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_makernotes_lens_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_makernotes_lens_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensID as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensID)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).Lens as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(Lens)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensFormat as *const _ as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensFormat)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensMount as *const _ as usize
        },
        138usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensMount)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).CamID as *const _ as usize },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(CamID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).CameraFormat as *const _ as usize
        },
        152usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(CameraFormat)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).CameraMount as *const _ as usize
        },
        154usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(CameraMount)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).body as *const _ as usize },
        156usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(body)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).FocalType as *const _ as usize
        },
        220usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(FocalType)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensFeatures_pre as *const _
                as usize
        },
        222usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensFeatures_pre)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensFeatures_suf as *const _
                as usize
        },
        238usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensFeatures_suf)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinFocal as *const _ as usize
        },
        256usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MaxFocal as *const _ as usize
        },
        260usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MaxFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MaxAp4MinFocal as *const _
                as usize
        },
        264usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MaxAp4MinFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MaxAp4MaxFocal as *const _
                as usize
        },
        268usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MaxAp4MaxFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinAp4MinFocal as *const _
                as usize
        },
        272usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinAp4MinFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinAp4MaxFocal as *const _
                as usize
        },
        276usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinAp4MaxFocal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MaxAp as *const _ as usize },
        280usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MaxAp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinAp as *const _ as usize },
        284usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinAp)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).CurFocal as *const _ as usize
        },
        288usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(CurFocal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).CurAp as *const _ as usize },
        292usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(CurAp)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MaxAp4CurFocal as *const _
                as usize
        },
        296usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MaxAp4CurFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinAp4CurFocal as *const _
                as usize
        },
        300usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinAp4CurFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).MinFocusDistance as *const _
                as usize
        },
        304usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(MinFocusDistance)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).FocusRangeIndex as *const _
                as usize
        },
        308usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(FocusRangeIndex)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).LensFStops as *const _ as usize
        },
        312usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(LensFStops)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).TeleconverterID as *const _
                as usize
        },
        320usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(TeleconverterID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).Teleconverter as *const _ as usize
        },
        328usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(Teleconverter)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).AdapterID as *const _ as usize
        },
        456usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(AdapterID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).Adapter as *const _ as usize
        },
        464usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(Adapter)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).AttachmentID as *const _ as usize
        },
        592usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(AttachmentID)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).Attachment as *const _ as usize
        },
        600usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(Attachment)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).FocalUnits as *const _ as usize
        },
        728usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(FocalUnits)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_makernotes_lens_t>())).FocalLengthIn35mmFormat
                as *const _ as usize
        },
        732usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_lens_t),
            "::",
            stringify!(FocalLengthIn35mmFormat)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_nikonlens_t {
    pub EffectiveMaxAp: f32,
    pub LensIDNumber: uchar,
    pub LensFStops: uchar,
    pub MCUVersion: uchar,
    pub LensType: uchar,
}
#[test]
fn bindgen_test_layout_libraw_nikonlens_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_nikonlens_t>(),
        8usize,
        concat!("Size of: ", stringify!(libraw_nikonlens_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_nikonlens_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_nikonlens_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikonlens_t>())).EffectiveMaxAp as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikonlens_t),
            "::",
            stringify!(EffectiveMaxAp)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_nikonlens_t>())).LensIDNumber as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikonlens_t),
            "::",
            stringify!(LensIDNumber)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_nikonlens_t>())).LensFStops as *const _ as usize },
        5usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikonlens_t),
            "::",
            stringify!(LensFStops)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_nikonlens_t>())).MCUVersion as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikonlens_t),
            "::",
            stringify!(MCUVersion)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_nikonlens_t>())).LensType as *const _ as usize },
        7usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_nikonlens_t),
            "::",
            stringify!(LensType)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq)]
pub struct libraw_dnglens_t {
    pub MinFocal: f32,
    pub MaxFocal: f32,
    pub MaxAp4MinFocal: f32,
    pub MaxAp4MaxFocal: f32,
}
#[test]
fn bindgen_test_layout_libraw_dnglens_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_dnglens_t>(),
        16usize,
        concat!("Size of: ", stringify!(libraw_dnglens_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_dnglens_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_dnglens_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dnglens_t>())).MinFocal as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dnglens_t),
            "::",
            stringify!(MinFocal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_dnglens_t>())).MaxFocal as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dnglens_t),
            "::",
            stringify!(MaxFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dnglens_t>())).MaxAp4MinFocal as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dnglens_t),
            "::",
            stringify!(MaxAp4MinFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_dnglens_t>())).MaxAp4MaxFocal as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_dnglens_t),
            "::",
            stringify!(MaxAp4MaxFocal)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_lensinfo_t {
    pub MinFocal: f32,
    pub MaxFocal: f32,
    pub MaxAp4MinFocal: f32,
    pub MaxAp4MaxFocal: f32,
    pub EXIF_MaxAp: f32,
    pub LensMake: [libc::c_char; 128usize],
    pub Lens: [libc::c_char; 128usize],
    pub LensSerial: [libc::c_char; 128usize],
    pub InternalLensSerial: [libc::c_char; 128usize],
    pub FocalLengthIn35mmFormat: ushort,
    pub nikon: libraw_nikonlens_t,
    pub dng: libraw_dnglens_t,
    pub makernotes: libraw_makernotes_lens_t,
}
#[test]
fn bindgen_test_layout_libraw_lensinfo_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_lensinfo_t>(),
        1296usize,
        concat!("Size of: ", stringify!(libraw_lensinfo_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_lensinfo_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_lensinfo_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).MinFocal as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(MinFocal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).MaxFocal as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(MaxFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_lensinfo_t>())).MaxAp4MinFocal as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(MaxAp4MinFocal)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_lensinfo_t>())).MaxAp4MaxFocal as *const _ as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(MaxAp4MaxFocal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).EXIF_MaxAp as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(EXIF_MaxAp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).LensMake as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(LensMake)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).Lens as *const _ as usize },
        148usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(Lens)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).LensSerial as *const _ as usize },
        276usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(LensSerial)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_lensinfo_t>())).InternalLensSerial as *const _ as usize
        },
        404usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(InternalLensSerial)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_lensinfo_t>())).FocalLengthIn35mmFormat as *const _
                as usize
        },
        532usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(FocalLengthIn35mmFormat)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).nikon as *const _ as usize },
        536usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(nikon)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).dng as *const _ as usize },
        544usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(dng)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_lensinfo_t>())).makernotes as *const _ as usize },
        560usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_lensinfo_t),
            "::",
            stringify!(makernotes)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_makernotes_t {
    pub canon: libraw_canon_makernotes_t,
    pub nikon: libraw_nikon_makernotes_t,
    pub hasselblad: libraw_hasselblad_makernotes_t,
    pub fuji: libraw_fuji_info_t,
    pub olympus: libraw_olympus_makernotes_t,
    pub sony: libraw_sony_info_t,
    pub kodak: libraw_kodak_makernotes_t,
    pub panasonic: libraw_panasonic_makernotes_t,
    pub pentax: libraw_pentax_makernotes_t,
    pub phaseone: libraw_p1_makernotes_t,
    pub samsung: libraw_samsung_makernotes_t,
    pub common: libraw_metadata_common_t,
}
#[test]
fn bindgen_test_layout_libraw_makernotes_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_makernotes_t>(),
        2976usize,
        concat!("Size of: ", stringify!(libraw_makernotes_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_makernotes_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_makernotes_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).canon as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(canon)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).nikon as *const _ as usize },
        664usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(nikon)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).hasselblad as *const _ as usize },
        904usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(hasselblad)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).fuji as *const _ as usize },
        1288usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(fuji)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).olympus as *const _ as usize },
        1376usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(olympus)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).sony as *const _ as usize },
        1720usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(sony)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).kodak as *const _ as usize },
        1872usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(kodak)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).panasonic as *const _ as usize },
        2116usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(panasonic)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).pentax as *const _ as usize },
        2172usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(pentax)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).phaseone as *const _ as usize },
        2192usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(phaseone)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).samsung as *const _ as usize },
        2640usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(samsung)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_makernotes_t>())).common as *const _ as usize },
        2776usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_makernotes_t),
            "::",
            stringify!(common)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_shootinginfo_t {
    pub DriveMode: libc::c_short,
    pub FocusMode: libc::c_short,
    pub MeteringMode: libc::c_short,
    pub AFPoint: libc::c_short,
    pub ExposureMode: libc::c_short,
    pub ExposureProgram: libc::c_short,
    pub ImageStabilization: libc::c_short,
    pub BodySerial: [libc::c_char; 64usize],
    pub InternalBodySerial: [libc::c_char; 64usize],
}
#[test]
fn bindgen_test_layout_libraw_shootinginfo_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_shootinginfo_t>(),
        142usize,
        concat!("Size of: ", stringify!(libraw_shootinginfo_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_shootinginfo_t>(),
        2usize,
        concat!("Alignment of ", stringify!(libraw_shootinginfo_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).DriveMode as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(DriveMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).FocusMode as *const _ as usize
        },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(FocusMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).MeteringMode as *const _ as usize
        },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(MeteringMode)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_shootinginfo_t>())).AFPoint as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(AFPoint)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).ExposureMode as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(ExposureMode)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).ExposureProgram as *const _ as usize
        },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(ExposureProgram)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).ImageStabilization as *const _
                as usize
        },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(ImageStabilization)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).BodySerial as *const _ as usize
        },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(BodySerial)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_shootinginfo_t>())).InternalBodySerial as *const _
                as usize
        },
        78usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_shootinginfo_t),
            "::",
            stringify!(InternalBodySerial)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_custom_camera_t {
    pub fsize: libc::c_uint,
    pub rw: ushort,
    pub rh: ushort,
    pub lm: uchar,
    pub tm: uchar,
    pub rm: uchar,
    pub bm: uchar,
    pub lf: ushort,
    pub cf: uchar,
    pub max: uchar,
    pub flags: uchar,
    pub t_make: [libc::c_char; 10usize],
    pub t_model: [libc::c_char; 20usize],
    pub offset: ushort,
}
#[test]
fn bindgen_test_layout_libraw_custom_camera_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_custom_camera_t>(),
        52usize,
        concat!("Size of: ", stringify!(libraw_custom_camera_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_custom_camera_t>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_custom_camera_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).fsize as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(fsize)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).rw as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(rw)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).rh as *const _ as usize },
        6usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(rh)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).lm as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(lm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).tm as *const _ as usize },
        9usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(tm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).rm as *const _ as usize },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(rm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).bm as *const _ as usize },
        11usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(bm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).lf as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(lf)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).cf as *const _ as usize },
        14usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(cf)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).max as *const _ as usize },
        15usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(max)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).flags as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(flags)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).t_make as *const _ as usize },
        17usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(t_make)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).t_model as *const _ as usize },
        27usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(t_model)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_custom_camera_t>())).offset as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_custom_camera_t),
            "::",
            stringify!(offset)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_data_t {
    pub image: *mut [ushort; 4usize],
    pub sizes: libraw_image_sizes_t,
    pub idata: libraw_iparams_t,
    pub lens: libraw_lensinfo_t,
    pub makernotes: libraw_makernotes_t,
    pub shootinginfo: libraw_shootinginfo_t,
    pub params: libraw_output_params_t,
    pub progress_flags: libc::c_uint,
    pub process_warnings: libc::c_uint,
    pub color: libraw_colordata_t,
    pub other: libraw_imgother_t,
    pub thumbnail: libraw_thumbnail_t,
    pub rawdata: libraw_rawdata_t,
    pub parent_class: *mut libc::c_void,
}
#[test]
fn bindgen_test_layout_libraw_data_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_data_t>(),
        381000usize,
        concat!("Size of: ", stringify!(libraw_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_data_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).image as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(image)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).sizes as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(sizes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).idata as *const _ as usize },
        184usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(idata)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).lens as *const _ as usize },
        624usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(lens)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).makernotes as *const _ as usize },
        1920usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(makernotes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).shootinginfo as *const _ as usize },
        4896usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(shootinginfo)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).params as *const _ as usize },
        5040usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(params)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).progress_flags as *const _ as usize },
        5376usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(progress_flags)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).process_warnings as *const _ as usize },
        5380usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(process_warnings)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).color as *const _ as usize },
        5384usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(color)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).other as *const _ as usize },
        192424usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(other)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).thumbnail as *const _ as usize },
        193224usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(thumbnail)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).rawdata as *const _ as usize },
        193248usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(rawdata)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_data_t>())).parent_class as *const _ as usize },
        380992usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_data_t),
            "::",
            stringify!(parent_class)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct fuji_compressed_params {
    pub q_table: *mut i8,
    pub q_point: [libc::c_int; 5usize],
    pub max_bits: libc::c_int,
    pub min_value: libc::c_int,
    pub raw_bits: libc::c_int,
    pub total_values: libc::c_int,
    pub maxDiff: libc::c_int,
    pub line_width: ushort,
}
#[test]
fn bindgen_test_layout_fuji_compressed_params() {
    assert_eq!(
        ::core::mem::size_of::<fuji_compressed_params>(),
        56usize,
        concat!("Size of: ", stringify!(fuji_compressed_params))
    );
    assert_eq!(
        ::core::mem::align_of::<fuji_compressed_params>(),
        8usize,
        concat!("Alignment of ", stringify!(fuji_compressed_params))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<fuji_compressed_params>())).q_table as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(q_table)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<fuji_compressed_params>())).q_point as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(q_point)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<fuji_compressed_params>())).max_bits as *const _ as usize
        },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(max_bits)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<fuji_compressed_params>())).min_value as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(min_value)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<fuji_compressed_params>())).raw_bits as *const _ as usize
        },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(raw_bits)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<fuji_compressed_params>())).total_values as *const _ as usize
        },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(total_values)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<fuji_compressed_params>())).maxDiff as *const _ as usize },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(maxDiff)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<fuji_compressed_params>())).line_width as *const _ as usize
        },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(fuji_compressed_params),
            "::",
            stringify!(line_width)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct internal_data_t {
    pub input: *mut LibRaw_abstract_datastream,
    pub output: *mut FILE,
    pub input_internal: libc::c_int,
    pub meta_data: *mut libc::c_char,
    pub profile_offset: INT64,
    pub toffset: INT64,
    pub pana_black: [libc::c_uint; 4usize],
}
#[test]
fn bindgen_test_layout_internal_data_t() {
    assert_eq!(
        ::core::mem::size_of::<internal_data_t>(),
        64usize,
        concat!("Size of: ", stringify!(internal_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<internal_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(internal_data_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).input as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(input)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).output as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(output)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).input_internal as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(input_internal)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).meta_data as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(meta_data)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).profile_offset as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(profile_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).toffset as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(toffset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<internal_data_t>())).pana_black as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(internal_data_t),
            "::",
            stringify!(pana_black)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct output_data_t {
    pub histogram: *mut [libc::c_int; 8192usize],
    pub oprof: *mut libc::c_uint,
}
#[test]
fn bindgen_test_layout_output_data_t() {
    assert_eq!(
        ::core::mem::size_of::<output_data_t>(),
        16usize,
        concat!("Size of: ", stringify!(output_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<output_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(output_data_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<output_data_t>())).histogram as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(output_data_t),
            "::",
            stringify!(histogram)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<output_data_t>())).oprof as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(output_data_t),
            "::",
            stringify!(oprof)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct identify_data_t {
    pub olympus_exif_cfa: libc::c_uint,
    pub unique_id: libc::c_ulonglong,
    pub OlyID: libc::c_ulonglong,
    pub tiff_nifds: libc::c_uint,
    pub tiff_flip: libc::c_int,
    pub metadata_blocks: libc::c_int,
}
#[test]
fn bindgen_test_layout_identify_data_t() {
    assert_eq!(
        ::core::mem::size_of::<identify_data_t>(),
        40usize,
        concat!("Size of: ", stringify!(identify_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<identify_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(identify_data_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<identify_data_t>())).olympus_exif_cfa as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(olympus_exif_cfa)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<identify_data_t>())).unique_id as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(unique_id)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<identify_data_t>())).OlyID as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(OlyID)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<identify_data_t>())).tiff_nifds as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(tiff_nifds)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<identify_data_t>())).tiff_flip as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(tiff_flip)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<identify_data_t>())).metadata_blocks as *const _ as usize
        },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(identify_data_t),
            "::",
            stringify!(metadata_blocks)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct crx_data_header_t {
    pub version: i32,
    pub f_width: i32,
    pub f_height: i32,
    pub tileWidth: i32,
    pub tileHeight: i32,
    pub nBits: i32,
    pub nPlanes: i32,
    pub cfaLayout: i32,
    pub encType: i32,
    pub imageLevels: i32,
    pub hasTileCols: i32,
    pub hasTileRows: i32,
    pub mdatHdrSize: i32,
    pub MediaSize: u32,
    pub MediaOffset: INT64,
    pub MediaType: u32,
}
#[test]
fn bindgen_test_layout_crx_data_header_t() {
    assert_eq!(
        ::core::mem::size_of::<crx_data_header_t>(),
        72usize,
        concat!("Size of: ", stringify!(crx_data_header_t))
    );
    assert_eq!(
        ::core::mem::align_of::<crx_data_header_t>(),
        8usize,
        concat!("Alignment of ", stringify!(crx_data_header_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).version as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(version)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).f_width as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(f_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).f_height as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(f_height)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).tileWidth as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(tileWidth)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).tileHeight as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(tileHeight)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).nBits as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(nBits)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).nPlanes as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(nPlanes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).cfaLayout as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(cfaLayout)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).encType as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(encType)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).imageLevels as *const _ as usize },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(imageLevels)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).hasTileCols as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(hasTileCols)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).hasTileRows as *const _ as usize },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(hasTileRows)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).mdatHdrSize as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(mdatHdrSize)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).MediaSize as *const _ as usize },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(MediaSize)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).MediaOffset as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(MediaOffset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<crx_data_header_t>())).MediaType as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(crx_data_header_t),
            "::",
            stringify!(MediaType)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct unpacker_data_t {
    pub order: libc::c_short,
    pub sraw_mul: [ushort; 4usize],
    pub cr2_slice: [ushort; 3usize],
    pub kodak_cbpp: libc::c_uint,
    pub strip_offset: INT64,
    pub data_offset: INT64,
    pub meta_offset: INT64,
    pub data_size: libc::c_uint,
    pub meta_length: libc::c_uint,
    pub thumb_misc: libc::c_uint,
    pub fuji_layout: libc::c_uint,
    pub tiff_samples: libc::c_uint,
    pub tiff_bps: libc::c_uint,
    pub tiff_compress: libc::c_uint,
    pub tiff_sampleformat: libc::c_uint,
    pub zero_after_ff: libc::c_uint,
    pub tile_width: libc::c_uint,
    pub tile_length: libc::c_uint,
    pub load_flags: libc::c_uint,
    pub data_error: libc::c_uint,
    pub hasselblad_parser_flag: libc::c_int,
    pub posRAFData: libc::c_longlong,
    pub lenRAFData: libc::c_uint,
    pub fuji_total_lines: libc::c_int,
    pub fuji_total_blocks: libc::c_int,
    pub fuji_block_width: libc::c_int,
    pub fuji_bits: libc::c_int,
    pub fuji_raw_type: libc::c_int,
    pub pana_encoding: libc::c_int,
    pub pana_bpp: libc::c_int,
    pub crx_header: [crx_data_header_t; 16usize],
    pub crx_track_selected: libc::c_int,
    pub CR3_CTMDtag: libc::c_short,
    pub CR3_Version: libc::c_short,
    pub CM_found: libc::c_int,
    pub is_NikonTransfer: libc::c_uint,
    pub is_Sony: libc::c_uint,
    pub is_pana_raw: libc::c_uint,
    pub is_4K_RAFdata: libc::c_uint,
    pub is_PentaxRicohMakernotes: libc::c_uint,
    pub dng_frames: [libc::c_uint; 20usize],
    pub raw_stride: libc::c_ushort,
}
#[test]
fn bindgen_test_layout_unpacker_data_t() {
    assert_eq!(
        ::core::mem::size_of::<unpacker_data_t>(),
        1416usize,
        concat!("Size of: ", stringify!(unpacker_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<unpacker_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(unpacker_data_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).order as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(order)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).sraw_mul as *const _ as usize },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(sraw_mul)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).cr2_slice as *const _ as usize },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(cr2_slice)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).kodak_cbpp as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(kodak_cbpp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).strip_offset as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(strip_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).data_offset as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(data_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).meta_offset as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(meta_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).data_size as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(data_size)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).meta_length as *const _ as usize },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(meta_length)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).thumb_misc as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(thumb_misc)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).fuji_layout as *const _ as usize },
        60usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_layout)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).tiff_samples as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tiff_samples)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).tiff_bps as *const _ as usize },
        68usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tiff_bps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).tiff_compress as *const _ as usize },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tiff_compress)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).tiff_sampleformat as *const _ as usize
        },
        76usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tiff_sampleformat)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).zero_after_ff as *const _ as usize },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(zero_after_ff)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).tile_width as *const _ as usize },
        84usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tile_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).tile_length as *const _ as usize },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(tile_length)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).load_flags as *const _ as usize },
        92usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(load_flags)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).data_error as *const _ as usize },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(data_error)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).hasselblad_parser_flag as *const _ as usize
        },
        100usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(hasselblad_parser_flag)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).posRAFData as *const _ as usize },
        104usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(posRAFData)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).lenRAFData as *const _ as usize },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(lenRAFData)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).fuji_total_lines as *const _ as usize
        },
        116usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_total_lines)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).fuji_total_blocks as *const _ as usize
        },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_total_blocks)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).fuji_block_width as *const _ as usize
        },
        124usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_block_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).fuji_bits as *const _ as usize },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_bits)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).fuji_raw_type as *const _ as usize },
        132usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(fuji_raw_type)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).pana_encoding as *const _ as usize },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(pana_encoding)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).pana_bpp as *const _ as usize },
        140usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(pana_bpp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).crx_header as *const _ as usize },
        144usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(crx_header)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).crx_track_selected as *const _ as usize
        },
        1296usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(crx_track_selected)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).CR3_CTMDtag as *const _ as usize },
        1300usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(CR3_CTMDtag)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).CR3_Version as *const _ as usize },
        1302usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(CR3_Version)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).CM_found as *const _ as usize },
        1304usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(CM_found)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).is_NikonTransfer as *const _ as usize
        },
        1308usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(is_NikonTransfer)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).is_Sony as *const _ as usize },
        1312usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(is_Sony)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).is_pana_raw as *const _ as usize },
        1316usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(is_pana_raw)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).is_4K_RAFdata as *const _ as usize },
        1320usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(is_4K_RAFdata)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<unpacker_data_t>())).is_PentaxRicohMakernotes as *const _
                as usize
        },
        1324usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(is_PentaxRicohMakernotes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).dng_frames as *const _ as usize },
        1328usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(dng_frames)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<unpacker_data_t>())).raw_stride as *const _ as usize },
        1408usize,
        concat!(
            "Offset of field: ",
            stringify!(unpacker_data_t),
            "::",
            stringify!(raw_stride)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct libraw_internal_data_t {
    pub internal_data: internal_data_t,
    pub internal_output_params: libraw_internal_output_params_t,
    pub output_data: output_data_t,
    pub identify_data: identify_data_t,
    pub unpacker_data: unpacker_data_t,
}
#[test]
fn bindgen_test_layout_libraw_internal_data_t() {
    assert_eq!(
        ::core::mem::size_of::<libraw_internal_data_t>(),
        1552usize,
        concat!("Size of: ", stringify!(libraw_internal_data_t))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_internal_data_t>(),
        8usize,
        concat!("Alignment of ", stringify!(libraw_internal_data_t))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_data_t>())).internal_data as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_data_t),
            "::",
            stringify!(internal_data)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_data_t>())).internal_output_params as *const _
                as usize
        },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_data_t),
            "::",
            stringify!(internal_output_params)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_data_t>())).output_data as *const _ as usize
        },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_data_t),
            "::",
            stringify!(output_data)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_data_t>())).identify_data as *const _ as usize
        },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_data_t),
            "::",
            stringify!(identify_data)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_internal_data_t>())).unpacker_data as *const _ as usize
        },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_internal_data_t),
            "::",
            stringify!(unpacker_data)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct decode {
    pub branch: [*mut decode; 2usize],
    pub leaf: libc::c_int,
}
#[test]
fn bindgen_test_layout_decode() {
    assert_eq!(
        ::core::mem::size_of::<decode>(),
        24usize,
        concat!("Size of: ", stringify!(decode))
    );
    assert_eq!(
        ::core::mem::align_of::<decode>(),
        8usize,
        concat!("Alignment of ", stringify!(decode))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<decode>())).branch as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(decode),
            "::",
            stringify!(branch)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<decode>())).leaf as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(decode),
            "::",
            stringify!(leaf)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct tiff_ifd_t {
    pub t_width: libc::c_int,
    pub t_height: libc::c_int,
    pub bps: libc::c_int,
    pub comp: libc::c_int,
    pub phint: libc::c_int,
    pub offset: libc::c_int,
    pub t_flip: libc::c_int,
    pub samples: libc::c_int,
    pub bytes: libc::c_int,
    pub extrasamples: libc::c_int,
    pub t_tile_width: libc::c_int,
    pub t_tile_length: libc::c_int,
    pub sample_format: libc::c_int,
    pub predictor: libc::c_int,
    pub rows_per_strip: libc::c_int,
    pub strip_offsets: *mut libc::c_int,
    pub strip_offsets_count: libc::c_int,
    pub strip_byte_counts: *mut libc::c_int,
    pub strip_byte_counts_count: libc::c_int,
    pub t_filters: libc::c_uint,
    pub t_vwidth: libc::c_int,
    pub t_vheight: libc::c_int,
    pub t_lm: libc::c_int,
    pub t_tm: libc::c_int,
    pub t_fuji_width: libc::c_int,
    pub t_shutter: f32,
    pub opcode2_offset: INT64,
    pub lineartable_offset: INT64,
    pub lineartable_len: libc::c_int,
    pub dng_color: [libraw_dng_color_t; 2usize],
    pub dng_levels: libraw_dng_levels_t,
    pub newsubfiletype: libc::c_int,
}
#[test]
fn bindgen_test_layout_tiff_ifd_t() {
    assert_eq!(
        ::core::mem::size_of::<tiff_ifd_t>(),
        33400usize,
        concat!("Size of: ", stringify!(tiff_ifd_t))
    );
    assert_eq!(
        ::core::mem::align_of::<tiff_ifd_t>(),
        8usize,
        concat!("Alignment of ", stringify!(tiff_ifd_t))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_width as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_height as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_height)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).bps as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(bps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).comp as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(comp)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).phint as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(phint)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).offset as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_flip as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_flip)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).samples as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(samples)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).bytes as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(bytes)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).extrasamples as *const _ as usize },
        36usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(extrasamples)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_tile_width as *const _ as usize },
        40usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_tile_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_tile_length as *const _ as usize },
        44usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_tile_length)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).sample_format as *const _ as usize },
        48usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(sample_format)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).predictor as *const _ as usize },
        52usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(predictor)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).rows_per_strip as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(rows_per_strip)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).strip_offsets as *const _ as usize },
        64usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(strip_offsets)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).strip_offsets_count as *const _ as usize },
        72usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(strip_offsets_count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).strip_byte_counts as *const _ as usize },
        80usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(strip_byte_counts)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<tiff_ifd_t>())).strip_byte_counts_count as *const _ as usize
        },
        88usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(strip_byte_counts_count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_filters as *const _ as usize },
        92usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_filters)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_vwidth as *const _ as usize },
        96usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_vwidth)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_vheight as *const _ as usize },
        100usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_vheight)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_lm as *const _ as usize },
        104usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_lm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_tm as *const _ as usize },
        108usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_tm)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_fuji_width as *const _ as usize },
        112usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_fuji_width)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).t_shutter as *const _ as usize },
        116usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(t_shutter)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).opcode2_offset as *const _ as usize },
        120usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(opcode2_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).lineartable_offset as *const _ as usize },
        128usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(lineartable_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).lineartable_len as *const _ as usize },
        136usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(lineartable_len)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).dng_color as *const _ as usize },
        140usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(dng_color)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).dng_levels as *const _ as usize },
        476usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(dng_levels)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_ifd_t>())).newsubfiletype as *const _ as usize },
        33396usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_ifd_t),
            "::",
            stringify!(newsubfiletype)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct jhead {
    pub algo: libc::c_int,
    pub bits: libc::c_int,
    pub high: libc::c_int,
    pub wide: libc::c_int,
    pub clrs: libc::c_int,
    pub sraw: libc::c_int,
    pub psv: libc::c_int,
    pub restart: libc::c_int,
    pub vpred: [libc::c_int; 6usize],
    pub quant: [ushort; 64usize],
    pub idct: [ushort; 64usize],
    pub huff: [*mut ushort; 20usize],
    pub free: [*mut ushort; 20usize],
    pub row: *mut ushort,
}
#[test]
fn bindgen_test_layout_jhead() {
    assert_eq!(
        ::core::mem::size_of::<jhead>(),
        640usize,
        concat!("Size of: ", stringify!(jhead))
    );
    assert_eq!(
        ::core::mem::align_of::<jhead>(),
        8usize,
        concat!("Alignment of ", stringify!(jhead))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).algo as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(algo)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).bits as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(bits)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).high as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(high)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).wide as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(wide)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).clrs as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(clrs)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).sraw as *const _ as usize },
        20usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(sraw)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).psv as *const _ as usize },
        24usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(psv)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).restart as *const _ as usize },
        28usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(restart)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).vpred as *const _ as usize },
        32usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(vpred)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).quant as *const _ as usize },
        56usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(quant)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).idct as *const _ as usize },
        184usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(idct)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).huff as *const _ as usize },
        312usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(huff)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).free as *const _ as usize },
        472usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(free)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<jhead>())).row as *const _ as usize },
        632usize,
        concat!(
            "Offset of field: ",
            stringify!(jhead),
            "::",
            stringify!(row)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct libraw_tiff_tag {
    pub tag: ushort,
    pub type_: ushort,
    pub count: libc::c_int,
    pub val: libraw_tiff_tag__bindgen_ty_1,
}
#[repr(C)]
#[derive(Copy, Clone)]
pub union libraw_tiff_tag__bindgen_ty_1 {
    pub c: [libc::c_char; 4usize],
    pub s: [libc::c_short; 2usize],
    pub i: libc::c_int,
    _bindgen_union_align: u32,
}
#[test]
fn bindgen_test_layout_libraw_tiff_tag__bindgen_ty_1() {
    assert_eq!(
        ::core::mem::size_of::<libraw_tiff_tag__bindgen_ty_1>(),
        4usize,
        concat!("Size of: ", stringify!(libraw_tiff_tag__bindgen_ty_1))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_tiff_tag__bindgen_ty_1>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_tiff_tag__bindgen_ty_1))
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_tiff_tag__bindgen_ty_1>())).c as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag__bindgen_ty_1),
            "::",
            stringify!(c)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_tiff_tag__bindgen_ty_1>())).s as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag__bindgen_ty_1),
            "::",
            stringify!(s)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<libraw_tiff_tag__bindgen_ty_1>())).i as *const _ as usize
        },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag__bindgen_ty_1),
            "::",
            stringify!(i)
        )
    );
}
#[test]
fn bindgen_test_layout_libraw_tiff_tag() {
    assert_eq!(
        ::core::mem::size_of::<libraw_tiff_tag>(),
        12usize,
        concat!("Size of: ", stringify!(libraw_tiff_tag))
    );
    assert_eq!(
        ::core::mem::align_of::<libraw_tiff_tag>(),
        4usize,
        concat!("Alignment of ", stringify!(libraw_tiff_tag))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_tiff_tag>())).tag as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag),
            "::",
            stringify!(tag)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_tiff_tag>())).type_ as *const _ as usize },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag),
            "::",
            stringify!(type_)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_tiff_tag>())).count as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag),
            "::",
            stringify!(count)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<libraw_tiff_tag>())).val as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(libraw_tiff_tag),
            "::",
            stringify!(val)
        )
    );
}
#[repr(C)]
#[derive(Copy, Clone)]
pub struct tiff_hdr {
    pub t_order: ushort,
    pub magic: ushort,
    pub ifd: libc::c_int,
    pub pad: ushort,
    pub ntag: ushort,
    pub tag: [libraw_tiff_tag; 23usize],
    pub nextifd: libc::c_int,
    pub pad2: ushort,
    pub nexif: ushort,
    pub exif: [libraw_tiff_tag; 4usize],
    pub pad3: ushort,
    pub ngps: ushort,
    pub gpst: [libraw_tiff_tag; 10usize],
    pub bps: [libc::c_short; 4usize],
    pub rat: [libc::c_int; 10usize],
    pub gps: [libc::c_uint; 26usize],
    pub t_desc: [libc::c_char; 512usize],
    pub t_make: [libc::c_char; 64usize],
    pub t_model: [libc::c_char; 64usize],
    pub soft: [libc::c_char; 32usize],
    pub date: [libc::c_char; 20usize],
    pub t_artist: [libc::c_char; 64usize],
}
#[test]
fn bindgen_test_layout_tiff_hdr() {
    assert_eq!(
        ::core::mem::size_of::<tiff_hdr>(),
        1376usize,
        concat!("Size of: ", stringify!(tiff_hdr))
    );
    assert_eq!(
        ::core::mem::align_of::<tiff_hdr>(),
        4usize,
        concat!("Alignment of ", stringify!(tiff_hdr))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).t_order as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(t_order)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).magic as *const _ as usize },
        2usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(magic)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).ifd as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(ifd)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).pad as *const _ as usize },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(pad)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).ntag as *const _ as usize },
        10usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(ntag)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).tag as *const _ as usize },
        12usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(tag)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).nextifd as *const _ as usize },
        288usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(nextifd)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).pad2 as *const _ as usize },
        292usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(pad2)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).nexif as *const _ as usize },
        294usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(nexif)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).exif as *const _ as usize },
        296usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(exif)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).pad3 as *const _ as usize },
        344usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(pad3)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).ngps as *const _ as usize },
        346usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(ngps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).gpst as *const _ as usize },
        348usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(gpst)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).bps as *const _ as usize },
        468usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(bps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).rat as *const _ as usize },
        476usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(rat)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).gps as *const _ as usize },
        516usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(gps)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).t_desc as *const _ as usize },
        620usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(t_desc)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).t_make as *const _ as usize },
        1132usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(t_make)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).t_model as *const _ as usize },
        1196usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(t_model)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).soft as *const _ as usize },
        1260usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(soft)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).date as *const _ as usize },
        1292usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(date)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<tiff_hdr>())).t_artist as *const _ as usize },
        1312usize,
        concat!(
            "Offset of field: ",
            stringify!(tiff_hdr),
            "::",
            stringify!(t_artist)
        )
    );
}
extern "C" {
    pub fn libraw_strerror(errorcode: libc::c_int) -> *const libc::c_char;
}
extern "C" {
    pub fn libraw_strprogress(arg1: LibRaw_progress) -> *const libc::c_char;
}
extern "C" {
    pub fn libraw_init(flags: libc::c_uint) -> *mut libraw_data_t;
}
extern "C" {
    pub fn libraw_open_file(arg1: *mut libraw_data_t, arg2: *const libc::c_char) -> libc::c_int;
}
extern "C" {
    pub fn libraw_open_file_ex(
        arg1: *mut libraw_data_t,
        arg2: *const libc::c_char,
        max_buff_sz: INT64,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_open_buffer(
        arg1: *mut libraw_data_t,
        buffer: *const libc::c_void,
        size: usize,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_unpack(arg1: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_unpack_thumb(arg1: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_recycle_datastream(arg1: *mut libraw_data_t);
}
extern "C" {
    pub fn libraw_recycle(arg1: *mut libraw_data_t);
}
extern "C" {
    pub fn libraw_close(arg1: *mut libraw_data_t);
}
extern "C" {
    pub fn libraw_subtract_black(arg1: *mut libraw_data_t);
}
extern "C" {
    pub fn libraw_raw2image(arg1: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_free_image(arg1: *mut libraw_data_t);
}
extern "C" {
    pub fn libraw_version() -> *const libc::c_char;
}
extern "C" {
    pub fn libraw_versionNumber() -> libc::c_int;
}
extern "C" {
    pub fn libraw_cameraList() -> *mut *const libc::c_char;
}
extern "C" {
    pub fn libraw_cameraCount() -> libc::c_int;
}
extern "C" {
    pub fn libraw_set_memerror_handler(
        arg1: *mut libraw_data_t,
        cb: memory_callback,
        datap: *mut libc::c_void,
    );
}
extern "C" {
    pub fn libraw_set_exifparser_handler(
        arg1: *mut libraw_data_t,
        cb: exif_parser_callback,
        datap: *mut libc::c_void,
    );
}
extern "C" {
    pub fn libraw_set_dataerror_handler(
        arg1: *mut libraw_data_t,
        func: data_callback,
        datap: *mut libc::c_void,
    );
}
extern "C" {
    pub fn libraw_set_progress_handler(
        arg1: *mut libraw_data_t,
        cb: progress_callback,
        datap: *mut libc::c_void,
    );
}
extern "C" {
    pub fn libraw_unpack_function_name(lr: *mut libraw_data_t) -> *const libc::c_char;
}
extern "C" {
    pub fn libraw_get_decoder_info(
        lr: *mut libraw_data_t,
        d: *mut libraw_decoder_info_t,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_COLOR(
        arg1: *mut libraw_data_t,
        row: libc::c_int,
        col: libc::c_int,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_capabilities() -> libc::c_uint;
}
extern "C" {
    pub fn libraw_adjust_sizes_info_only(arg1: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_dcraw_ppm_tiff_writer(
        lr: *mut libraw_data_t,
        filename: *const libc::c_char,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_dcraw_thumb_writer(
        lr: *mut libraw_data_t,
        fname: *const libc::c_char,
    ) -> libc::c_int;
}
extern "C" {
    pub fn libraw_dcraw_process(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_dcraw_make_mem_image(
        lr: *mut libraw_data_t,
        errc: *mut libc::c_int,
    ) -> *mut libraw_processed_image_t;
}
extern "C" {
    pub fn libraw_dcraw_make_mem_thumb(
        lr: *mut libraw_data_t,
        errc: *mut libc::c_int,
    ) -> *mut libraw_processed_image_t;
}
extern "C" {
    pub fn libraw_dcraw_clear_mem(arg1: *mut libraw_processed_image_t);
}
extern "C" {
    pub fn libraw_set_demosaic(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_set_output_color(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_set_user_mul(lr: *mut libraw_data_t, index: libc::c_int, val: f32);
}
extern "C" {
    pub fn libraw_set_output_bps(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_set_gamma(lr: *mut libraw_data_t, index: libc::c_int, value: f32);
}
extern "C" {
    pub fn libraw_set_no_auto_bright(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_set_bright(lr: *mut libraw_data_t, value: f32);
}
extern "C" {
    pub fn libraw_set_highlight(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_set_fbdd_noiserd(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_get_raw_height(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_get_raw_width(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_get_iheight(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_get_iwidth(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_get_cam_mul(lr: *mut libraw_data_t, index: libc::c_int) -> f32;
}
extern "C" {
    pub fn libraw_get_pre_mul(lr: *mut libraw_data_t, index: libc::c_int) -> f32;
}
extern "C" {
    pub fn libraw_get_rgb_cam(
        lr: *mut libraw_data_t,
        index1: libc::c_int,
        index2: libc::c_int,
    ) -> f32;
}
extern "C" {
    pub fn libraw_get_color_maximum(lr: *mut libraw_data_t) -> libc::c_int;
}
extern "C" {
    pub fn libraw_set_output_tif(lr: *mut libraw_data_t, value: libc::c_int);
}
extern "C" {
    pub fn libraw_get_iparams(lr: *mut libraw_data_t) -> *mut libraw_iparams_t;
}
extern "C" {
    pub fn libraw_get_lensinfo(lr: *mut libraw_data_t) -> *mut libraw_lensinfo_t;
}
extern "C" {
    pub fn libraw_get_imgother(lr: *mut libraw_data_t) -> *mut libraw_imgother_t;
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __locale_data {
    pub _address: u8,
}
pub type __builtin_va_list = [__va_list_tag; 1usize];
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct __va_list_tag {
    pub gp_offset: libc::c_uint,
    pub fp_offset: libc::c_uint,
    pub overflow_arg_area: *mut libc::c_void,
    pub reg_save_area: *mut libc::c_void,
}
#[test]
fn bindgen_test_layout___va_list_tag() {
    assert_eq!(
        ::core::mem::size_of::<__va_list_tag>(),
        24usize,
        concat!("Size of: ", stringify!(__va_list_tag))
    );
    assert_eq!(
        ::core::mem::align_of::<__va_list_tag>(),
        8usize,
        concat!("Alignment of ", stringify!(__va_list_tag))
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__va_list_tag>())).gp_offset as *const _ as usize },
        0usize,
        concat!(
            "Offset of field: ",
            stringify!(__va_list_tag),
            "::",
            stringify!(gp_offset)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__va_list_tag>())).fp_offset as *const _ as usize },
        4usize,
        concat!(
            "Offset of field: ",
            stringify!(__va_list_tag),
            "::",
            stringify!(fp_offset)
        )
    );
    assert_eq!(
        unsafe {
            &(*(::core::ptr::null::<__va_list_tag>())).overflow_arg_area as *const _ as usize
        },
        8usize,
        concat!(
            "Offset of field: ",
            stringify!(__va_list_tag),
            "::",
            stringify!(overflow_arg_area)
        )
    );
    assert_eq!(
        unsafe { &(*(::core::ptr::null::<__va_list_tag>())).reg_save_area as *const _ as usize },
        16usize,
        concat!(
            "Offset of field: ",
            stringify!(__va_list_tag),
            "::",
            stringify!(reg_save_area)
        )
    );
}
#[repr(C)]
#[derive(Debug, Copy, Clone, PartialEq, Eq)]
pub struct LibRaw_abstract_datastream {
    pub _address: u8,
}
